/*
 *	Copyright, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */


#import "weightlosssolutions_appDelegate.h"
#import "BT_application.h"


@interface weightlosssolutions_appDelegate : NSObject <UIApplicationDelegate>{

}


@property (nonatomic, strong) UIWindow *window;
@property (nonatomic, strong) BT_reachability *rootNetworkMonitor;
@property (nonatomic, strong) BT_locationManager *rootLocationManager;
@property (nonatomic, strong) BT_application *rootApp;
@property (nonatomic, strong) BT_device *rootDevice;
@property (nonatomic, strong) BT_user *rootUser;
@property (nonatomic, strong) BT_database *rootDatabase;
@property (nonatomic, strong) BT_soundEffectPlayer *rootSoundEffectPlayer;
@property (nonatomic, strong) BT_contextMenu *contextMenu;
@property (nonatomic, strong) NSString *currentMode;
@property (nonatomic, assign) BOOL uiIsVisible;
@property (nonatomic, retain) NSMutableData *receivedData;
@property (nonatomic, assign) BOOL isRefreshing;


//launch methods..
-(BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions;
-(void)applicationDidBecomeActive:(UIApplication *)application;
-(void)applicationWillTerminate:(UIApplication *)application;
-(void)applicationWillResignActive:(UIApplication *)application;
-(void)applicationDidEnterBackground:(UIApplication *)application;
-(void)applicationWillEnterForeground:(UIApplication *)application;

//refresh methods...
-(void)reportToCloud;
-(void)refreshAppData;
-(void)promptForLocationAccessAfterDelay;
-(void)promptforPushNotificationsAfterDelay;


//push notification methods...
-(void)application:(UIApplication*)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData*)deviceToken;
-(void)application:(UIApplication*)application didFailToRegisterForRemoteNotificationsWithError:(NSError*)error;
-(void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo;

//sound effects...
-(void)playSoundFromPushMessage:(NSString *)soundEffectFileName;
-(void)playSoundEffect:(NSString *)theFileName;


//helper methods...
-(BT_navController *)getNavigationController;
-(BT_viewController *)getViewController;
-(void)networkConnectionTypeChanged:(NSNotification*)note;
-(void)showContextMenu;
-(void)hideContextMenu;

//orientation methods...
-(NSUInteger)application:(UIApplication *)application supportedInterfaceOrientationsForWindow:(UIWindow *)window;

@end













