/*
 *	Copyright, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */


#import "weightlosssolutions_appDelegate.h"
#import "BT_loading.h"

@implementation weightlosssolutions_appDelegate


//didFinishLaunchingWithOptions...
-(BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"didFinishLaunchingWithOptions (iOS)%@", @""]];
        
    /*
     Storyboard App
     ---------------------------------------------------------------------------
     This app launches using a Storyboard. The Storyboard Entry Point (iOS) is
     intentionally left blank. If you look at the project settings (in Xcode) you'll
     see there is no value for "Main Interface" entry. Additionally, there is
     not an entry in the BT_info.plist for UIMainStoryboardFile. Again, this is intentional.
     
     
     The app instantiates an instance of the BT_loading class, which uses the BT_loading.storyboard
     file as it's interface. This becomes the launching point for the app. See the BT_loading.m
     file to see how it then instantiates an instance of a BT_application, sets up it's initial
     view controller, then continues to show the app's home screen.
     
     */
    
    //hide status bar on launch...
    [[UIApplication sharedApplication] setStatusBarHidden:TRUE withAnimation:UIStatusBarAnimationNone];
    
    //setup the window on launch...
    [self setWindow:[[UIWindow alloc] initWithFrame:UIScreen.mainScreen.bounds]];
    [self.window setBackgroundColor:[BT_color getColorFromHexString:@"#FFFFFF"]];
    
    //load initial storyboard, "BT_loading"...
    UIStoryboard *launchStoryboard = [UIStoryboard storyboardWithName:@"BT_loading" bundle:nil];
    BT_viewController *launchViewController = [launchStoryboard instantiateViewControllerWithIdentifier:@"BT_loading"];
    
    //set the rootViewController on the window to the BT_loading storyboard...
    self.window.rootViewController = launchViewController;
    [self.window makeKeyAndVisible];
    
    //return...
    return TRUE;
    
}


//when app becomes active again
- (void)applicationDidBecomeActive:(UIApplication *)application{
    
    //make sure we have an app...
    if([self rootApp] == nil){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"applicationDidBecomeActive rootApp not available yet (iOS).%@", @""]];
    }else{
        
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"applicationDidBecomeActive (iOS)%@", @""]];

        //flag as visible...
        [self setUiIsVisible:TRUE];
        
        //report to cloud (not all apps do this, dataURL and reportToCloudURL required)...
        if([[self.rootApp dataURL] length] > 1 && [[self.rootApp reportToCloudURL] length] > 1){
            if(![self isRefreshing]){
                [self reportToCloud];
            }
        }
        
        //if we have a location manager...
        if([self rootLocationManager] != nil){
            [self.rootLocationManager setUpdateCount:0];
            [self.rootLocationManager.locationManager startUpdatingLocation];
        }
        
    }
}

//applicationWillTerminate...
-(void)applicationWillTerminate:(UIApplication *)application{
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"applicationWillTerminate (iOS)%@", @""]];
    
    //set ui as not visible...
    [self setUiIsVisible:FALSE];
    
}

//applicationWillResignActive...
-(void)applicationWillResignActive:(UIApplication *)application{
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"applicationWillResignActive (iOS)%@", @""]];
    
    //set ui as not visible...
    [self setUiIsVisible:FALSE];
    
}

//applicationDidEnterBackground...
-(void)applicationDidEnterBackground:(UIApplication *)application{
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"applicationDidEnterBackground (iOS)%@", @""]];
    
    //set ui as not visible...
    [self setUiIsVisible:FALSE];
    
}

//applicationWillEnterForeground...
-(void)applicationWillEnterForeground:(UIApplication *)application{
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"applicationWillEnterForeground (iOS)%@", @""]];
    
    //set ui as visible...
    [self setUiIsVisible:TRUE];
    
}

//networkConnectionTypeChanged...
-(void)networkConnectionTypeChanged:(NSNotification*)note{
    BT_reachability *reach = [note object];
    if([reach isReachable]){
        [self.rootDevice setDeviceConnectionType:[reach currentReachabilityString]];
        [self.rootDevice setIsOnline:TRUE];
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"networkConnectionTypeChanged. CONNECTED ON: %@", [reach currentReachabilityString]]];
    }else{
        [self.rootDevice setDeviceConnectionType:@"NA"];
        [self.rootDevice setIsOnline:FALSE];
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"networkConnectionTypeChanged%@", @" NOT CONNECT TO NETWORK"]];
    }
}

//reportToCloud...
-(void)reportToCloud{
    
    //must have rootApp...
    if(self.rootApp == nil){
        [BT_debugger showIt:self message:@"reportToCloud: NOT REPORTING. rootApp is nil"];
        return;
    }else{
        [BT_debugger showIt:self message:@"reportToCloud"];

        __block BOOL didReportToCloud = false;
        __block NSString *previousModifiedOnServer = [BT_strings getPrefString:@"lastModifiedOnServer"];
        __block NSString *lastModifiedOnServer = @"";        __block NSString *response = nil;
        dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0ul);
        dispatch_async(queue, ^{
            
            //if we don't have a refresh URL, use the local data...
            if([self.rootApp.reportToCloudURL length] > 1){
                
                //download...
                NSString *useDataURL = [BT_strings mergeBTVariablesInString:[self.rootApp reportToCloudURL]];

                NSURL *escapedURL = [NSURL URLWithString:[useDataURL stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
                [BT_debugger showIt:self message:[NSString stringWithFormat:@"reportToCloud: reporting to: %@", [escapedURL absoluteString]]];
                
                NSError* error;
                response = [NSString stringWithContentsOfURL:escapedURL encoding:NSASCIIStringEncoding error:&error];
                if(error != nil){
                    [BT_debugger showIt:self message:[NSString stringWithFormat:@"reportToCloud (1): ERROR: %@", [error description]]];
                }else{
                    didReportToCloud = true;
                }
                
            }
            
            //move back to main thread...
            dispatch_sync(dispatch_get_main_queue(), ^{
                if(didReportToCloud && [response length] > 1){
                    
                    [BT_debugger showIt:self message:[NSString stringWithFormat:@"reportToCloud: Response:%@", response]];
                    
                    NSError *jsonError = nil;
                    NSData *jsonDataFromString = [response dataUsingEncoding:NSUTF8StringEncoding];
                    NSDictionary *jsonData = [NSJSONSerialization JSONObjectWithData:jsonDataFromString options:0 error:&jsonError];
                    if(jsonError != nil){
                        [BT_debugger showIt:self message:[NSString stringWithFormat:@"reportToCloud (2): ERROR: %@", jsonError.description]];
                    }else{
                        
                        //returned data format: {"lastModifiedUTC":"2011-02-22 02:13:25"}
                        if([jsonData objectForKey:@"lastModifiedUTC"]){
                            if([[jsonData objectForKey:@"lastModifiedUTC"] length] > 3 && [previousModifiedOnServer length] > 3){
                                lastModifiedOnServer = [jsonData objectForKey:@"lastModifiedUTC"];
                                
                                //refresh app data if something changed...
                                if(![lastModifiedOnServer isEqualToString:previousModifiedOnServer]){
                                    [BT_debugger showIt:self message:@"reportToCloud: app data modified on server, refreshing..."];
                                    [self refreshAppData];
                                }else{
                                    [BT_debugger showIt:self message:@"reportToCloud: app data not modified on server, not refreshing"];
                                }
                                
                                //post "appDidReportToCloud" notification...
                                [[NSNotificationCenter defaultCenter] postNotificationName:@"appDidReportToCloud" object:nil];
                                
                            }else{
                                [BT_debugger showIt:self message:@"reportToCloud: lastModifiedOnServer not on device, not refreshing (first launch?)"];
                            }
                            
                            //save previousModifiedOnServer...
                            [BT_strings setPrefString:@"lastModifiedOnServer" valueOfPref:[jsonData objectForKey:@"lastModifiedUTC"]];
                            
                        }
                    }
                }
            });
            
        });
    }
    
}


//refreshAppData...
-(void)refreshAppData{
    [BT_debugger showIt:self message:@"refreshAppData"];
    
    //if not refreshing...
    if(![self isRefreshing]){
        [self setIsRefreshing:true];
        
        //delete data cached by Plugins...
        [BT_fileManager deleteAllLocalData];
        
        //fade out the current view...
        [UIView animateWithDuration:0.5
         animations:^{
             self.window.rootViewController.view.alpha = 1.0;
             self.window.rootViewController.view.alpha = 0.0;
         }
         completion:^(BOOL finished){
             
             //reset the alpha so the loading view shows...
             self.window.rootViewController.view.alpha = 1.0;
             
             //hide status bar (see "prefersStatusBarHidden" in BT_viewController)...
             [[UIApplication sharedApplication] setStatusBarHidden:TRUE withAnimation:UIStatusBarAnimationFade];
             
             //hide nav bar...
             [[self.window.rootViewController navigationController] setNavigationBarHidden:TRUE];
             
             //json data for BT_loadConfigDataViewController...
             BT_item *tmpScreen = [[BT_item alloc] init];
             [tmpScreen setItemId:@"bt_loading"];
             [tmpScreen setItemType:@"bt_loading"];
             
             //create a dictionary for the dynamic screen.
             NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                   @"bt_loading", @"itemId",
                                   @"bt_loading", @"itemType", nil];
             [tmpScreen setJsonVars:dict];
             
             //init the load config data view controller...
             BT_loading *loadingViewController = [[BT_loading alloc] initWithScreenData:tmpScreen];
             [loadingViewController.view setFrame:[[UIScreen mainScreen] bounds]];
             [loadingViewController.view setTag:1];
             
             //set rootViewController...
             [self.window setRootViewController:loadingViewController];
             
             //tell iOS7 the top bar needs updated..
             if(floor(NSFoundationVersionNumber) > NSFoundationVersionNumber_iOS_6_1){
                 [self.window.rootViewController setNeedsStatusBarAppearanceUpdate];
             }
          
         
     }];
    
    }//isRefreshing...
    
}



//promptForLocationAccessAfterDelay...
-(void)promptForLocationAccessAfterDelay{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"promptForLocationAccessAfterDelay%@", @""]];
    
    if((int)[appDelegate.rootApp gpsUpdateIntervalSeconds] > 0 && appDelegate.rootLocationManager != nil){
        if(appDelegate.rootLocationManager != nil){
            [appDelegate.rootLocationManager setShouldTrackLocation:TRUE];
            [appDelegate.rootLocationManager setUpdateRestartSeconds:(int)[appDelegate.rootApp gpsUpdateIntervalSeconds]];
            [appDelegate.rootLocationManager startLocationUpdates];
        }
    }else{
        if(appDelegate.rootLocationManager != nil){
            [appDelegate.rootLocationManager setShouldTrackLocation:FALSE];
        }
    }
    
}

//promptforPushNotificationsAfterDelay...
-(void)promptforPushNotificationsAfterDelay{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"promptforPushNotificationsAfterDelay%@", @""]];
    
    if([[UIApplication sharedApplication] respondsToSelector:@selector(registerUserNotificationSettings:)]){
        [[UIApplication sharedApplication] registerUserNotificationSettings:[UIUserNotificationSettings settingsForTypes:(UIUserNotificationTypeSound | UIUserNotificationTypeAlert | UIUserNotificationTypeBadge) categories:nil]];
        [[UIApplication sharedApplication] registerForRemoteNotifications];
    }
    
}


//showContextMenu...
-(void)showContextMenu{
    [BT_debugger showIt:self message:@"showContextMenu"];
    
    //show the context menu's view...
    if(self.contextMenu != nil){
        
        //current nav controller and view controller...
        BT_navController *theNavController = [self getNavigationController];
        
        //position depends on device orientation
        UIInterfaceOrientation interfaceOrientation = [[UIApplication sharedApplication] statusBarOrientation];
        
        //if the view isn't already on the nav controller..add it. contextMenu view has tag "888"
        BOOL haveContextMenu = FALSE;
        for(UIView *view in theNavController.view.subviews) {
            if([view tag] == 888){
                haveContextMenu = TRUE;
                break;
            }
        }
        //add the subview to this controller if we don't already have it
        if(!haveContextMenu){
            [theNavController.view addSubview:[self.contextMenu view]];
        }
        
        //make mask and menu in context view full size...
        [self.contextMenu.mask setFrame:theNavController.view.bounds];
        [self.contextMenu.menuTable setFrame:theNavController.view.bounds];
        
        //bring context menu's view to the front so we can see it when it fades in..
        [theNavController.view bringSubviewToFront:[self.contextMenu view]];
        
        //position the menu's list to top right...
        int w = [self.rootDevice deviceWidth];
        int h = [self.rootDevice deviceHeight];
        
        //if landscape mode the values are opposite...
        if(UIInterfaceOrientationIsLandscape(interfaceOrientation)){
            w = [self.rootDevice deviceHeight];
            h = [self.rootDevice deviceWidth];
        }else{
            w = [self.rootDevice deviceWidth];
            h = [self.rootDevice deviceHeight];
        }
        
        //table top position...
        int tableTop = 0;
        
        //need status bar and navBar style to determine table top position...
        BT_item *tmpScreenData = [self.rootApp currentScreenData];
        NSString *tmpStatusStyle = [BT_strings getStyleValueForScreen:tmpScreenData nameOfProperty:@"statusBarStyle" defaultValue:@""];
        NSString *tmpNavBarStyle = [BT_strings getStyleValueForScreen:tmpScreenData nameOfProperty:@"navBarStyle" defaultValue:@""];
        if(![tmpStatusStyle isEqualToString:@"hidden"]){
            tableTop += 20;
        }
        if(![tmpNavBarStyle isEqualToString:@"hidden"]){
            
            //iOS 7 uses a different nav. bar height...
            if(floor(NSFoundationVersionNumber) > NSFoundationVersionNumber_iOS_6_1){
                tableTop += 45;
            }else{
                tableTop += 44;
            }
        }
        
        //if device is landscape, remove some pixels...
        if(tableTop > 0){
            if(UIInterfaceOrientationIsLandscape(interfaceOrientation)){
                tableTop -= 12;
                
                //if this is an iPad, add a few more pixels...
                if([self.rootDevice isIPad]){
                    tableTop += 12;
                }
                
            }
        }
        
        //set table size...
        [self.contextMenu.menuTable setFrame:CGRectMake(0, tableTop, (w / 2), 300)];
        
        //move the table to the right of the screen...
        CGRect frame = self.contextMenu.menuTable.frame;
        CGFloat xPosition = w / 2;
        
        //when landscape the w is the height...
        if(UIInterfaceOrientationIsLandscape(interfaceOrientation)){
            //xPosition = h / 2;
        }
        
        frame.origin.x = xPosition;
        self.contextMenu.menuTable.frame = frame;
        
        //animation block...
        [UIView animateWithDuration:0.5
                         animations:^{
                             [self.contextMenu.view setHidden:FALSE];
                             self.contextMenu.view.alpha = 0.0;
                             self.contextMenu.view.alpha = 1.0;
                         }
                         completion:^(BOOL finished){
                             //ignore...
                         }];
        
        
    }
    
    
}

//hideContextMenu...
-(void)hideContextMenu{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"hideContextMenu %@", @""]];
    
    //move the the context menu off the screen and hide it
    if(self.contextMenu != nil){
        
        //animate context menu...
        [UIView animateWithDuration:0.25
                         animations:^{
                             self.contextMenu.view.alpha = 1.0;
                             self.contextMenu.view.alpha = 0.0;
                         }
                         completion:^(BOOL finished){
                             //ignore...
                         }];
        
    }
    
}


//didRegisterForRemoteNotificationsWithDeviceToken...
-(void)application:(UIApplication*)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData*)deviceToken{
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"didRegisterForRemoteNotificationsWithDeviceToken (iOS): Device Token: %@", deviceToken]];
    
    
    /*
     
    	iOS will only prompt for this ONCE per app, even if the app is deleted and re-installed.
    	To re-set the APN prompt, do this:
    	
    	1)	Delete your app from the device.
     2)	Turn the device off completely and turn it back on.
     3)	Go to Settings > General > Date & Time and set the date ahead a day or more.
     4)	Turn the device off completely again and turn it back on.
     
     */
    
    
    //if we have a token, and a  register it...
    if([deviceToken length] > 1 && [[self.rootApp registerForPushURL] length] > 1){
        
        //clean up token...
        NSString *useToken = [NSString stringWithFormat:@"%@", deviceToken];
        useToken = [useToken stringByReplacingOccurrencesOfString:@"<"withString:@""];
        useToken = [useToken stringByReplacingOccurrencesOfString:@">"withString:@""];
        useToken = [useToken stringByReplacingOccurrencesOfString:@" "withString:@""];
        
        //save it for next time...
        [BT_strings setPrefString:@"lastDeviceToken" valueOfPref:useToken];
        
        //append deviceToken and deviceType to end of URL...
        NSString *useURL = [[self.rootApp registerForPushURL] stringByAppendingString:[NSString stringWithFormat:@"&deviceType=%@", @"ios"]];
        useURL = [useURL stringByAppendingString:[NSString stringWithFormat:@"&deviceToken=%@", useToken]];
        
        //append currentMode ("Live" or "Design") to end of URL...
        useURL = [useURL stringByAppendingString:[NSString stringWithFormat:@"&currentMode=%@", [self currentMode]]];
        
        //merge environment variables in URL...
        useURL = [BT_strings mergeBTVariablesInString:useURL];
        
        //escape the URL...
        NSString *escapedUrl = [useURL stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        
        //tell the BT_device to register on the server (the device class makes the URL request)...
        [appDelegate.rootDevice registerForPushNotifications:escapedUrl];
        
    }
    
}


//didFailToRegisterForRemoteNotificationsWithError...
-(void)application:(UIApplication*)application didFailToRegisterForRemoteNotificationsWithError:(NSError*)error{
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"didFailToRegisterForRemoteNotificationsWithError (iOS): ERROR: %@", error]];
    
    /*
    	Simulators ALWAYS fail with this error and will NEVER prompt the user to allow APN.
     
     
    	iOS will only prompt for this ONCE per app, even if the app is deleted and re-installed.
    	To re-set the APN prompt, do this:
    	
    	1)	Delete your app from the device.
        2)	Turn the device off completely and turn it back on.
        3)	Go to Settings > General > Date & Time and set the date ahead a day or more.
        4)	Turn the device off completely again and turn it back on.
     
     */
    
    
}

//didReceiveRemoteNotification..
-(void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo{
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"didReceiveRemoteNotification (iOS)%@", @""]];
    
    //don't do anything if the app is not in the foreground. iOS handles inbound APNS message when app is in the background...
    if(application.applicationState == UIApplicationStateActive){
        
        
        NSString *alertMsg;
        NSString *badge;
        NSString *sound;
        
        //alert message...
        if([[userInfo objectForKey:@"aps"] objectForKey:@"alert"] != NULL){
            alertMsg = [[userInfo objectForKey:@"aps"] objectForKey:@"alert"];
        }
        
        //badge...
        if([[userInfo objectForKey:@"aps"] objectForKey:@"badge"] != NULL){
            badge = [[userInfo objectForKey:@"aps"] objectForKey:@"badge"];
        }
        
        //sound...
        if([[userInfo objectForKey:@"aps"] objectForKey:@"sound"] != NULL){
            sound = [[userInfo objectForKey:@"aps"] objectForKey:@"sound"];
        }
        
        //if we have a sound...
        if([sound length] > 1){
            [self performSelector:@selector(playSoundFromPushMessage:) withObject:sound afterDelay:.1];
        }
        
        //show messsage...
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"" message:alertMsg delegate:nil cancelButtonTitle:NSLocalizedString(@"ok", "OK") otherButtonTitles:nil];
        [alert show];
        
        
    }//in foreground...
}

//playSoundFromPushMessage...
-(void)playSoundFromPushMessage:(NSString *)soundEffectFileName{
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"playSoundFromPushMessage: %@", soundEffectFileName]];
    
    NSString *theFileName = soundEffectFileName;
    if([BT_fileManager doesFileExistInBundle:theFileName]){
        NSURL *soundFileUrl = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/%@", [[NSBundle mainBundle] resourcePath], theFileName]];
        NSError *error;
        AVAudioPlayer *tmpPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:soundFileUrl error:&error];
        if(!error){
            [tmpPlayer setNumberOfLoops:0];
            [tmpPlayer prepareToPlay];
            [tmpPlayer play];
        }else{
            [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"didReceiveRemoteNotification soundEffectPlayer ERROR: %@", [error description]]];
        }
    }
    
}

//playSoundEffect...
-(void)playSoundEffect:(NSString *)theFileName{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"playSoundEffect %@", theFileName]];
    if([theFileName length] > 3){
        
        if([self.rootSoundEffectPlayer.soundEffectNames containsObject:theFileName]){
            int playerIndex = (int)[self.rootSoundEffectPlayer.soundEffectNames indexOfObject:theFileName];
            AVAudioPlayer *tmpPlayer = (AVAudioPlayer *)[self.rootSoundEffectPlayer.soundEffectPlayers objectAtIndex:playerIndex];
            if(tmpPlayer){
                [tmpPlayer play];
            }
        }else{
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"playSoundInBundle:ERROR. This sound effect is not included in the list of available sounds: %@", theFileName]];
        }
    }
    
}



//getNavigationController...
-(BT_navController *)getNavigationController{
    
    BT_navController *theNavController;
    if([self.rootApp rootTabBarController] != nil){
        
        /*
         NOTE:
         rootApp.rootTabBarController is instantiated with an array of NAVIGATION controllers, not
         view controllers. This means the "selectedViewController" property returns an NAV controller,
         not a VIEW controller. Note how we cast this to a BT_navigationController before returning it.
         */
        
        theNavController = (BT_navController *)[self.rootApp.rootTabBarController selectedViewController];
        
    }else{
        theNavController = (BT_navController *)[self.rootApp rootNavController];
    }
    
    //return the nav controller...
    return theNavController;
    
}

//getViewController...
-(BT_viewController *)getViewController{
    
    BT_viewController *theViewController;
    if([self.rootApp rootTabBarController] != nil){
        
        /*
         NOTE:
         rootApp.rootTabBarController is instantiated with an array of NAVIGATION controllers, not
         view controllers. This means we can't reference the tab bars selectedViewController becuase it
         will return a navigation controller (not a view controller). We reference this navigation
         controllers "visibleViewController" property to ge the current view controller.
         */
        
        BT_navController *theNavController = [self getNavigationController];
        theViewController = (BT_viewController *)[theNavController visibleViewController];
        
    }else{
        theViewController = (BT_viewController *)[self.rootApp.rootNavController visibleViewController];
    }
    
    //return the view controller...
    return theViewController;
    
}



//supportedInterfaceOrientations is needed for iOS 6 >
-(NSUInteger)application:(UIApplication *)application supportedInterfaceOrientationsForWindow:(UIWindow *)window{
    //[BT_debugger showIt:self message:[NSString stringWithFormat:@"iOS supportedInterfaceOrientationsForWindow %@", @""]];
    
    /*
     This method only used on iOS 6.0 >. The UIApplicationClass did not
     handle the supportedInterfaceOrientationsForWindow method prior to iOS 6.
     */
    
    
    
    //allow / dissallow rotations, asume we do NOT allow rotations...
    BOOL canRotate = FALSE;
    
    //appDelegate
    if([self.rootDevice isIPad]){
        
        //iPads always support rotations...
        canRotate = TRUE;
        
    }else{
        
        //should we allow rotations on all devices?
        if([[self.rootApp allowRotation] isEqualToString:@"allDevices"]){
            canRotate = TRUE;
        }
        
    }
    
    //bitwise OR operator...
    NSUInteger mask = 0;
    mask |= UIInterfaceOrientationMaskPortrait;
    if(canRotate){
        mask |= UIInterfaceOrientationMaskLandscapeLeft;
        mask |= UIInterfaceOrientationMaskLandscapeRight;
        mask |= UIInterfaceOrientationMaskPortraitUpsideDown;
    }
    
    //return...
    return mask;
    
}




@end










