/*
 *	Copyright, Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */


#import "BT_missing.h"
#import "BT_background_view.h"


@implementation BT_application

//init application...
-(id)init{
    if((self = [super init])){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"INIT%@", @""]];
        
        //init vars...
        self.buzztouchAppId = @"";
        self.buzztouchAPIKey = @"";
        self.themeItemId = @"";
        self.homeScreenItemId = @"";
        self.splashScreenItemId = @"";
        self.registrationScreenItemId = @"";
        self.registrationRequired = @"";
        self.allowRotation = @"";
        self.dataURL = @"";
        self.reportToCloudURL = @"";
        self.registerForPushURL = @"";
        self.gpsUpdateIntervalSeconds = 0;
        self.promptForPushNotifications = @"0";
        self.rootNavController = nil;
        self.rootTabBarController = nil;
        self.rootTheme = nil;
        self.transitionTypeHistory = [[NSMutableArray alloc] init];
        
    }
    return self;
}






//getItemDataByItemId...
-(BT_item *)getItemDataByItemId:(NSString *)theItemId{
    
    //must have itemId...
    if([theItemId length] < 1){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"getItemDataByItemId: ERROR, no itemId provided%@", @""]];
        return nil;
    }
    
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getItemDataByItemId: \"%@\"", theItemId]];
    
    //get BT_item from database...
    BT_item *returnItem = [appDelegate.rootDatabase getBtItemWithId:theItemId];
    
    if(returnItem != nil){
        if([returnItem.itemNickname length] > 1){
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"itemType is %@ for item with nickname: \"%@\" and itemId: %@", [returnItem itemType], [returnItem itemNickname], theItemId]];
        }else{
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"itemType is %@ for item with nickname: \"%@\" and itemId: %@", [returnItem itemType], @"no nickname?", theItemId]];
        }
    }else{
        
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"could not find screen with itemId: %@", theItemId]];
        
        //json data for error screen...
        returnItem = [[BT_item alloc] init];
        [returnItem setItemId:@"n/a"];
        [returnItem setItemType:@"BT_missing"];
        
        //create a dictionary for the dynamic screen.
        NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                     @"n/a", @"itemId",
                                     @"BT_missing", @"itemType",
                                     @"Missing Plugin?", @"navBarTitleText", nil];
        [returnItem setJsonVars:dict];
        
    }
    return returnItem;
}


//getItemDataByNickname...
-(BT_item *)getItemDataByNickname:(NSString *)theItemNickname{
    
    //must have nickname...
    if([theItemNickname length] < 1){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"getItemDataByNickname: ERROR, no nickname provided%@", @""]];
        return nil;
    }
    
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getItemDataByNickname %@", theItemNickname]];
    
    //need itemId from item using nickname...
    NSString *itemId = [appDelegate.rootDatabase getOneValue:[NSString stringWithFormat:@"SELECT guid FROM %@ WHERE nickname = '%@'", bt_itemsTableName, theItemNickname]];
    BT_item *returnScreen = [self getItemDataByItemId:itemId];
    return returnScreen;
    
}



//getBtItemsWithType...
-(NSMutableArray *)getBtItemsWithType:(NSString *)theItemType{
    
    //must have type...
    if([theItemType length] < 1){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"getBtItemsWithType: ERROR, not itemType provided%@", @""]];
        return nil;
    }
    
    //get BT_items from database...
    return [appDelegate.rootDatabase getBtItemsWithQuery:[NSString stringWithFormat:@"SELECT * from %@ WHERE itemTypeLabel = '%@'", bt_itemsTableName, theItemType]];
    
}

//getThemes...
-(NSMutableArray *)getThemes{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getThemes %@", @""]];
    
    //get BT_items from database...
    return [self getBtItemsWithType:@"Theme"];
}

//getTabs...
-(NSMutableArray *)getTabs{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getTabs %@", @""]];
    
    //get BT_items from database...
    return [self getBtItemsWithType:@"Tab"];
    
}

//getMenus...
-(NSMutableArray *)getMenus{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getMenus %@", @""]];
    
    //get BT_items from database...
    return [self getBtItemsWithType:@"Menu"];
    
}

//getScreens...
-(NSMutableArray *)getScreens{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getScreens %@", @""]];
    
    //get BT_items from database...
    return [self getBtItemsWithType:@"Screen"];
    
}

//getFiles...
-(NSMutableArray *)getFiles{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getFiles %@", @""]];
    
    //get BT_items from database...
    return [self getBtItemsWithType:@"File"];
    
}

//getImages...
-(NSMutableArray *)getImages{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getImages %@", @""]];
    
    //get BT_items from database...
    return [self getBtItemsWithType:@"Image"];
    
}


//buildInterface...
-(void)buildInterface{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"buildInterface app interface%@",@""]];
    
    /*
     1) 	Create a nav controller interface or a tabbed navigation interface if we have tabs.
     2) 	Create an optional splash screen, bring it to the front.
     */
    
    //init the rootBackgroundView...
    self.rootBackgroundView = [[BT_background_view alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.rootBackgroundView.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight);
    
    //get tabs and screens...
    NSMutableArray *tabs = [self getTabs];
    
    NSLog(@"###### TABS COUNT: %lu", (unsigned long)[tabs count] );
    
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //	1A) If this app does NOT have any tabs in it's tabs array, setup view controller for the home screen
    if([tabs count] < 1){
        
        //must have homeScreenItemId...
        if((int)[self.homeScreenItemId length] < 0){
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"buildInterface: ERROR, no homeScreenItemId found in app config data, cannot create navigation controller.%@", @""]];
        }else{
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"buildInterface: building a single navigation controller with homeScreenItemId: \"%@\"", @""]];
        }
        
        //screen data for the home screen...
        BT_item *homeScreen = [self getItemDataByItemId:self.homeScreenItemId];
        [homeScreen setIsHomeScreen:TRUE];
        
        //remember this screen as the "currently loaded screen", also make it the "previously loaded screen"
        [self setCurrentScreenData:homeScreen];
        [self setPreviousScreenData:homeScreen];
        
        //initialize a view controller for the appropriate screen type for the home screen
        BT_viewController *useViewController = (BT_viewController *)[self getViewControllerForScreen:homeScreen];
        
        //maintain full size on rotation...
        [useViewController.view setAutoresizingMask:(UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight)];
        
        //initizlize the navigation controller
        self.rootNavController = [[BT_navController alloc] initWithRootViewController:useViewController];
        
        //add the background view to the nav controller's view...
        [self.rootNavController.view addSubview:self.rootBackgroundView];
        [self.rootNavController.view sendSubviewToBack:self.rootBackgroundView];
        
        
    }
    
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //	1B) If this app uses tabs, create a tabbed navigation application and set the default screen for each tab
    if([tabs count] > 0){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"building a tabbed based navigation app%@", @""]];
        
        //initialize the tab bar controller
        self.rootTabBarController = [[BT_tabBarController alloc] init];
        
        //add the background view to the tab controller's view...
        [self.rootTabBarController.view addSubview:self.rootBackgroundView];
        [self.rootTabBarController.view sendSubviewToBack:self.rootBackgroundView];
        
        //fill a temporary array of view controllers to assign to tab bar controller...
        NSMutableArray *tmpViewControllers = [[NSMutableArray alloc] init];
        
        //loop through each tab bar item in application data
        for(int i = 0; i < [tabs count]; i++){
            
            //this tab
            BT_item *thisTab = (BT_item *)[tabs objectAtIndex:i];
            NSString *textLabel = [[thisTab jsonVars] objectForKey:@"textLabel"];
            UIImage *tabIcon = [UIImage imageNamed:[[thisTab jsonVars] objectForKey:@"iconName"]];
            
            //get the screen from the apps array of screens for this tab's view controller
            if([[thisTab jsonVars] objectForKey:@"homeScreenItemId"]){
                
                BT_item *thisTabsDefaultScreenData = [self getItemDataByItemId:[[thisTab jsonVars] objectForKey:@"homeScreenItemId"]];
                
                //if this is the first tab in the list, remember it as the "currently loaded screen", also make it the "previously loaded screen"
                if(i == 0){
                    [self setCurrentScreenData:thisTabsDefaultScreenData];
                    [self setPreviousScreenData:thisTabsDefaultScreenData];
                    [thisTabsDefaultScreenData setIsHomeScreen:TRUE];
                }
                
                //initialize a view controller for this type of screen (ClassName == BT_item.itemType)
                BT_viewController *thisTabsDefaultViewController = (BT_viewController *)[self getViewControllerForScreen:thisTabsDefaultScreenData];
                
                //initialize a navigation controller using the view controller
                BT_navController *thisTabsNavController = [[BT_navController alloc] initWithRootViewController:thisTabsDefaultViewController];
                [thisTabsNavController.tabBarItem setTitle:textLabel];
                [thisTabsNavController.tabBarItem setImage:tabIcon];
                
                //add this navigation controller to the temporary array of view controllers
                [tmpViewControllers addObject:thisTabsNavController];
                
            }else{
                [BT_debugger showIt:self message:[NSString stringWithFormat:@"ERROR: This tab does not have a homeScreenItemId in it's configuration data%@", @""]];
            }
            
        }//end for each tab in this apps list array of tabs
        
        //customize the tab bar color...
        if(self.rootTheme != nil){
            NSString *tabBarColor = [BT_strings getStyleValueForScreen:self.rootTheme nameOfProperty:@"tabBarColor" defaultValue:@""];
            if([tabBarColor length] > 1){
                UIColor *tmpColor = [BT_color getColorFromHexString:tabBarColor];
                [[self.rootTabBarController tabBar] setBackgroundColor:tmpColor];
            }
            
        }
        
        //assign temporary array of view controllers to tab bar controller
        self.rootTabBarController.viewControllers = tmpViewControllers;
        
        //if we have tabs..(something seriously wrong if we don't!)
        if([tmpViewControllers count] > 0){
            
            //select first tab
            [self.rootTabBarController setSelectedIndex:0];
            
            //fire the viewWillAppear method in the selected tab
            [self.rootTabBarController.navigationController viewWillAppear:NO];
        }
        
        
    }//end tabBarController
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //	2) 	Add a splash screen if one is configured. This screen will remove itself after the
    if([self.splashScreenItemId length] > 0){
        
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"initialzing a splash screen with itemId: %@", [self splashScreenItemId]]];
        
        //get the BT_item for the splash screen...
        BT_item *splashScreenData = [self getItemDataByItemId:[self splashScreenItemId]];
        
        //initialize a view controller with ClassName == itemType
        UIViewController *splashViewController = [self getViewControllerForScreen:splashScreenData];
        [splashViewController.view setFrame:[[UIScreen mainScreen] bounds]];
        
        //add splash screen to appropriate navigation controller (see step 1)
        if(tabs.count > 0 ){
            [self.rootTabBarController.view addSubview:[splashViewController view]];
            [self.rootTabBarController.view bringSubviewToFront:[splashViewController view]];
        }else{
            [self.rootNavController.view addSubview:[splashViewController view]];
            [self.rootNavController.view bringSubviewToFront:[splashViewController view]];
        }
        
    }else{
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"This app does not use a splash screen%@", @""]];
    }
    
}


//getViewControllerForScreen...
-(BT_viewController *)getViewControllerForScreen:(BT_item *)theScreenData{
    if([theScreenData itemId] == nil){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"getViewControllerForScreen: ERROR no itemId in screenData%@", @""]];
        return nil;
    }else{
        if([theScreenData.itemNickname length] > 1){
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"getViewControllerForScreen nickname: \"%@\" itemId: %@ type: %@", [theScreenData itemNickname], [theScreenData itemId], [theScreenData itemType]]];
        }else{
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"getViewControllerForScreen nickname: \"%@\" itemId: %@ type: %@", @"no nickname?", [theScreenData itemId], [theScreenData itemType]]];
        }
    }
    
    /*
     Instantiate a view controller with ClassName == BT_item.itemType. If the itemType is a custom plug-in we need to get
     the type of view controller to allocate from the JSON data...
     */
    
    //return this view controller.
    BT_viewController *theViewController = nil;
    NSString *theClassName = [theScreenData itemType];
    
    //are we loading a custom plugin?
    if([[theScreenData itemType] isEqualToString:@"BT_screen_plugIn"]){
        
        //get the class name of the custom UIViewController we want to load...
        theClassName = [BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"classFileName" defaultValue:@""];
        
    }
    
    //theClassName required...
    if([theClassName length] > 0){
        
        
        //first try to load a storyboard, if it exists...
        UIStoryboard *theStoryboard = nil;
        @try{
            theStoryboard = [UIStoryboard storyboardWithName:theClassName bundle:nil];
        }@catch (NSException *theException){
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"getViewControllerForScreen: NO STORYBOARD for \"%@\", looking for .XIB instead", theClassName]];
        }
        
        //instantiate the UIViewController in this storyboard if we found one...
        if(theStoryboard != nil){
            
            theViewController = [theStoryboard instantiateViewControllerWithIdentifier:theClassName];
            [theViewController setScreenData:theScreenData];
            
        }else{
            
            //storyboard did not exist, load the UIViewController (it will load a .xib if it exists)...
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
            
            //instantiate the UIViewController from the class name, it will unarchive a matching .xib file if included in the bundle...
            Class theClass = NSClassFromString(theClassName);
            if(theClass != nil){
                SEL selector = NSSelectorFromString(@"alloc");
                if([theClass respondsToSelector:selector]){
                    theViewController = [[theClass performSelector:selector] initWithScreenData:theScreenData];
                    return theViewController;
                }
            }
#pragma clang diagnostic pop
            
        }
        
    }//theClassName length
    
    if(theViewController == nil){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"getViewControllerForScreen: ERROR, could not initialize view controller for screen with itemId: %@", [theScreenData itemId]]];
        
        //json data for error screen...
        BT_item *errorData = [[BT_item alloc] init];
        [errorData setItemId:@"n/a"];
        [errorData setItemType:@"BT_missing"];
        
        //create a dictionary for the dynamic screen.
        NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                     @"n/a", @"itemId",
                                     @"BT_missing", @"itemType",
                                     @"Missing plugin?", @"navBarTitleText", nil];
        [errorData setJsonVars:dict];
        
        //load the BT_missing view controller...
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
        Class theClass = NSClassFromString(@"BT_missing");
        SEL selector = NSSelectorFromString(@"alloc");
        if([theClass respondsToSelector:selector]){
            theViewController = [[theClass performSelector:selector] initWithScreenData:theScreenData];
        }
#pragma clang diagnostic pop
        
    }
    
    //should not be here.
    return theViewController;
    
}


/////////////////////////////////////////////////////////////////////////////////
//DEPRECATED METHODS, SHOULD NOT BE USED ANYMORE, mapped to newer methods....

//getScreenDataByItemId DEPRECATED...
-(BT_item *)getScreenDataByItemId:(NSString *)theItemId{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getScreenDataByItemId: DEPRECATED %@", theItemId]];
    
    //use new method...
    BT_item *returnItem = [self getItemDataByItemId:theItemId];
    return returnItem;
    
}

//getScreenDataByNickname DEPRECATED...
-(BT_item *)getScreenDataByNickname:(NSString *)theScreenNickname{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getScreenDataByNickname: DEPRECATED %@", theScreenNickname]];
    
    //use new method...
    BT_item *returnItem = [self getItemDataByNickname:theScreenNickname];
    return returnItem;
    
}

//getThemeDataByItemId DEPRECATED...
-(BT_item *)getThemeDataByItemId:(NSString *)theThemeItemId{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getThemeDataByItemId: DEPRECATED %@", theThemeItemId]];
    
    //use new method...
    BT_item *returnItem = [self getItemDataByItemId:theThemeItemId];
    return returnItem;
    
}

//getMenuDataByItemId DEPRECATED...
-(BT_item *)getMenuDataByItemId:(NSString *)theMenuItemId{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"getMenuDataByItemId: DEPRECATED %@", theMenuItemId]];
    
    //use new method...
    BT_item *returnItem = [self getItemDataByItemId:theMenuItemId];
    return returnItem;
    
}


@end











