/*
 *	Copyright, Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */





#import <sqlite3.h>
#import "BT_item.h"

@interface BT_database : NSObject{
    sqlite3 *sqlite3Database;
}

@property (nonatomic, strong) NSString *documentsDirectory;
@property (nonatomic, strong) NSString *databaseFilename;


-(instancetype)initWithDatabaseFilename:(NSString *)dbFilename;
-(void)copyDatabaseFromBundle;
-(void)createDefaultTables;
-(void)openDatabase;
-(void)closeDatabase;
-(NSArray*)getTableList;
-(NSArray*)getColumnInfoForTable:(NSString *)tableName;
-(NSArray*)getIndexInfoForTable:(NSString *)tableName;
-(BOOL)createTableWithName:(NSString *)tableName withColumnInfo:(NSArray *)columnInfo;
-(BOOL)createIndexInTable:(NSString *)tableName onColumn:(NSString *)columnName;
-(void)insertRowIntoTable:(NSString *)tableName withRow:(NSDictionary *)row;
-(void)insertRowsIntoTable:(NSString *)tableName withRows:(NSArray *)rows;
-(void)updateRowInTable:(NSString *)tableName withRow:(NSDictionary *)row;
-(void)updateRowsInTable:(NSString *)tableName withRows:(NSArray *)rows;
-(void)deleteRowFromTable:(NSString *)tableName withGuid:(NSString *)guid;
-(void)deleteRowsFromTable:(NSString *)tableName withGuids:(NSArray *)guids;
-(void)executeQuery:(NSString *)sqlQuery;
-(id)getOneValue:(NSString *)query;
-(NSData *)getOneBlob:(NSString *)query;
-(NSMutableArray *)getSimpleStringListWithQuery:(NSString *)query;
-(NSDictionary *)getItemWithQuery:(NSString *)query;
-(NSMutableArray *)getItemsWithQuery:(NSString *)query;
-(BT_item *)getBtItemWithId:(NSString *)theId;
-(NSMutableArray *)getBtItemsWithQuery:(NSString *)query;
-(void)updateConfigDataInDatabase:(NSString *)appDataJSONString;

@end


















