/*
 *	Copyright, Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */


#import "BT_dates.h"


@implementation BT_dates




//getLocalDateTime...
+(NSString *)getLocalDateTime{
    
    NSDate* localDate = [NSDate date];
    NSDateFormatter* dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    NSString *dateString = [dateFormatter stringFromDate:localDate];
    return dateString;
    
}


//getCurrentTime...
+(NSString *)getCurrentTime{
    
    NSDate *nowUTC = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setTimeZone:[NSTimeZone localTimeZone]];
    [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
    [dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
    return [dateFormatter stringFromDate:nowUTC];
    
}

//returns the amount of "interval" between "now" and the date passed
+(int)getTimeElapsed:(NSString *)previousDateString interval:(NSString *)theInterval{
    
    NSDate *now = [NSDate date];
    NSDateFormatter* dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    NSDate *pastDate = [dateFormatter dateFromString:previousDateString];
    
    NSCalendar *gregorian = [[NSCalendar alloc] initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
    unsigned int unitFlags = NSCalendarUnitYear | NSCalendarUnitMonth | NSCalendarUnitDay | NSCalendarUnitHour | NSCalendarUnitMinute | NSCalendarUnitSecond;
    NSDateComponents *dateParts = [gregorian components:unitFlags fromDate:pastDate toDate:now options:0];
    
    //return the appropriate part
    if([theInterval isEqualToString:@"month"]) return (int)[dateParts month];
    if([theInterval isEqualToString:@"day"]) return (int)[dateParts day];
    if([theInterval isEqualToString:@"hour"]) return (int)[dateParts hour];
    if([theInterval isEqualToString:@"second"]) return (int)[dateParts second];
    
    return 0;
    
}

//formatApproxTime...
+(NSString *)formatApproxTime:(NSDate *)dateTime{
    
    NSInteger MinInterval;
    NSInteger HourInterval;
    NSInteger DayInterval;
    NSInteger DayModules;
    
    NSDate *nowDate = [NSDate date];
    NSInteger interval = abs((int)[nowDate timeIntervalSinceDate:dateTime]);
    NSString *plural = @"s";
    
    if(interval >= 86400){
        DayInterval  = interval / 86400;
        DayModules = interval % 86400;
        if(DayModules!=0){
            if(DayModules >= 3600){
                
                //HourInterval = DayModules/3600;
                if(DayInterval < 2) plural = @"";
                return [NSString stringWithFormat:@"%i day%@ ago", (int)DayInterval, plural];
                
            }else{
                if(DayModules >= 60){
                    
                    //MinInterval = DayModules/60;
                    if(DayInterval < 2) plural = @"";
                    return [NSString stringWithFormat:@"%i day%@ ago", (int)DayInterval, plural];
                    
                }else{
                    
                    if(DayInterval < 2) plural = @"";
                    return [NSString stringWithFormat:@"%i day%@ ago", (int)DayInterval, plural];
                    
                }
            }
        }else{
            
            if(DayInterval < 2) plural = @"";
            return [NSString stringWithFormat:@"%i day%@ ago", (int)DayInterval, plural];
            
        }
        
    }else{
        
        if(interval >= 3600){
            
            HourInterval = interval / 3600;
            if(HourInterval < 2) plural = @"";
            return [NSString stringWithFormat:@"%i hour%@ ago", (int)HourInterval, plural];
            
        }else if(interval >= 60){
            
            MinInterval = interval / 60;
            if(MinInterval < 2) plural = @"";
            return [NSString stringWithFormat:@"%i minute%@ ago", (int)MinInterval, plural];
            
        }else{
            if(interval < 2) plural = @"";
            return [NSString stringWithFormat:@"%i second%@ ago", (int)interval, plural];
            
        }
        
    }
    
}


//formatUTCUnixToLocal...
+(NSString *)formatUTCUnixToLocal:(NSString *)unixUTCTimeStamp{
    
    NSTimeInterval interval = [unixUTCTimeStamp doubleValue];
    NSDate *date = [NSDate dateWithTimeIntervalSince1970:interval];
    
    NSDateFormatter* dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    return [dateFormatter stringFromDate:date];
    
}

//formatMySqlNowUTC...
+(NSString *)formatMySqlNowUTC:(NSString *)dateStringUTC{
    
    NSDateFormatter *objFormatter = [[NSDateFormatter alloc] init];
    [objFormatter setTimeZone:[NSTimeZone systemTimeZone]];
    [objFormatter setDateFormat:@"yyyy-MM-dd hh:mm:ss"];
    NSDate *date = [objFormatter dateFromString:dateStringUTC];
    [objFormatter setDateFormat:@"yyyy-MM-dd hh:mm:ss a"];
    return [objFormatter stringFromDate:date];
    
}


//getMySqlNowUTC...
+(NSString *)getMySqlNowUTC{
    
    NSDate *dateNow = [NSDate date];
    NSDateFormatter *objFormatter = [[NSDateFormatter alloc] init];
    [objFormatter setTimeZone:[NSTimeZone timeZoneWithAbbreviation:@"UTC"]];
    [objFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    return [objFormatter stringFromDate:dateNow];
    
}

//getUnixNowUTC...
+(NSString *)getUnixNowUTC{
    
    NSDate *dateNow = [NSDate date];
    NSDateFormatter *objFormatter = [[NSDateFormatter alloc] init];
    [objFormatter setTimeZone:[NSTimeZone timeZoneWithAbbreviation:@"UTC"]];
    [objFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    NSString *strUTCTime = [objFormatter stringFromDate:dateNow];
    NSDate *objUTCDate  = [objFormatter dateFromString:strUTCTime];
    int seconds = [objUTCDate timeIntervalSince1970];
    NSString *UNIXTimeStamp = [NSString stringWithFormat:@"%d", seconds];
    return UNIXTimeStamp;
    
}




@end







