/*
 *	Copyright, Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */


#import "BT_locationManager.h"

@implementation BT_locationManager


-(id)init{
    self = [super init];
    if (self != nil){
		[BT_debugger showIt:self message:[NSString stringWithFormat:@"INIT%@", @""]];

		//vars...
        [self setShouldTrackLocation:FALSE];
        [self setUpdateCount:0];
        [self setUpdateRestartSeconds:60];
		
    }
    return self;
}

//start location updates
-(void)startLocationUpdates{
	[BT_debugger showIt:self message:[NSString stringWithFormat:@"startLocationUpdates %@", @""]];
	
    //reset update count...
    [self setUpdateCount:0];
    
	@try{
	
		//if we have not already done this..
		if(self.locationManager == nil && self.shouldTrackLocation){

            //init location manager
        	self.locationManager = [[CLLocationManager alloc] init];
			[self.locationManager setDelegate:self];
			[self.locationManager setDesiredAccuracy:kCLLocationAccuracyBest];
		
        }
        
        //start location updates...
        if(self.locationManager != nil && self.shouldTrackLocation){
            [self.locationManager startUpdatingLocation];
        }
		
        //start heading updates...
        if([CLLocationManager headingAvailable] && self.shouldTrackLocation){
            self.locationManager.headingFilter = 5;
            [self.locationManager startUpdatingHeading];
        }

	}@catch (NSException * e){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"startLocationUpdates EXCEPTION: %@", [e description]]];
	}@finally{
 	}
	
}

//stop location udpates
-(void)stopLocationUpdates{
	[BT_debugger showIt:self message:[NSString stringWithFormat:@"stopLocationUpdates %@", @""]];
	[self.locationManager stopUpdatingLocation];
}


//didUpdateToLocation...
-(void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"didUpdateToLocation: restarts every %d seconds", [self updateRestartSeconds]]];
	
    //for log...
    NSString *altitudeInfo = @"";
    NSString *distanceMovedInfo = @"";
    NSString *speedInfo = @"";
    NSString *timeSinceLastInfo = @"";
    
    
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	[dateFormatter setDateStyle:NSDateFormatterMediumStyle];
	[dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
	
    //speed of the device...
    [appDelegate.rootDevice setDeviceSpeed:(int)newLocation.speed];
    
	//horizontal coordinates
	if(signbit(newLocation.horizontalAccuracy)) {
	
		//negative accuracy means an invalid or unavailable measurement
	
	}else{
	
		//coreLocation returns positive for North & East, negative for South & West
        NSString *latLon = @"";
        latLon = [latLon stringByAppendingString:[NSString stringWithFormat:@"%f %@ ", newLocation.coordinate.latitude, signbit(newLocation.coordinate.latitude) ? @"South" : @"North"]];
        latLon = [latLon stringByAppendingString:[NSString stringWithFormat:@"%f %@ ", newLocation.coordinate.longitude, signbit(newLocation.coordinate.longitude) ? @"West" : @"East"]];
        NSLog(@"%@", latLon);
        
    }
	
	//altitude
	if (signbit(newLocation.verticalAccuracy)) {
	
        //negative accuracy means an invalid or unavailable measurement
        altitudeInfo = @"Altitude not available";
        
    } else {
	
        //positive and negative in altitude denote above & below sea level, respectively
        //[update appendFormat:LocStr(@"AltitudeFormat"), fabs(newLocation.altitude),	(signbit(newLocation.altitude)) ? LocStr(@"BelowSeaLevel") : LocStr(@"AboveSeaLevel")];

         
        altitudeInfo = [NSString stringWithFormat:@"Altitude: %f %@", newLocation.altitude, (signbit(newLocation.altitude)) ? @"BSL" : @"ASL"];

         
    }
	
    
    speedInfo = [NSString stringWithFormat:@"SPEED: %f", [newLocation speed]];

    
	//calculate disatance moved and time elapsed, but only if we have an "old" location
	if(oldLocation != nil) {
	
        CLLocationDistance distanceMoved = [newLocation distanceFromLocation:oldLocation];
		NSTimeInterval timeElapsed = [newLocation.timestamp timeIntervalSinceDate:oldLocation.timestamp];
        
        
        distanceMovedInfo = [NSString stringWithFormat:@"Distance Moved: %f", distanceMoved];

        timeSinceLastInfo = [NSString stringWithFormat:@"Time since last report: %f (seconds)", timeElapsed];
        
        //[update appendFormat:LocStr(@"LocationChangedFormat"), distanceMoved];
		if (signbit(timeElapsed)){
            
            
			//[update appendString:LocStr(@"FromPreviousMeasurement")];
		}else{
			//[update appendFormat:LocStr(@"TimeElapsedFormat"), timeElapsed];

		}
	}

    /*
    NSLog(@"############ Distance Moved: %@ (meters)", distanceMovedInfo);
    NSLog(@"############ Time Since: %@", timeSinceLastInfo);
    NSLog(@"############ Altitude: %@", altitudeInfo);
    NSLog(@"############ Speed MPS: %f (meters per second)", [newLocation speed]);
    NSLog(@"############ Speed KPH: %f (Kilometers per hour)", ([newLocation speed] * 3.6));
    NSLog(@"############ Speed MPH: %f (Miles per hour)", ([newLocation speed] * 2.23693629));
    NSLog(@"############ Desc: %@", [newLocation description]);
    NSLog(@"############ Device: %@", [appDelegate.rootDevice deviceLastLocationReport]);
    */
    
    
	//appDelegate 
	[appDelegate.rootDevice setDeviceLastLatitude:[NSString stringWithFormat:@"%f", newLocation.coordinate.latitude]];
	[appDelegate.rootDevice setDeviceLastLongitude:[NSString stringWithFormat:@"%f", newLocation.coordinate.longitude]];
    [appDelegate.rootDevice setDeviceLastLocationReport:[BT_dates getLocalDateTime]];
    [appDelegate.rootDevice setCanReportLocation:TRUE];
	
	//turn off updates after 10 reports. This ensures accuracy but reduces battery load.
	self.updateCount = (self.updateCount + 1);
	if(self.updateCount > 5){
        
        //reset counter...
        [self setUpdateCount:0];
        
        //stop updating...
		[self.locationManager stopUpdatingLocation];
        
        //invalidate timer if we have one...
        if(self.updateTimer == nil){
            [self.updateTimer invalidate];
        }
        
        //set timer to re-do this after x seconds...
        if([self updateRestartSeconds] > 0){
            [self setUpdateTimer:[NSTimer scheduledTimerWithTimeInterval:[self updateRestartSeconds] target:self selector:@selector(startLocationUpdates) userInfo:NULL repeats:NO]];
        }
        
	}
	
}



//didUpdateHeading...
- (void)locationManager:(CLLocationManager *)manager didUpdateHeading:(CLHeading *)newHeading{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"didUpdateHeading: %f", [newHeading trueHeading]]];

    if(newHeading.headingAccuracy < 0){
        [appDelegate.rootDevice setDeviceHeading:-1];
        return;
    }
    
    //remember true heading in rootDevice...
    CLLocationDirection theHeading = ((newHeading.trueHeading > 0) ? newHeading.trueHeading : newHeading.magneticHeading);
    [appDelegate.rootDevice setDeviceHeading:(int)theHeading];
    
}


//location update errors
- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error{
    NSMutableString *errorString = [[NSMutableString alloc] init];
    
    //stop updating...
    [self.locationManager stopUpdatingLocation];
    [appDelegate.rootDevice setCanReportLocation:FALSE];
    self.updateCount = 0;
    if(self.updateTimer == nil){
        [self.updateTimer invalidate];
    }

    //figure out the error...
    if ([error domain] == kCLErrorDomain) {
        
        //handle CoreLocation errors...
        switch ([error code]) {
            case kCLErrorDenied:
                [errorString appendFormat:@"%@\n", @"Location Services denied to this app"];
                break;
            case kCLErrorLocationUnknown:
                [errorString appendFormat:@"%@\n", @"Location unknown"];
                break;
            default:
                [errorString appendFormat:@"GENERIC %@ %li\n", @"Generic location services error: ", (long)[error code]];
                break;
        }
 
        //debug
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"didFailWithError: %@", errorString]];
        
    }
    
    
}



@end





