/*
 *	Copyright, Buzztouch
 *  SEE ADDITIONAL COPYRIGHT BELOW. THIS SOURCE WAS MODIFIED FROM TONY MILLION'S ORIGINAL VERSION
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */

/*
 Copyright (c) 2011, Tony Million.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "BT_reachability.h"


NSString *const bt_NetworkStatusChangedNotification = @"bt_NetworkStatusChangedNotification";

@interface BT_reachability()

@property (nonatomic, assign) SCNetworkReachabilityRef reachabilityRef;


#if NEEDS_DISPATCH_RETAIN_RELEASE
@property (nonatomic, assign) dispatch_queue_t reachabilitySerialQueue;
#else
@property (nonatomic, strong) dispatch_queue_t reachabilitySerialQueue;
#endif


@property (nonatomic, strong) id reachabilityObject;

-(void)reachabilityChanged:(SCNetworkReachabilityFlags)flags;
-(BOOL)isReachableWithFlags:(SCNetworkReachabilityFlags)flags;

@end

static NSString *reachabilityFlags(SCNetworkReachabilityFlags flags){
    return [NSString stringWithFormat:@"%c%c %c%c%c%c%c%c%c",
    #if	TARGET_OS_IPHONE
            (flags & kSCNetworkReachabilityFlagsIsWWAN)               ? 'W' : '-',
    #else
            'X',
    #endif
            (flags & kSCNetworkReachabilityFlagsReachable)            ? 'R' : '-',
            (flags & kSCNetworkReachabilityFlagsConnectionRequired)   ? 'c' : '-',
            (flags & kSCNetworkReachabilityFlagsTransientConnection)  ? 't' : '-',
            (flags & kSCNetworkReachabilityFlagsInterventionRequired) ? 'i' : '-',
            (flags & kSCNetworkReachabilityFlagsConnectionOnTraffic)  ? 'C' : '-',
            (flags & kSCNetworkReachabilityFlagsConnectionOnDemand)   ? 'D' : '-',
            (flags & kSCNetworkReachabilityFlagsIsLocalAddress)       ? 'l' : '-',
            (flags & kSCNetworkReachabilityFlagsIsDirect)             ? 'd' : '-'];
}


//listen for reachability notifications...
static void TMReachabilityCallback(SCNetworkReachabilityRef target, SCNetworkReachabilityFlags flags, void* info){
#pragma unused (target)
    #if __has_feature(objc_arc)
        BT_reachability *reachability = ((__bridge BT_reachability*)info);
    #else
        Reachability *reachability = ((Reachability*)info);
    #endif
    
    @autoreleasepool{
        [reachability reachabilityChanged:flags];
    }
}


@implementation BT_reachability

@synthesize reachabilityRef;
@synthesize reachabilitySerialQueue;
@synthesize reachableOnWWAN;
@synthesize reachableBlock;
@synthesize unreachableBlock;
@synthesize reachabilityObject;

#pragma mark - class constructor methods
+(BT_reachability*)reachabilityWithHostname:(NSString*)hostname{
    SCNetworkReachabilityRef ref = SCNetworkReachabilityCreateWithName(NULL, [hostname UTF8String]);
    if (ref){
        id reachability = [[self alloc] initWithReachabilityRef:ref];
        
        #if __has_feature(objc_arc)
                return reachability;
        #else
                return [reachability autorelease];
        #endif
        
    }
    
    return nil;
}

// initialization methods
-(BT_reachability *)initWithReachabilityRef:(SCNetworkReachabilityRef)ref{
    self = [super init];
    if (self != nil){
        self.reachableOnWWAN = YES;
        self.reachabilityRef = ref;
    }
    
    return self;
}

-(void)dealloc{
    [self stopNotifier];
    
    if(self.reachabilityRef){
        CFRelease(self.reachabilityRef);
        self.reachabilityRef = nil;
    }
    
	self.reachableBlock		= nil;
	self.unreachableBlock	= nil;
    
    #if !(__has_feature(objc_arc))
        [super dealloc];
    #endif
    
    
}

#pragma mark - notifier methods

// Notifier
// NOTE: this uses GCD to trigger the blocks - they *WILL NOT* be called on THE MAIN THREAD
// - In other words DO NOT DO ANY UI UPDATES IN THE BLOCKS.
//   INSTEAD USE dispatch_async(dispatch_get_main_queue(), ^{UISTUFF}) (or dispatch_sync if you want)

-(BOOL)startNotifier{
    SCNetworkReachabilityContext    context = { 0, NULL, NULL, NULL, NULL };
    
    self.reachabilityObject = self;
    self.reachabilitySerialQueue = dispatch_queue_create("com.buzztouch.com", NULL);
    if(!self.reachabilitySerialQueue){
        return NO;
    }
    
    #if __has_feature(objc_arc)
        context.info = (__bridge void *)self;
    #else
        context.info = (void *)self;
    #endif
    
    if (!SCNetworkReachabilitySetCallback(self.reachabilityRef, TMReachabilityCallback, &context)){
        
        //clear out the dispatch queue
        if(self.reachabilitySerialQueue){
            #if NEEDS_DISPATCH_RETAIN_RELEASE
                dispatch_release(self.reachabilitySerialQueue);
            #endif
            self.reachabilitySerialQueue = nil;
        }
        
        self.reachabilityObject = nil;
        
        return NO;
    }
    
    // set it as our reachability queue which will retain the queue
    if(!SCNetworkReachabilitySetDispatchQueue(self.reachabilityRef, self.reachabilitySerialQueue)){
        
        // first stop any callbacks!
        SCNetworkReachabilitySetCallback(self.reachabilityRef, NULL, NULL);
        
        // then clear out the dispatch queue
        if(self.reachabilitySerialQueue){
            #if NEEDS_DISPATCH_RETAIN_RELEASE
                dispatch_release(self.reachabilitySerialQueue);
            #endif
            self.reachabilitySerialQueue = nil;
        }
        
        self.reachabilityObject = nil;
        
        return NO;
    }
    
    return YES;
}

-(void)stopNotifier{
    // first stop any callbacks!
    SCNetworkReachabilitySetCallback(self.reachabilityRef, NULL, NULL);
    
    // unregister target from the GCD serial dispatch queue
    SCNetworkReachabilitySetDispatchQueue(self.reachabilityRef, NULL);
    
    if(self.reachabilitySerialQueue){
        #if NEEDS_DISPATCH_RETAIN_RELEASE
            dispatch_release(self.reachabilitySerialQueue);
        #endif
        self.reachabilitySerialQueue = nil;
    }
    
    self.reachabilityObject = nil;
}

#pragma mark - reachability tests

#define testcase (kSCNetworkReachabilityFlagsConnectionRequired | kSCNetworkReachabilityFlagsTransientConnection)

-(BOOL)isReachableWithFlags:(SCNetworkReachabilityFlags)flags{
    BOOL connectionUP = YES;
    
    if(!(flags & kSCNetworkReachabilityFlagsReachable))
        connectionUP = NO;
    if( (flags & testcase) == testcase )
        connectionUP = NO;
    
    #if	TARGET_OS_IPHONE
        if(flags & kSCNetworkReachabilityFlagsIsWWAN){
            // we're on 3G
            if(!self.reachableOnWWAN){
                // we dont want to connect when on 3G
                connectionUP = NO;
            }
        }
    #endif
    
    return connectionUP;
}

-(BOOL)isReachable{
    SCNetworkReachabilityFlags flags;
    
    if(!SCNetworkReachabilityGetFlags(self.reachabilityRef, &flags))
        return NO;
    
    return [self isReachableWithFlags:flags];
}

-(BOOL)isReachableViaWWAN{
    #if	TARGET_OS_IPHONE
        SCNetworkReachabilityFlags flags = 0;
    
        if(SCNetworkReachabilityGetFlags(reachabilityRef, &flags)){
            // check we're REACHABLE
            if(flags & kSCNetworkReachabilityFlagsReachable){
                // now, check we're on WWAN
                if(flags & kSCNetworkReachabilityFlagsIsWWAN){
                    return YES;
                }
            }
        }
    #endif
    return NO;
}

-(BOOL)isReachableViaWiFi{
    SCNetworkReachabilityFlags flags = 0;
    
    if(SCNetworkReachabilityGetFlags(reachabilityRef, &flags)){
        // check we're reachable
        if((flags & kSCNetworkReachabilityFlagsReachable)){
            #if	TARGET_OS_IPHONE
                // check we're NOT on WWAN
                if((flags & kSCNetworkReachabilityFlagsIsWWAN)){
                    return NO;
                }
            #endif
            return YES;
        }
    }
    
    return NO;
}


-(BOOL)isConnectionRequired{
    return [self connectionRequired];
}

-(BOOL)connectionRequired{
    SCNetworkReachabilityFlags flags;
	
	if(SCNetworkReachabilityGetFlags(reachabilityRef, &flags)){
		return (flags & kSCNetworkReachabilityFlagsConnectionRequired);
	}
    
    return NO;
}

// Dynamic, on demand connection?
-(BOOL)isConnectionOnDemand{
	SCNetworkReachabilityFlags flags;
	
	if (SCNetworkReachabilityGetFlags(reachabilityRef, &flags)){
		return ((flags & kSCNetworkReachabilityFlagsConnectionRequired) &&
				(flags & (kSCNetworkReachabilityFlagsConnectionOnTraffic | kSCNetworkReachabilityFlagsConnectionOnDemand)));
	}
	
	return NO;
}

// Is user intervention required?
-(BOOL)isInterventionRequired{
    SCNetworkReachabilityFlags flags;
	
	if (SCNetworkReachabilityGetFlags(reachabilityRef, &flags)){
		return ((flags & kSCNetworkReachabilityFlagsConnectionRequired) &&
				(flags & kSCNetworkReachabilityFlagsInterventionRequired));
	}
	
	return NO;
}


#pragma mark - reachability status stuff

-(NetworkStatus)currentReachabilityStatus{
    if([self isReachable]){
        if([self isReachableViaWiFi]){
            return ReachableViaWiFi;
        }
        
        #if	TARGET_OS_IPHONE
            return ReachableViaWWAN;
        #endif
    }
    
    return NotReachable;
}

-(SCNetworkReachabilityFlags)reachabilityFlags{
    SCNetworkReachabilityFlags flags = 0;
    if(SCNetworkReachabilityGetFlags(reachabilityRef, &flags)){
        return flags;
    }
    return 0;
}

-(NSString*)currentReachabilityString{
	NetworkStatus temp = [self currentReachabilityStatus];
	if(temp == reachableOnWWAN){
		return NSLocalizedString(@"Cellular", @"");
	}
	if (temp == ReachableViaWiFi){
		return NSLocalizedString(@"WiFi", @"");
	}
	return NSLocalizedString(@"No Connection", @"");
}

-(NSString*)currentReachabilityFlags{
    return reachabilityFlags([self reachabilityFlags]);
}

-(void)reachabilityChanged:(SCNetworkReachabilityFlags)flags{
    if([self isReachableWithFlags:flags]){
        if(self.reachableBlock){
            self.reachableBlock(self);
        }
    }else{
        if(self.unreachableBlock){
            self.unreachableBlock(self);
        }
    }
    
    //post notification on the main thread...
    dispatch_async(dispatch_get_main_queue(), ^{
        [[NSNotificationCenter defaultCenter] postNotificationName:bt_NetworkStatusChangedNotification object:self];
    });
}

-(NSString *)description;{
    NSString *description = [NSString stringWithFormat:@"<%@: %#x>", NSStringFromClass([self class]), (unsigned int) self];
    return description;
}

@end





