/*
 *	Copyright, Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */

#import "BT_soundEffectPlayer.h"


@implementation BT_soundEffectPlayer

//init application...
-(id)init{
    if((self = [super init])){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"INIT%@", @""]];
        
    }
    return self;
}

//loadSoundEffects...
-(void)loadSoundEffects{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadSoundEffects%@", @""]];
    
    //load sound effects in the background using Grand Central Dispatch...
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{

        //fill an array of sound effect file names...
        [self setSoundEffectNames:[[NSMutableArray alloc] init]];
        [self.soundEffectNames addObject:@"bt_correct.mp3"];
        [self.soundEffectNames addObject:@"bt_incorrect.mp3"];
        [self.soundEffectNames addObject:@"bt_right.mp3"];
        [self.soundEffectNames addObject:@"bt_wrong.mp3"];
        [self.soundEffectNames addObject:@"bt_blow.mp3"];
        [self.soundEffectNames addObject:@"bt_bottle.mp3"];
        [self.soundEffectNames addObject:@"bt_frog.mp3"];
        [self.soundEffectNames addObject:@"bt_funk.mp3"];
        [self.soundEffectNames addObject:@"bt_glass.mp3"];
        [self.soundEffectNames addObject:@"bt_hero.mp3"];
        [self.soundEffectNames addObject:@"bt_morse.mp3"];
        [self.soundEffectNames addObject:@"bt_ping.mp3"];
        [self.soundEffectNames addObject:@"bt_pop.mp3"];
        [self.soundEffectNames addObject:@"bt_purr.mp3"];
        [self.soundEffectNames addObject:@"bt_sosumi.mp3"];
        [self.soundEffectNames addObject:@"bt_submarine.mp3"];
        [self.soundEffectNames addObject:@"bt_tink.mp3"];
        
        //setup audio session for sound effects...
        [[AVAudioSession sharedInstance] setCategory: AVAudioSessionCategoryAmbient error: nil];
        [[AVAudioSession sharedInstance] setActive: YES error: nil];
        
        //fill an array of sound effect player objects to pre-load with each audio track...
        [self setSoundEffectPlayers:[[NSMutableArray alloc] init]];
        for(int x = 0; x < [self.soundEffectNames count]; x++){
            NSString *theFileName = [self.soundEffectNames objectAtIndex:x];
            if([BT_fileManager doesFileExistInBundle:theFileName]){
                NSURL *soundFileUrl = [NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/%@", [[NSBundle mainBundle] resourcePath], theFileName]];
                NSError *error;
                AVAudioPlayer *tmpPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:soundFileUrl error:&error];
                [tmpPlayer setNumberOfLoops:0];
                [tmpPlayer setDelegate:self];
                [self.soundEffectPlayers addObject:tmpPlayer];
            }
        }
        
    });
    
}


//playSoundEffect...
-(void)playSoundEffect:(NSString *)theFileName{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"playSoundEffect %@", theFileName]];
    if([theFileName length] > 3){
        if([self.soundEffectNames containsObject:theFileName]){
            int playerIndex = (int)[self.soundEffectNames indexOfObject:theFileName];
            AVAudioPlayer *tmpPlayer = (AVAudioPlayer *)[self.soundEffectPlayers objectAtIndex:playerIndex];
            if(tmpPlayer){
                [tmpPlayer play];
            }
        }else{
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"playSoundInBundle:ERROR. This sound effect is not included in the list of available sounds: %@", theFileName]];
        }
    }
    
}



@end














