/*
 *	Copyright, Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */


#import <CommonCrypto/CommonDigest.h>
#import "BT_strings.h"
#import "BT_keyChain.h"
#import <Security/Security.h>


@implementation BT_strings

//base64EncodingTable...
static char base64EncodingTable[64] = {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
    'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
    'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
    'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/'
};

//createGuid...
+(NSString *)createGuid{
    CFUUIDRef uuidObject = CFUUIDCreate(kCFAllocatorDefault);
    NSString *uuidStr = (NSString *)CFBridgingRelease(CFUUIDCreateString(kCFAllocatorDefault, uuidObject));
    return uuidStr;
}

//getPrefString...
+(NSString *)getPrefString:(NSString *)nameOfPref{
    //returns emtpy string or string value of saved preference
    NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
    NSString *ret = @"";
    if([prefs stringForKey:nameOfPref] != nil){
        ret = [prefs stringForKey:nameOfPref];
    }
    return ret;
}

//setPrefString...
+(void)setPrefString:(NSString *)nameOfPref valueOfPref:(NSString *)valueOfPref{
    NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
    [prefs setObject:valueOfPref forKey:nameOfPref];
    [prefs synchronize];
}

//replaceString...
+(NSString *)replaceString:(NSString *)original findString:(NSString *)findString replaceWith:(NSString *)replaceWith{
    NSString *ret = original;
    ret = [ret stringByReplacingOccurrencesOfString:findString withString:replaceWith];
    return ret;
}

//mergeBTVariablesInString...
+(NSString *)mergeBTVariablesInString:(NSString *)theString{
    //[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"mergeBTVariablesInString (before): %@", theString]];
    NSString *ret = theString;
    
    //spaces in URL's...
    ret = [ret stringByReplacingOccurrencesOfString:@"%20" withString:@" "];
    
    //if we have an app...
    if([appDelegate rootApp] != nil){
        
        //application
        
        ret = [ret stringByReplacingOccurrencesOfString:@"[buzztouchAppId]" withString:[appDelegate.rootApp buzztouchAppId]];
        ret = [ret stringByReplacingOccurrencesOfString:@"[buzztouchAPIKey]" withString:[appDelegate.rootApp buzztouchAPIKey]];
        
        //screen
        if([appDelegate.rootApp currentScreenData] != nil){
            if([appDelegate.rootApp.currentScreenData.jsonVars objectForKey:@"itemId"] != nil){
                ret = [ret stringByReplacingOccurrencesOfString:@"[screenId]" withString:[appDelegate.rootApp.currentScreenData.jsonVars objectForKey:@"itemId"]];
            }
        }

        //user
        ret = [ret stringByReplacingOccurrencesOfString:@"[userId]" withString:[appDelegate.rootUser userId]];
        ret = [ret stringByReplacingOccurrencesOfString:@"[userEmail]" withString:[appDelegate.rootUser userEmail]];
        ret = [ret stringByReplacingOccurrencesOfString:@"[userLogInId]" withString:[appDelegate.rootUser userLogInId]];
        
        //device
        ret = [ret stringByReplacingOccurrencesOfString:@"[deviceId]" withString:[appDelegate.rootDevice deviceId]];
        ret = [ret stringByReplacingOccurrencesOfString:@"[deviceLatitude]" withString:[appDelegate.rootDevice deviceLastLatitude]];
        ret = [ret stringByReplacingOccurrencesOfString:@"[deviceLongitude]" withString:[appDelegate.rootDevice deviceLastLongitude]];
        ret = [ret stringByReplacingOccurrencesOfString:@"[deviceModel]" withString:[appDelegate.rootDevice deviceModel]];
        
    }//root app..
    
    //send it back
    return ret;
    
}


//getJsonPropertyValue...
+(NSString *)getJsonPropertyValue:(NSDictionary *)theValues nameOfProperty:(NSString *) nameOfProperty defaultValue:(NSString *) defaultValue{
    
    //-(void) getAddress:(NSString *)objectType coordinate:(CLLocationCoordinate2D) objectCoordinate;
    
    if([theValues objectForKey:nameOfProperty]){
        return (NSString *)[theValues objectForKey:nameOfProperty];
    }else{
        return (NSString *)defaultValue;
    }
    
}

//getStyleValueForScreen...
+(NSString *)getStyleValueForScreen:(BT_item *)theParentScreenData nameOfProperty:(NSString *)nameOfProperty defaultValue:(NSString *)defaultValue{
    //[BT_debugger showIt:self:[NSString stringWithFormat:@"getStyleValueForScreen", @""]];
    
    //send retValue back...
    NSString *retValue = defaultValue;
    
    //find the value form the app's global theme first then over-ride as needed
    if(appDelegate.rootApp.rootTheme != nil){
        if([[appDelegate.rootApp.rootTheme jsonVars] objectForKey:nameOfProperty]){
            if([[[appDelegate.rootApp.rootTheme jsonVars] objectForKey:nameOfProperty] length] > 0){
                retValue = [[appDelegate.rootApp.rootTheme jsonVars] objectForKey:nameOfProperty];
            }
        }
        //screen over-ride...
        if([[theParentScreenData jsonVars] objectForKey:nameOfProperty]){
            if([[[theParentScreenData jsonVars] objectForKey:nameOfProperty] length] > 0){
                retValue = [[theParentScreenData jsonVars] objectForKey:nameOfProperty];
            }
        }
    }else{
        [BT_debugger showIt:self theMessage:@"getStyleValueForScreen, appDelegate.rootApp.rootTheme is null"];
    }//not using a theme...
    
    //return
    return retValue;
}

//formatTimeFromSeconds...
+(NSString *)formatTimeFromSeconds:(NSString *)seconds{
    
    // Return variable.
    NSString *result = @"";
    
    // Int variables for calculation.
    int secs = [seconds intValue];
    int tempHour    = 0;
    int tempMinute  = 0;
    int tempSecond  = 0;
    
    NSString *hour      = @"";
    NSString *minute    = @"";
    NSString *second    = @"";
    
    // Convert the seconds to hours, minutes and seconds.
    tempHour    = secs / 3600;
    tempMinute  = secs / 60 - tempHour * 60;
    tempSecond  = secs - (tempHour * 3600 + tempMinute * 60);
    
    hour    = [[NSNumber numberWithInt:tempHour] stringValue];
    minute  = [[NSNumber numberWithInt:tempMinute] stringValue];
    second  = [[NSNumber numberWithInt:tempSecond] stringValue];
    
    // Make time look like 00:00:00 and not 0:0:0
    if(tempHour < 10){
        hour = [@"0" stringByAppendingString:hour];
    }
    
    if(tempMinute < 10){
        minute = [@"0" stringByAppendingString:minute];
    }
    
    if(tempSecond < 10){
        second = [@"0" stringByAppendingString:second];
    }
    
    if(tempHour == 0){
        result = [NSString stringWithFormat:@"%@:%@", minute, second];
    }else{
        result = [NSString stringWithFormat:@"%@:%@:%@",hour, minute, second];
    }
    
    return result;
    
}

//dateGMTDateFromString...
+(NSString *)dateGMTDateFromString:(NSString *)theGMTDateString{
    
    //make date object from the passed in string..
    NSDateFormatter *df = [[NSDateFormatter alloc] init];
    NSLocale *locale = [[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"];
    [df setLocale:locale];
    [df setDateFormat:@"EEE, dd MMM yyyy HH:mm:ss Z"];
    NSDate *formattedDate = [df dateFromString:theGMTDateString];
    df = nil;
    
    //format the date object
    NSDateFormatter *displayFormatter = [[NSDateFormatter alloc] init];
    [displayFormatter setLocale:locale];
    [displayFormatter setDateFormat:@"EEEE dd MMMM yyyy HH:mm"];
    NSString *newDateString = [displayFormatter stringFromDate:formattedDate];
    displayFormatter = nil;
    
    return newDateString;
    
}

//cleanUpCharacterData...
+(NSString *)cleanUpCharacterData:(NSString *)theString{
    
    //replace crazy stuff that should not be in UTF-8 data...
    NSString *newString = theString;
    newString = [newString stringByReplacingOccurrencesOfString:@"&amp;"  withString:@"&"];
    newString = [newString stringByReplacingOccurrencesOfString:@"&quot;" withString:@"\""];
    newString = [newString stringByReplacingOccurrencesOfString:@"&#039;"  withString:@"'"];
    newString = [newString stringByReplacingOccurrencesOfString:@"&apos;"  withString:@"'"];
    newString = [newString stringByReplacingOccurrencesOfString:@"&gt;"  withString:@">"];
    newString = [newString stringByReplacingOccurrencesOfString:@"&lt;"  withString:@"<"];
    
    //return..
    return newString;
}


//stripHTMLFromString...
+(NSString *)stripHTMLFromString:(NSString *)html {
    
    NSScanner *theScanner;
    NSString *text = nil;
    
    theScanner = [NSScanner scannerWithString:html];
    
    while ([theScanner isAtEnd] == FALSE) {
        
        // find start of tag
        [theScanner scanUpToString:@"<" intoString:nil];
        
        // find end of tag
        [theScanner scanUpToString:@">" intoString:&text];
        
        // replace the found tag with a space
        //(you can filter multi-spaces out later if you wish)
        html = [html stringByReplacingOccurrencesOfString:[NSString stringWithFormat:@"%@>", text]
                                               withString:@" "];
        
    }//while
    
    return html;
    
}

/*
 gets file name from a URL
 
 1) Look for a saveAsFileName in the URL string. URL must have at least one & (ampersand). Like...
 mydomain.com/file_stream.php?id=234234&saveAsFileName=myCoolImage.png
 
 ELSE
 
 2) Use objective C lastPathComponent method like
 mydomain.com/images/myCoolImage.png
 
 */

//getFileNameFromURL...
+(NSString *)getFileNameFromURL:(NSString *)theURLString{
    NSString *ret = @"";
    BOOL foundFileName = FALSE;
    if([theURLString length] > 0){
        if([theURLString rangeOfString:@"&" options:NSCaseInsensitiveSearch].location != NSNotFound){
            NSArray *queryStringParts = [theURLString componentsSeparatedByString:@"&"];
            if([queryStringParts count] > 0){
                for(NSString* key in queryStringParts){
                    if([key rangeOfString:@"saveAsFileName" options:NSCaseInsensitiveSearch].location != NSNotFound){
                        NSArray *fileNameParts = [key componentsSeparatedByString:@"="];
                        if([fileNameParts count] == 2){
                            foundFileName = true;
                            ret = [fileNameParts objectAtIndex:1];
                        }
                    }
                }
            }
            
        }
    }
    if(!foundFileName && [theURLString length] > 0){
        ret = [theURLString lastPathComponent];
    }
    
    return ret;
}




//md5String...
+(NSString *)md5String:(NSString *)theString{
    
    const char *cStr = [theString UTF8String];
    unsigned char result[CC_MD5_DIGEST_LENGTH];
    CC_MD5( cStr, (int)strlen(cStr), result );
    return [NSString stringWithFormat:@"%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X",
            result[0], result[1],
            result[2], result[3],
            result[4], result[5],
            result[6], result[7],
            result[8], result[9],
            result[10], result[11],
            result[12], result[13],
            result[14], result[15]
            ];
}



//getOrCreateDeviceId...
+(NSString *)getOrCreateDeviceId{
    NSString *retrieveUUID = [BT_keyChain passwordForService:@"com.buztouch.com" account:@"user"];
    if(retrieveUUID == nil){
        retrieveUUID  = [self createGuid];
        [BT_keyChain setPassword:retrieveUUID forService:@"com.buztouch.com" account:@"user"];
    }
    
    //return it...
    return retrieveUUID;
    
}

//isNumeric...
+(BOOL)isNumeric:(NSString *)theString{
    NSString *number = theString;
    NSString *regex = @"[0-9]*";
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", regex];
    return [predicate evaluateWithObject:number];
}


//getBase64StringFromData...
+(NSString *)getBase64StringFromData:(NSData *)data length:(int)length{
    unsigned long ixtext, lentext;
    long ctremaining;
    unsigned char input[3], output[4];
    short i, charsonline = 0, ctcopy;
    const unsigned char *raw;
    NSMutableString *result;
    
    lentext = [data length];
    if (lentext < 1)
        return @"";
    result = [NSMutableString stringWithCapacity: lentext];
    raw = [data bytes];
    ixtext = 0;
    
    while (true) {
        ctremaining = lentext - ixtext;
        if (ctremaining <= 0)
            break;
        for (i = 0; i < 3; i++) {
            unsigned long ix = ixtext + i;
            if (ix < lentext)
                input[i] = raw[ix];
            else
                input[i] = 0;
        }
        output[0] = (input[0] & 0xFC) >> 2;
        output[1] = ((input[0] & 0x03) << 4) | ((input[1] & 0xF0) >> 4);
        output[2] = ((input[1] & 0x0F) << 2) | ((input[2] & 0xC0) >> 6);
        output[3] = input[2] & 0x3F;
        ctcopy = 4;
        switch (ctremaining) {
            case 1:
                ctcopy = 2;
                break;
            case 2:
                ctcopy = 3;
                break;
        }
        
        for (i = 0; i < ctcopy; i++)
            [result appendString: [NSString stringWithFormat: @"%c",             base64EncodingTable[output[i]]]];
        
        for (i = ctcopy; i < 4; i++)
            [result appendString: @"="];
        
        ixtext += 3;
        charsonline += 4;
        
        if ((length > 0) && (charsonline >= length))
            charsonline = 0;
    }
    return result;
}

//formatBytes...
+(NSString *)formatBytes:(int)numberOfBytes{
    return [NSByteCountFormatter stringFromByteCount:numberOfBytes countStyle:NSByteCountFormatterCountStyleFile];
}


@end









