/*
 *	Copyright, buzztouch.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */

#import <MediaPlayer/MediaPlayer.h>
#import "BT_viewController.h"

@implementation BT_viewController


//initWithScreenData
-(id)initWithScreenData:(BT_item *)theScreenData{
    if((self = [super init])){
        [BT_debugger showIt:self message:@"INIT"];
        
        //set screen data
        [self setScreenData:theScreenData];
        
    }
    return self;
}

//viewDidLoad...
-(void)viewDidLoad{
    [BT_debugger showIt:self theMessage:@"viewDidLoad (BT_viewController)"];
    [super viewDidLoad];
    
    //adjust view edges on iOS7, depending on the navBar transparency...
    if([self respondsToSelector:@selector(edgesForExtendedLayout)]){
        if(![[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@""] isEqualToString:@"transparent"]){
            self.edgesForExtendedLayout = UIRectEdgeNone;
        }
    }
    
    //listen for networkMonitor changes using NSNotification observer....
    [[NSNotificationCenter defaultCenter] addObserver:appDelegate selector:@selector(networkConnectionTypeChanged:) name:bt_NetworkStatusChangedNotification object:nil];
    [appDelegate.rootNetworkMonitor startNotifier];
    
    //listen for appDidReportToCloud notification so we can run appDidReportToCloud when it's complete...
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appDidReportToCloud) name:@"appDidReportToCloud" object:nil];
    

    
    
}

//viewWillAppear...
-(void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [BT_debugger showIt:self theMessage:@"viewWillAppear (BT_viewController)"];
    

    //flag this as the current screen...
    appDelegate.rootApp.currentScreenData = self.screenData;
    
    //setup navBar...
    [self configureNavBar];
    
    //setup background...
    [self configureBackground];
    
    //show view...
    [self fadeInOnEntry];
    
}

//viewWillLayoutSubviews...
- (void)viewWillLayoutSubviews{
    [super viewWillLayoutSubviews];
    [BT_debugger showIt:self theMessage:@"viewWillLayoutSubviews (BT_viewController)"];
    
    //is the nav bar hidden...
    if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@""] isEqualToString:@"hidden"]){
        [self.navigationController setNavigationBarHidden:TRUE animated:FALSE];
    }else{
        [self.navigationController setNavigationBarHidden:FALSE animated:FALSE];
    }
    
}



//viewWillDisappear...
-(void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [BT_debugger showIt:self theMessage:@"viewWillDisappear (BT_viewController)"];
    
    //hide view so next view does not overlay...
    [self fadeOutOnExit];
    
    
}

//fadeInOnEntry...
-(void)fadeInOnEntry{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"fadeInOnEntry (BT_viewController)%@:", @""]];
    
    //animate the menu...
    [UIView animateWithDuration:0.5
                     animations:^{
                         [self.view setAlpha:1.0];
                     }
                     completion:^(BOOL finished){
                         
                     }];
    
}

//fadeOutOnExit...
-(void)fadeOutOnExit{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"fadeOutOnExit (BT_viewController)%@:", @""]];
    
    //animate the menu...
    [UIView animateWithDuration:0.25
                     animations:^{
                         [self.view setAlpha:0];
                     }
                     completion:^(BOOL finished){
                         
                     }];
    
}


//prefersStatusBarHidden...
-(BOOL)prefersStatusBarHidden{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"prefersStatusBarHidden%@", @""]];
    
   //hide the status bar if this is the BT_loading screen (app launching)...
    if(self.screenData == nil || [self.screenData.itemType isEqualToString:@"BT_loading"]){
        return YES;
    }else{
        
        //hide or show status bar...
        if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"statusBarStyle" defaultValue:@""] isEqualToString:@"hidden"]){
            return YES;
        }else{
            return NO;
        }
    }
    
    //show not be here...
    return YES;
    
}

//preferredStatusBarStyle...
-(UIStatusBarStyle)preferredStatusBarStyle{
    //[BT_debugger showIt:self message:[NSString stringWithFormat:@"preferredStatusBarStyle%@", @""]];
    
    /*
     statusBarStyle options
     ----------------------
     "default"       status bar shows default appearance
     "solid"         status bar is hidden entirely
     "transparent"   status bar
     "hidden"        status bar is hidden entirely
     
     */
    
    //hidden handled by prefersStatusBarHidden (above)...
    if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"statusBarStyle" defaultValue:@""] isEqualToString:@"hidden"]){
        return UIStatusBarStyleDefault;
    }else{
        
        //default...
        if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"statusBarStyle" defaultValue:@""] isEqualToString:@"default"]){
            return UIStatusBarStyleDefault;
        }
        
        //solid...
        if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"statusBarStyle" defaultValue:@""] isEqualToString:@"solid"]){
            return UIStatusBarStyleDefault;
        }
        
        //translucent...
        if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"statusBarStyle" defaultValue:@""] isEqualToString:@"transparent"]){
            return UIStatusBarStyleDefault;
        }
    }
    
    //should not be here...
    return UIStatusBarStyleDefault;
    
}


//configureNavBar...
-(void)configureNavBar{
    
    if(self.screenData == nil){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"configureNavBar (BT_viewController) self.screenData not available, not configuring nav bar.%@", @""]];
    }else{
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"configureNavBar (BT_viewController) for screen with itemId: %@", [self.screenData itemId]]];
        
        /*
         Hiding the navigation bar is handled with: self.viewWillLayoutSubviews
         Hiding the status bar is handled with: prefersStatusBarHidden
         */
        
        
        //reset the nav bar from previous screen...
        [self.navigationController.navigationBar setBarStyle:UIBarStyleDefault];
        [self.navigationController.navigationBar setTranslucent:TRUE];
        [self.navigationController.navigationBar setTintColor:nil];
        [self.navigationController.navigationBar setBarTintColor:nil];
        [self.navigationController.navigationBar setBackgroundColor:nil];
        
        //sets the title text from the jsonVars...
        NSString *navBarText = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"navBarTitleText" defaultValue:@""];
        [self.navigationItem setTitle:navBarText];
        
        //possible title text color...
        NSString *tmpTitleTextColor = [BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarTitleTextColor" defaultValue:@"#000000"];
        if([tmpTitleTextColor length] > 1){
            [[UINavigationBar appearance] setTitleTextAttributes:@{NSForegroundColorAttributeName:[BT_color getColorFromHexString:tmpTitleTextColor]}];
        }
        
        //possible tint color for nav bar icons...
        NSString *tmpIconTintColor = [BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarIconTintColor" defaultValue:@"#000000"];
        [self.navigationController.navigationBar setTintColor:[BT_color getColorFromHexString:tmpTitleTextColor]];
        
        
        ///navBarBackgroundColor, else use default...
        NSString *tmpBarBackgroundColor = [BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarBackgroundColor" defaultValue:@"#DDDDDD"];
        
        
        //default...
        if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@"default"] isEqualToString:@"default"]){
            if([tmpBarBackgroundColor length] > 1){
                [self.navigationController.navigationBar setBackgroundColor:[BT_color getColorFromHexString:tmpBarBackgroundColor]];
            }
        }
        
        //translucent...
        if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@""] isEqualToString:@"translucent"]){
            [self.navigationController.navigationBar setBackgroundColor:[BT_color getColorFromHexString:tmpBarBackgroundColor]];
        }
        
        //solid...
        if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@""] isEqualToString:@"solid"]){
            [self.navigationController.navigationBar setTranslucent:FALSE];
            [self.navigationController.navigationBar setBarStyle:UIBarStyleBlackOpaque];
            [self.navigationController.navigationBar setBarTintColor:[BT_color getColorFromHexString:tmpBarBackgroundColor]];
        }
        
        //transparent...
        if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@""] isEqualToString:@"transparent"]){
            [self.navigationController.navigationBar setTranslucent:TRUE];
            [self.navigationController.navigationBar setBackgroundImage:[UIImage new] forBarMetrics:UIBarMetricsDefault];
            [self.navigationController.navigationBar setBackgroundColor:[UIColor clearColor]];
            [self.navigationController.view setBackgroundColor:[UIColor clearColor]];
        }
        
        
        //add a back button on all screens that are not the "home" screen...
        if([self.navigationController.viewControllers count] < 2){
            
            //add refresh button on home screen (if data URL is used)...
            SEL refreshAppData = NSSelectorFromString(@"refreshAppData");
            if([appDelegate respondsToSelector:refreshAppData]){
                if([appDelegate.rootApp.dataURL length] > 3){
                    
                    //tabbed apps only the first tab will show the refresh button.
                    if([self.screenData isHomeScreen]){
                        
                        UIBarButtonItem *theRefreshButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:appDelegate action:refreshAppData];
                        [theRefreshButtonItem setTintColor:[BT_color getColorFromHexString:tmpIconTintColor]];
                        [self.navigationItem setLeftBarButtonItem:theRefreshButtonItem];
                    }
                    
                }
            }
            
        }else{
            
            SEL sel = NSSelectorFromString(@"navLeftTap");
            if([self respondsToSelector:sel]){
                
                //hide default back button so previous screen's title doesn't show...
                [self.navigationController.navigationItem setHidesBackButton:TRUE];
                
                //use custom back button...
                UIImage *imgBack = [UIImage imageNamed:@"bt_back.png"];
                imgBack = [imgBack imageWithRenderingMode:UIImageRenderingModeAlwaysTemplate];
                UIButton *btnBack = [UIButton buttonWithType:UIButtonTypeCustom];
                [btnBack addTarget:self action:sel forControlEvents:UIControlEventTouchUpInside];
                btnBack.bounds = CGRectMake( 0, 0, 20, 30);
                [btnBack setImage:imgBack forState:UIControlStateNormal];
                UIBarButtonItem *barBtnBack = [[UIBarButtonItem alloc] initWithCustomView:btnBack];
                [barBtnBack setTintColor:[BT_color getColorFromHexString:@"#FFCC66"]];
                [self.navigationItem setLeftBarButtonItem:barBtnBack];
                
            }
            
        }
        
        //create a toolbar to hold the the possible right-side navigation buttons...
        BOOL useRightButtons = false;
        
        //create an array to hold the buttons...
        NSMutableArray *rightBarButtonArray = [[NSMutableArray alloc] initWithCapacity:2];
        
        //if we're using a context menu, create another button...
        NSString *contextMenuItemId = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"contextMenuItemId" defaultValue:@""];
        if([contextMenuItemId length] > 0){
            useRightButtons = TRUE;
            
            //selector for right button...
            SEL sel = NSSelectorFromString(@"showContextMenu");
            
            UIImage *iconImage = [[UIImage imageNamed:@"bt_hamburger.png"] imageWithRenderingMode:UIImageRenderingModeAlwaysTemplate];
            CGRect iconRect = CGRectMake(0, 0, (iconImage.size.width + 3), (iconImage.size.height + 3));
            UIImageView *icon = [[UIImageView alloc] initWithImage:iconImage];
            UIView *menuIconBox = [[UIView alloc] initWithFrame:iconRect];
            UIButton *tmpButton = [[UIButton alloc] initWithFrame:iconRect];
            [tmpButton setTintColor:[BT_color getColorFromHexString:tmpIconTintColor]];
            [tmpButton addTarget:self action:sel forControlEvents:UIControlEventTouchUpInside];
            
            //add parts, set the button...
            [menuIconBox addSubview:icon];
            [menuIconBox addSubview:tmpButton];
            UIBarButtonItem *contextMenuButton = [[UIBarButtonItem alloc] initWithCustomView:menuIconBox];
            
            //add button to array...
            [rightBarButtonArray addObject:contextMenuButton];
            
        }
        
        //add the navBarRightButtonType if needed...
        NSString *navBarRightButtonType = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"navBarRightButtonType" defaultValue:@""];
        if([navBarRightButtonType length] > 1){
            
            NSArray *supportedButtonTypes = [NSArray arrayWithObjects:@"home", @"next", @"infoLight", @"infoDark", @"details", @"addBlue", @"done",
                                             @"cancel", @"edit", @"save", @"add", @"compose", @"reply", @"action", @"organize",
                                             @"bookmark", @"search", @"refresh", @"camera", @"trash", @"play", @"pause",
                                             @"stop", @"rewind", @"fastForward", nil];
            
            //navBarRightButtonType must be in our list of options...
            if([supportedButtonTypes containsObject:navBarRightButtonType]){
                useRightButtons = TRUE;
                
                //create the button
                UIButton *theButton = nil;
                UIBarButtonItem *theBarButtonItem = nil;
                SEL selector = NSSelectorFromString(@"navRightTap");
                
                //build the right button...
                if([navBarRightButtonType isEqualToString:@"home"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"home", @"Home") style:UIBarButtonItemStylePlain target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"infoLight"]) theButton = [UIButton buttonWithType:UIButtonTypeInfoLight];
                if([navBarRightButtonType isEqualToString:@"infoDark"]) theButton = [UIButton buttonWithType:UIButtonTypeInfoDark];
                if([navBarRightButtonType isEqualToString:@"details"]) theButton = [UIButton buttonWithType:UIButtonTypeDetailDisclosure];
                if([navBarRightButtonType isEqualToString:@"addBlue"]) theButton = [UIButton buttonWithType:UIButtonTypeContactAdd];
                if([navBarRightButtonType isEqualToString:@"next"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"next", @"Next") style:UIBarButtonItemStylePlain target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"done"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"cancel"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"edit"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemEdit target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"save"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemSave target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"add"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"compose"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCompose target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"reply"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemReply target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"action"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAction target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"organize"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemOrganize target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"bookmark"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemBookmarks target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"search"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemSearch target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"refresh"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"camera"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCamera target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"trash"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemTrash target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"play"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemPlay target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"pause"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemPause target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"stop"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemStop target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"rewind"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRewind target:self action:selector ];
                if([navBarRightButtonType isEqualToString:@"fastForward"]) theBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFastForward target:self action:selector ];
                
                //add it only if the view controller can respond
                if([self respondsToSelector:selector]){
                    if(theButton != nil){
                        [theButton addTarget:self action:selector forControlEvents:UIControlEventTouchUpInside];
                        [theButton setTintColor:[BT_color getColorFromHexString:tmpIconTintColor]];
                        UIBarButtonItem *tmpBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView:theButton];
                        [rightBarButtonArray addObject:tmpBarButtonItem];
                    }
                    if(theBarButtonItem != nil){
                        [theBarButtonItem setTintColor:[BT_color getColorFromHexString:tmpIconTintColor]];
                        [rightBarButtonArray addObject:theBarButtonItem];
                    }
                }
                
            }
        }
        
        
        //are we adding one or two right buttons...
        if(useRightButtons){
            
            //add the button items array as right side button...
            [self.navigationItem setRightBarButtonItems:rightBarButtonArray];
            
        }
        
        //top bar needs updated...
        [self setNeedsStatusBarAppearanceUpdate];
        
    }//screenData == nil...
    
}

//configureNavBackground...
-(void)configureBackground{
    
    if(self.screenData == nil){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"configureBackground (BT_viewController) self.screenData not available, not configuring background.%@", @""]];
    }else{
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"configureBackground (BT_viewController) for screen with itemId %@:", [self.screenData itemId]]];
        
        //ask the global background to update itself....
        [[appDelegate.rootApp rootBackgroundView] updateProperties:self.screenData];
    
    }
}

//appDidReportToCloud...
-(void)appDidReportToCloud{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"appDidReportToCloud (BT_viewController)%@", @""]];

    /*
        Plugins can implement their own appDidReportToCloud method if they want to perform an action when
        when the "appDidReportToCloud" NSNotification is received. This NSNotification is broadcasted by the
        appDelegate after it downloads the "lastModifiedUTC" value from a server.
    */
    
}



//getNavController...
-(BT_navController *)getNavController{
    
    BT_navController *selNavController = nil;
    if([appDelegate.rootApp rootTabBarController] != nil){
        selNavController = (BT_navController *)[appDelegate.rootApp.rootTabBarController selectedViewController];
    }else{
        selNavController = [appDelegate.rootApp rootNavController];
    }
    return selNavController;
    
}


//showProgressWithMessage
-(void)showProgressWithMessage:(NSString *)theShortMessage{
    [BT_debugger showIt:self message:@"showProgressWithMessage (BT_viewController)"];
    
    //show progress view if not showing
    if(self.progressView == nil){
        self.progressView = [BT_viewUtilities getProgressView:theShortMessage];
        [self.navigationController.view addSubview:self.progressView];
    }
    
}

//show progress
-(void)showProgress{
    [BT_debugger showIt:self message:@"showProgress (BT_viewController)"];
    
    //show progress view if not showing
    if(self.progressView == nil){
        self.progressView = [BT_viewUtilities getProgressView:@""];
        [self.navigationController.view addSubview:self.progressView];
    }
    
}

//hide progress
-(void)hideProgress{
    [BT_debugger showIt:self message:@"hideProgress (BT_viewController)"];
    
    //remove progress view if already showing
    if(self.progressView != nil){
        [self.progressView removeFromSuperview];
        self.progressView = nil;
    }
    
}

//left button
-(void)navLeftTap{
    [BT_debugger showIt:self message:@"navLeftTap (BT_viewController)"];
    [[self navigationController] popViewControllerAnimated:YES];
}

//right button
-(void)navRightTap{
    [BT_debugger showIt:self message:@"navRightTap (BT_viewController)"];
    
    //get possible itemId of the screen to load
    NSString *loadScreenItemId = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"navBarRightButtonTapLoadScreenItemId" defaultValue:@""];
    
    //get possible nickname of the screen to load
    NSString *loadScreenNickname = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"navBarRightButtonTapLoadScreenNickname" defaultValue:@""];
    
    //bail if load screen = "none"
    if([loadScreenItemId isEqualToString:@"none"]){
        return;
    }
    
    //check for loadScreenWithItemId THEN loadScreenWithNickname THEN loadScreenObject
    BT_item *screenObjectToLoad = nil;
    if([loadScreenItemId length] > 1){
        screenObjectToLoad = [appDelegate.rootApp getItemDataByItemId:loadScreenItemId];
    }else{
        if([loadScreenNickname length] > 1){
            screenObjectToLoad = [appDelegate.rootApp getItemDataByNickname:loadScreenNickname];
        }else{
            if([self.screenData.jsonVars objectForKey:@"navBarRightButtonTapLoadScreenObject"]){
                screenObjectToLoad = [[BT_item alloc] init];
                [screenObjectToLoad setItemId:[[self.screenData.jsonVars objectForKey:@"navBarRightButtonTapLoadScreenObject"] objectForKey:@"itemId"]];
                [screenObjectToLoad setItemNickname:[[self.screenData.jsonVars objectForKey:@"navBarRightButtonTapLoadScreenObject"] objectForKey:@"itemNickname"]];
                [screenObjectToLoad setItemType:[[self.screenData.jsonVars objectForKey:@"navBarRightButtonTapLoadScreenObject"] objectForKey:@"itemType"]];
                [screenObjectToLoad setJsonVars:[self.screenData.jsonVars objectForKey:@"navBarRightButtonTapLoadScreenObject"]];
            }
        }
    }
    
    //if right button is "home" or "goHome"
    if([loadScreenItemId isEqualToString:@"home"] || [loadScreenItemId isEqualToString:@"goHome"]){
        
        //pop to root view controller...
        [self.navigationController popToRootViewControllerAnimated:YES];
        
        //bail
        return;
    }
    
    
    //load next screen if it's not nil
    if(screenObjectToLoad != nil){
        
        //build a temp button to pass to screen load method. We need this because the transition type is in the menu-item
        BT_item *tmpButtonData = [[BT_item alloc] init];
        
        //build an NSDictionary of values for the jsonVars property
        NSMutableDictionary *tmpDictionary = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                              @"unused", @"itemId",
                                              [self.screenData.jsonVars objectForKey:@"navBarRightButtonTapTransitionType"], @"transitionType",
                                              nil];
        [tmpButtonData setJsonVars:tmpDictionary];
        [tmpButtonData setItemId:@"0"];
        
        //load the next screen
        [self handleTapToLoadScreen:screenObjectToLoad theButtonData:tmpButtonData];
        
    }else{
        //show message
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"%@",NSLocalizedString(@"menuTapError",@"The application doesn't know how to handle this action?")]];
    }
    
    
}




//toggleTopBar...
-(void)toggleTopBar:(NSString *)withOptions{
    [BT_debugger showIt:self message:@"toggleTopBar (BT_viewController)"];
    
    CGFloat navBarHeight = 44;
    CGRect navBarFrame;
    
    //show / hide status bar...
    if([withOptions isEqualToString:@"status"] || [withOptions isEqualToString:@"both"]){
        if([[UIApplication sharedApplication] isStatusBarHidden]){
            [[UIApplication sharedApplication] setStatusBarHidden:FALSE withAnimation:UIStatusBarAnimationFade];
            navBarFrame = CGRectMake(0, 20, self.view.bounds.size.width, navBarHeight);
        }else{
            [[UIApplication sharedApplication] setStatusBarHidden:TRUE withAnimation:UIStatusBarAnimationFade];
            navBarFrame = CGRectMake(0, 0, self.view.bounds.size.width, navBarHeight);
        }
    }
    
    //show / hide navigation bar...
    if([withOptions isEqualToString:@"nav"] || [withOptions isEqualToString:@"both"]){
        if([[self.navigationController navigationBar] isHidden]){
            [[self navigationController] setNavigationBarHidden:FALSE animated:TRUE];
        }else{
            [[self navigationController] setNavigationBarHidden:TRUE animated:TRUE];
        }
        
    }
    
}

//setTopBarTitle...
-(void)setTopBarTitle:(NSString *)theTitle{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"setTopBarTitle (BT_viewController): %@", theTitle]];
    [[self navigationItem] setTitle:theTitle];
}


//canBecomeFirstResponder...
-(BOOL)canBecomeFirstResponder{
    
    //required to detect shake gestures..
    return YES;
    
}


//closeModalViewController...
-(void)closeModalViewController:(id)sender{
    [BT_debugger showIt:self message:@"closeModalViewController"];
    [self dismissViewControllerAnimated:TRUE completion:nil];
}


//showContextMenu...
-(void)showContextMenu{
    [BT_debugger showIt:self message:@"showContextMenu (BT_viewController)"];
    
    //get the contextMenu data for this screen...
    NSString *contextMenuItemId = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"contextMenuItemId" defaultValue:@""];
    BT_item *contextMenuData = nil;
    if([contextMenuItemId length] > 1){
        contextMenuData = [appDelegate.rootApp getItemDataByItemId:contextMenuItemId];
        if(contextMenuData != nil){
            
            //tell the contextMenu to load it's data...
            [appDelegate.contextMenu setMenuData:contextMenuData];
            [appDelegate.contextMenu loadData];
            
            //show the menu...
            [appDelegate showContextMenu];
            
            
        }
    }
    
}

//show alert
-(void)showAlert:(NSString *)theTitle theMessage:(NSString *)theMessage alertTag:(int)alertTag{
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:theTitle message:theMessage delegate:self
                                              cancelButtonTitle:NSLocalizedString(@"ok", "OK") otherButtonTitles:nil];
    [alertView setTag:alertTag];
    [alertView show];
}


//"OK" clicks on UIAlertView
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"alertView (BT_viewController) clickedButtonAtIndex: %li", (long)buttonIndex]];
    
    //handles OK click after emailing an image from BT_screen_imageEmail
    if([alertView tag] == 99){
        [self.navigationController popViewControllerAnimated:YES];
    }
    
    //handles OK click after sharing from BT_screen_shareFacebook or BT_screen_shareTwitter
    if([alertView tag] == 199){
        [self navLeftTap];
    }
    
}


//loadScreenWithItemId...
-(void)loadScreenWithItemId:(NSString *)theItemId{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadScreenWithItemId (BT_viewController) itemId:%@", theItemId]];
    
    //get the JSON data for screen with this itemId...
    BT_item *screenToLoad = [appDelegate.rootApp getItemDataByItemId:theItemId];
    
    //use the handleTapToLoadScreen method...
    [self handleTapToLoadScreen:screenToLoad theButtonData:nil];
    
}

//loadScreenWithNickname...
-(void)loadScreenWithNickname:(NSString *)theNickname{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadScreenWithNickname (BT_viewController) Nickname:%@", theNickname]];
    
    //get the JSON data for screen with this nickname...
    BT_item *screenToLoad = [appDelegate.rootApp getItemDataByNickname:theNickname];
    
    //use the handleTapToLoadScreen method...
    [self handleTapToLoadScreen:screenToLoad theButtonData:nil];
}

//loadScreenObject...
-(void)loadScreenObject:(BT_item *)theScreenData{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadScreenObject (BT_viewController) itemId:%@", [theScreenData itemId]]];
    [self handleTapToLoadScreen:theScreenData theButtonData:nil];
}


//handleTapToLoadScreen (with theMenuItemData)...
-(void)handleTapToLoadScreen:(BT_item *)theScreenData theMenuItemData:(BT_item *)theMenuItemData{
    [self handleTapToLoadScreen:theScreenData theButtonData:theMenuItemData];
}


//handleTapToLoadScreen (with theButtonData)...
-(void)handleTapToLoadScreen:(BT_item *)theScreenData theButtonData:(BT_item *)theButtonData{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"handleTapToLoadScreen (BT_viewController) loading nickname: \"%@\" itemId: %@ itemType: %@", [theScreenData itemNickname], [theScreenData itemId], [theScreenData itemType]]];
    
    //if the loadScreenItemId == "none".....
    if([[BT_strings getJsonPropertyValue:theButtonData.jsonVars nameOfProperty:@"loadScreenWithItemId" defaultValue:@""] isEqualToString:@"none"]){
        return;
    }
    
    //if the screen we are loading requires a login, we can't continue unless the user is logged in.
    //assume that the screen does not require a login.
    BOOL allowNextScreen = TRUE;
    if([[BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"loginRequired" defaultValue:@"0"] isEqualToString:@"1"]){
        if(![[appDelegate.rootUser userIsLoggedIn] isEqualToString:@"1"]){
            allowNextScreen = FALSE;
        }
    }
    
    //show password protected message or continue...
    if(!allowNextScreen){
        
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"loginRequired",@"~ Login Required ~")
                                                            message:NSLocalizedString(@"loginRequiredMessage", @"You are not logged in. A login is required to access this screen.")
                                                           delegate:nil
                                                  cancelButtonTitle:NSLocalizedString(@"ok", "OK")
                                                  otherButtonTitles:nil];
        [alertView setTag:101];
        [alertView show];
        
        //bail
        return;
        
    }
    if(allowNextScreen){
        
        //remember current button (whatever was tapped, could be a row, button, etc)...
        [appDelegate.rootApp setCurrentButtonData:theButtonData];
        
        //remember current screen object...
        [appDelegate.rootApp setCurrentScreenData:theScreenData];
        
        //remember previous screen object...
        [appDelegate.rootApp setPreviousScreenData:self.screenData];
        
        //some screens aren't screens at all! Like "Call Us" and "Email Us" item. In these cases, we only
        //trigger a method and do not load a BT_screen view controller.
        
        //place call
        if([[theScreenData itemType] isEqualToString:@"BT_screen_call"] ||
           [[theScreenData itemType] isEqualToString:@"BT_placeCall"]){
            
            if([appDelegate.rootDevice canMakePhoneCalls]){
                
                //trigger the place-call method
                [self placeCallWithScreenData:theScreenData];
                
            }else{
                
                //show error message
                UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"callsNotSupportedTitle",@"Calls Not Supported")
                                                                    message:NSLocalizedString(@"callsNotSupportedMessage", @"Placing calls is not supported on this device.")
                                                                   delegate:nil
                                                          cancelButtonTitle:NSLocalizedString(@"ok", "OK")
                                                          otherButtonTitles:nil];
                [alertView setTag:102];
                [alertView show];
            }
            
            //bail
            return;
        }
        
        //send email or share email
        if([[theScreenData itemType] isEqualToString:@"BT_screen_email"] ||
           [[theScreenData itemType] isEqualToString:@"BT_sendEmail"] ||
           [[theScreenData itemType] isEqualToString:@"BT_shareEmail"] ||
           [[theScreenData itemType] isEqualToString:@"BT_screen_shareEmail"]){
            
            if([appDelegate.rootDevice canSendEmails]){
                
                //trigger the email method
                [self sendEmailWithScreenData:theScreenData imageAttachment:nil imageAttachmentName:nil];
                
            }else{
                
                //show error message
                UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"emailsNotSupportedTitle",@"Email Not Supported")
                                                                    message:NSLocalizedString(@"emailsNotSupportedMessage", @"Sending eamils is not supported on this device.")
                                                                   delegate:nil
                                                          cancelButtonTitle:NSLocalizedString(@"ok", "OK")
                                                          otherButtonTitles:nil];
                [alertView setTag:103];
                [alertView show];
                
            }
            
            //bail
            return;
            
        }
        
        //send SMS or share SMS
        if([[theScreenData itemType] isEqualToString:@"BT_screen_sms"] ||
           [[theScreenData itemType] isEqualToString:@"BT_sendSms"] ||
           [[theScreenData itemType] isEqualToString:@"BT_sendSMS"] ||
           [[theScreenData itemType] isEqualToString:@"BT_shareSms"] ||
           [[theScreenData itemType] isEqualToString:@"BT_shareSMS"] ||
           [[theScreenData itemType] isEqualToString:@"BT_screen_shareSms"]){
            
            if([appDelegate.rootDevice canSendSMS]){
                
                //trigger the SMS method
                [self sendTextMessageWithScreenData:theScreenData];
                
            }else{
                
                //show error message
                UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"textMessageNotSupportedTitle", "SMS Not Supported")
                                                                    message:NSLocalizedString(@"textMessageNotSupportedMessage",  "Sending SMS / Text messages is not supported on this device.")
                                                                   delegate:nil
                                                          cancelButtonTitle:NSLocalizedString(@"ok", "OK")
                                                          otherButtonTitles:nil];
                [alertView setTag:104];
                [alertView show];
            }
            
            //bail
            return;
            
        }
        
        //BT_screen_video
        if([[theScreenData itemType] isEqualToString:@"BT_screen_video"]){
            
            NSString *localFileName = [BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"localFileName" defaultValue:@""];
            NSString *dataURL = [BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""];
            NSURL *escapedUrl = nil;
            
            /*
             video file
             --------------------------------
             a)	No dataURL is provided in the screen data - use the localFileName configured in the screen data
             b)	A dataURL is provided, check for local copy, download if not available
             
             */
            if([dataURL length] < 3){
                if([localFileName length] > 3){
                    if([BT_fileManager doesFileExistInBundle:localFileName]){
                        NSString *rootPath = [[NSBundle mainBundle] resourcePath];
                        NSString *filePath = [rootPath stringByAppendingPathComponent:localFileName];
                        escapedUrl = [NSURL fileURLWithPath:filePath isDirectory:NO];
                    }
                }
            }else{
                //merge possible varialbes in url
                dataURL = [BT_strings mergeBTVariablesInString:dataURL];
                escapedUrl = [NSURL URLWithString:[dataURL stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
            }
            
            //show movie player controller...
            if(escapedUrl != nil){
                [BT_debugger showIt:self message:[NSString stringWithFormat:@"Showing moving player controller%@", @""]];
                MPMoviePlayerViewController *moviePlayerController = [[MPMoviePlayerViewController alloc] initWithContentURL:escapedUrl];
                [moviePlayerController setModalTransitionStyle:UIModalTransitionStyleCrossDissolve];
                [self presentViewController:moviePlayerController animated:TRUE completion:^{}];
            }
            
            //bail
            return;
            
        }//end if video
        
        //BT_screen_launchNativeApp
        if([[theScreenData itemType] isEqualToString:@"BT_screen_launchNativeApp"] ||
           [[theScreenData itemType] isEqualToString:@"BT_launchNativeApp"]){
            /*
             Launching native app requires an "appType" and a "dataURL"
             App Types:	browser, youTube, googleMaps, musicStore, appStore, mail, dialer, sms
             */
            NSString *appToLaunch = [BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"appToLaunch" defaultValue:@""];
            NSString *dataURL = [BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""];
            NSString *encodedURL = @"";
            NSString *alertTitle = @"";
            NSString *alertMessage = @"";
            if([dataURL length] > 1){
                encodedURL =  [dataURL stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
            }
            if([appToLaunch length] > 1 && [encodedURL length] > 3){
                
                //browser, musicStore, appStore
                if([appToLaunch isEqualToString:@"browser"] || [appToLaunch isEqualToString:@"googleMaps"]
                   || [appToLaunch isEqualToString:@"musicStore"] || [appToLaunch isEqualToString:@"appStore"]
                   || [appToLaunch isEqualToString:@"youTube"]){
                    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:encodedURL]];
                }
                
                //google maps
                if([appToLaunch isEqualToString:@"googleMaps"]){
                    NSString *toAddress = [NSString stringWithFormat:@"http://maps.google.com/maps?q=%@", encodedURL];
                    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:toAddress]];
                }
                
                //mail
                if([appToLaunch isEqualToString:@"mail"]){
                    if([appDelegate.rootDevice canSendEmails]){
                        NSString *emailAddress = [NSString stringWithFormat:@"mailto:%@", encodedURL];
                        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:emailAddress]];
                    }else{
                        alertTitle = NSLocalizedString(@"emailsNotSupportedTitle", "Email Not Supported");
                        alertMessage = NSLocalizedString(@"emailsNotSupportedMessage", "Sending emails is not supported on this device");
                    }
                }
                
                //dialer
                if([appToLaunch isEqualToString:@"dialer"]){
                    if([appDelegate.rootDevice canMakePhoneCalls]){
                        NSString *phoneNumber = [NSString stringWithFormat:@"tel://%@", encodedURL];
                        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:phoneNumber]];
                    }else{
                        alertTitle = NSLocalizedString(@"callsNotSupportedTitle", "Calls Not Supported");
                        alertMessage = NSLocalizedString(@"emailsNotSupportedTitle", "Placing calls is not supported on this device");
                    }
                }
                
                //sms
                if([appToLaunch isEqualToString:@"sms"]){
                    if([appDelegate.rootDevice canSendSMS]){
                        NSString *smsAddress = [NSString stringWithFormat:@"sms:%@", encodedURL];
                        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:smsAddress]];
                    }else{
                        alertTitle = NSLocalizedString(@"textMessageNotSupportedTitle", "SMS Not Supported");
                        alertMessage = NSLocalizedString(@"textMessageNotSupportedMessage", "Sending SMS / Text messages is not supported on this device");
                    }
                }
                
                
                //book store URL
                if([appToLaunch isEqualToString:@"bookStore"]){
                    NSString *iBooksAddress = [NSString stringWithFormat:@"itms-books:%@", encodedURL];
                    if([[UIApplication sharedApplication] canOpenURL:[NSURL URLWithString:iBooksAddress]]) {
                        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:iBooksAddress]];
                    }else{
                        alertTitle = NSLocalizedString(@"customURLSchemeNotSupported", "Can't Open Application");
                        alertMessage = NSLocalizedString(@"customURLSchemeNotSupportedMessage", "This device cannot open the application with this URL Scheme");
                    }
                }
                
                
                //customURLScheme
                if([appToLaunch isEqualToString:@"customURLScheme"]){
                    if([[UIApplication sharedApplication] canOpenURL:[NSURL URLWithString:encodedURL]]) {
                        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:encodedURL]];
                    }else{
                        alertTitle = NSLocalizedString(@"customURLSchemeNotSupported", "Can't Open Application");
                        alertMessage = NSLocalizedString(@"customURLSchemeNotSupportedMessage", "This device cannot open the application with this URL Scheme");
                    }
                }
                
                //show alert?
                if([alertMessage length] > 3){
                    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:alertTitle
                                                                        message:alertMessage
                                                                       delegate:nil
                                                              cancelButtonTitle:NSLocalizedString(@"ok", "OK")
                                                              otherButtonTitles:nil];
                    [alertView setTag:105];
                    [alertView show];
                }
                
                
            }//dataURL, encodedURL length
            
            //bail
            return;
            
        }//end if launching native app
        
        
        ////////////////////////////////////////////////////////////////////////
        //if we are here, we are loading a new screen object
        UIViewController *theNextViewController = [appDelegate.rootApp getViewControllerForScreen:theScreenData];
        if(theNextViewController != nil){
            
            //always hide the lower tab-bar when screens transition in unless it's overridden
            BOOL hideBottomBar = FALSE;
            if([[BT_strings getJsonPropertyValue:appDelegate.rootApp.rootTheme.jsonVars nameOfProperty:@"hideBottomTabBarWhenScreenLoads" defaultValue:@""] isEqualToString:@"1"]){
                hideBottomBar = TRUE;
            }
            if([[BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"hideBottomTabBarWhenScreenLoads" defaultValue:@""] length] > 0){
                if([[BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"hideBottomTabBarWhenScreenLoads" defaultValue:@"1"] isEqualToString:@"0"]){
                    hideBottomBar = FALSE;
                }else{
                    hideBottomBar = TRUE;
                }
            }
            
            //always hide the bottom tab-bar for quiz screens
            if([[theScreenData itemType] isEqualToString:@"BT_screen_quiz"]){
                hideBottomBar = TRUE;
            }
            
            //always hide the "back" button...
            [theNextViewController.navigationItem setHidesBackButton:TRUE];
            
            //hide bottom bar if needed
            [theNextViewController setHidesBottomBarWhenPushed:hideBottomBar];
            
            //push view...
            [[self getNavController] pushViewController:theNextViewController animated:TRUE];
            
        }//theNextViewController...
    }//allowNextScreen
}





//placeCallWithScreenData...
-(void)placeCallWithScreenData:(BT_item *)theScreenData{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"placeCallWithScreenData (BT_viewController). Nickname: \"%@\" itemId: %@ itemType: %@", [theScreenData itemNickname], [theScreenData itemId], [theScreenData itemType]]];
    
    //WE WILL NOT BE HERE IF THE DEVICE IS NOT CAPABLE OF MAKING CALLS
    
    if([appDelegate.rootDevice canMakePhoneCalls] == TRUE){
        
        
        /*
         IMPORTANT. Numbers should not contain parenthesis, dashes are better.
         Exmaple: 123-123-1234 is better than (123)123-1234
         Not sure why but crazy results happen sometimes?
         */
        
        NSString *numberToCall = [NSString stringWithFormat:@"tel:%@", [BT_strings getJsonPropertyValue:theScreenData.jsonVars nameOfProperty:@"number" defaultValue:@""]];
        if([numberToCall length] > 5){
            
#if TARGET_IPHONE_SIMULATOR
            
            //show alert
            UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"callsNotSupportedTitle", "Calls Not Supported")
                                                                message:NSLocalizedString(@"callsNotSupportedMessage", "Placing calls is not supported on this device")
                                                               delegate:nil
                                                      cancelButtonTitle:NSLocalizedString(@"ok", "OK")
                                                      otherButtonTitles:nil];
            [alertView show];
            
#else
            //not in simulator...
            
            
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"launching dialer (BT_viewController): %@", numberToCall]];
            if(![[UIApplication sharedApplication] openURL:[NSURL URLWithString:numberToCall]]){
                
                //show alert
                UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:nil
                                                                    message:NSLocalizedString(@"errorTitle", "~ Error ~")
                                                                   delegate:nil
                                                          cancelButtonTitle:NSLocalizedString(@"ok", "OK")
                                                          otherButtonTitles:nil];
                [alertView show];
                
            }
            
#endif
            
            
        }else{
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"Could not launch dialer, no phone number?%@", @""]];
            
        }
        
        
    }else{
        
        //show alert...
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"callsNotSupportedTitle", "Calls Not Supported")
                                                        message:NSLocalizedString(@"callsNotSupportedMessage", "You need to be using an iPhone to make phone calls")
                                                       delegate:self cancelButtonTitle:NSLocalizedString(@"ok", "OK") otherButtonTitles:nil];
        [alert show];
        
    }
}



//sendEmailWithScreenData...
-(void)sendEmailWithScreenData:(BT_item *)theScreenData imageAttachment:(UIImage *)imageAttachment imageAttachmentName:(NSString *)imageAttachmentName{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"sendEmailWithScreenData (BT_viewController) nickname: \"%@\" itemId: %@ itemType: %@", [theScreenData itemNickname], [theScreenData itemId], [theScreenData itemType]]];
    
    //mail composer
    Class mailClass = (NSClassFromString(@"MFMailComposeViewController"));
    if(mailClass != nil){
        if([mailClass canSendMail]){
            
            //ask the app's delegate for the current view controller...
            BT_viewController *theViewController = [appDelegate getViewController];
            
            //setup the built in compose sheet...
            MFMailComposeViewController *picker = [[MFMailComposeViewController alloc] init];
            picker.mailComposeDelegate = self;
            
            //get subject
            if([theScreenData.jsonVars objectForKey:@"emailSubject"]){
                [picker setSubject:[theScreenData.jsonVars objectForKey:@"emailSubject"]];
            }
            
            //get to address
            if([theScreenData.jsonVars objectForKey:@"emailToAddress"]){
                NSArray *toRecipients = [NSArray arrayWithObject:[theScreenData.jsonVars objectForKey:@"emailToAddress"]];
                [picker setToRecipients:toRecipients];
            }
            
            //attach image if included
            if(imageAttachment != nil){
                NSData *imageData = UIImageJPEGRepresentation(imageAttachment, 1.0);
                if(imageData){
                    [picker addAttachmentData:imageData mimeType:@"image/jpeg" fileName:imageAttachmentName];
                }
            }
            
            
            //get body
            if([theScreenData.jsonVars objectForKey:@"emailMessage"]){
                
                NSString *emailMessage = [theScreenData.jsonVars objectForKey:@"emailMessage"];
                [picker setMessageBody:emailMessage isHTML:NO];
                
            }else{
                
                //if we have a subject set...
                NSString *emailSubject = @"";
                if([theScreenData.jsonVars objectForKey:@"emailSubject"]){
                    emailSubject = [theScreenData.jsonVars objectForKey:@"emailSubject"];
                }
                
                //empty message body or the imageTitle of an image we are emailing (if it has an imageTitle)..
                NSString *imageTitle = [BT_strings getPrefString:@"emailImageTitle"];
                if([emailSubject length] < 1){
                    [picker setSubject:imageTitle];
                }
                
                //erase emailImageTitle for next time...
                [BT_strings setPrefString:@"emailImageTitle" valueOfPref:@""];
                
            }
            
            //navigation bar tint color and text color...
            
            [[picker navigationBar] setBarTintColor:[BT_color getColorFromHexString:[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarBackgroundColor" defaultValue:@""]]];
            
            //show the model view...
            [theViewController presentViewController:picker animated:YES completion:nil];
            
            
        }//can send mail
    }
    
}


//sendEmailWithAttachmentFromScreenData...
-(void)sendEmailWithAttachmentFromScreenData:(BT_item *)theScreenData theAttachmentData:(NSData *)theAttachmentData attachmentName:(NSString *)attachmentName{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"sendEmailWithAttachmentFromScreenData (BT_viewController) nickname: \"%@\" itemId: %@ itemType: %@", [theScreenData itemNickname], [theScreenData itemId], [theScreenData itemType]]];
    
    //mail composer
    Class mailClass = (NSClassFromString(@"MFMailComposeViewController"));
    if(mailClass != nil){
        if([mailClass canSendMail]){
            
            //ask the app's delegate for the current view controller...
            BT_viewController *theViewController = [appDelegate getViewController];
            
            //setup the built in compose sheet...
            MFMailComposeViewController *picker = [[MFMailComposeViewController alloc] init];
            picker.mailComposeDelegate = self;
            
            //set possible subject
            if([theScreenData.jsonVars objectForKey:@"emailSubject"]){
                [picker setSubject:[theScreenData.jsonVars objectForKey:@"emailSubject"]];
            }
            
            //set possible to address
            if([theScreenData.jsonVars objectForKey:@"emailToAddress"]){
                NSArray *toRecipients = [NSArray arrayWithObject:[theScreenData.jsonVars objectForKey:@"emailToAddress"]];
                [picker setToRecipients:toRecipients];
            }
            
            //set possible email message
            if([theScreenData.jsonVars objectForKey:@"emailMessage"]){
                NSString *emailMessage = [theScreenData.jsonVars objectForKey:@"emailMessage"];
                [picker setMessageBody:emailMessage isHTML:NO];
            }
            
            
            //set possible attachment
            if(theAttachmentData != nil){
                [picker addAttachmentData:theAttachmentData mimeType:@"application/octet-stream" fileName:attachmentName];
            }
            
            //navigation bar tint color...
            [[picker navigationBar] setBarTintColor:[BT_color getColorFromHexString:[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarBackgroundColor" defaultValue:@""]]];
            
            //show the model view...
            [theViewController presentViewController:picker animated:YES completion:nil];
            
            
        }//can send mail
    }
    
}



//sendEmailFromWebLink... (triggered when link with "mailto" is clicked in a web-view)
-(void)sendEmailFromWebLink:(BT_item *)theScreenData toAddress:(NSString *)toAddress{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"sendEmailFromWebView (BT_viewController): %@", toAddress]];
    
    //mail composer
    Class mailClass = (NSClassFromString(@"MFMailComposeViewController"));
    if(mailClass != nil){
        if([mailClass canSendMail]){
            
            //ask the app's delegate for the current view controller...
            BT_viewController *theViewController = [appDelegate getViewController];
            
            //setup the built in compose sheet...
            MFMailComposeViewController *picker = [[MFMailComposeViewController alloc] init];
            picker.mailComposeDelegate = self;
            
            //set to address
            NSArray *toRecipients = [NSArray arrayWithObject:toAddress];
            [picker setToRecipients:toRecipients];
            
            //empty message body
            NSString *emailBody = @"";
            [picker setMessageBody:emailBody isHTML:NO];
            
            //navigation bar tint color...
            [[picker navigationBar] setBarTintColor:[BT_color getColorFromHexString:[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarBackgroundColor" defaultValue:@""]]];
            
            //show the model view...
            [theViewController presentViewController:picker animated:YES completion:nil];
            
            
        }//can send mail
    }
    
}



//sendTextMessageWithScreenData...
-(void)sendTextMessageWithScreenData:(BT_item *)theScreenData{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"sendTextMessageWithScreenData (BT_viewController) nickname: \"%@\" itemId: %@ itemType: %@", [theScreenData itemNickname], [theScreenData itemId], [theScreenData itemType]]];
    
    //SMS composer
    Class smsClass = (NSClassFromString(@"MFMessageComposeViewController"));
    if(smsClass != nil && [MFMessageComposeViewController canSendText]){
        
        MFMessageComposeViewController *picker = [[MFMessageComposeViewController alloc] init];
        if([MFMessageComposeViewController canSendText]){
            
            //ask the app's delegate for the current view controller...
            BT_viewController *theViewController = [appDelegate getViewController];
            
            //set delegate on the picker...
            picker.messageComposeDelegate = self;
            
            //set recipients
            if([theScreenData.jsonVars objectForKey:@"textToNumber"]){
                NSArray *toRecipients = [NSArray arrayWithObject:[theScreenData.jsonVars objectForKey:@"textToNumber"]];
                [picker setRecipients:toRecipients];
            }
            
            //set message body
            if([theScreenData.jsonVars objectForKey:@"textMessage"]){
                [picker setBody:[theScreenData.jsonVars objectForKey:@"textMessage"]];
            }
            
            //navigation bar tint color...
            [[picker navigationBar] setBarTintColor:[BT_color getColorFromHexString:[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarBackgroundColor" defaultValue:@""]]];
            
            //show the model view...
            [theViewController presentViewController:picker animated:YES completion:nil];
            
        }
    }
    
    
}



//mailComposeController...
-(void)mailComposeController:(MFMailComposeViewController*)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError*)error {
    [BT_debugger showIt:self message:@"mailComposeController (BT_viewController):didFinishComposingMail"];
    [self dismissViewControllerAnimated:TRUE completion:nil];
    
    //if this is an iPad AND our "currentScreenData" is BT_screen_imageEmail
    if([appDelegate.rootDevice isIPad]){
        if([[appDelegate.rootApp.currentScreenData.jsonVars objectForKey:@"itemType"] isEqualToString:@"BT_screen_imageEmail"]){
            
            UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:nil message:NSLocalizedString(@"emailImageDone", "Re-load this screen to re-start the process or to send another message") delegate:self
                                                      cancelButtonTitle:NSLocalizedString(@"ok", "OK") otherButtonTitles:nil];
            [alertView show];
            
        }
        
    }//is iPad
    
}


//messageComposeViewController...
-(void)messageComposeViewController:(MFMessageComposeViewController *)controller didFinishWithResult:(MessageComposeResult)result{
    [BT_debugger showIt:self message:@"messageComposeViewController (BT_viewController): didFinishComposingSMS"];
    [self dismissViewControllerAnimated:TRUE completion:nil];
    
}





/////////////////////////////////////////////////////////////////////
//rotation methods...
-(BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    //[BT_debugger showIt:self message:[NSString stringWithFormat:@"shouldAutorotateToInterfaceOrientation (BT_viewController) screen itemId: %@", [self.screenData itemId]]];
    
    //allow / dissallow rotations
    BOOL canRotate = TRUE;
    
    if([appDelegate.rootDevice isIPad]){
        canRotate = TRUE;
    }else{
        //should we prevent rotations on small devices?
        if([[appDelegate.rootApp allowRotation] isEqualToString:@"largeDevicesOnly"]){
            canRotate = FALSE;
        }
    }
    
    //can it rotate?
    if(canRotate){
        return YES;
    }else{
        return (interfaceOrientation == UIInterfaceOrientationPortrait);
    }
    
    //we should not get here
    return YES;
    
}



//will rotate
-(void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration{
    //[BT_debugger showIt:self message:[NSString stringWithFormat:@"willRotateToInterfaceOrientation (BT_viewController) screen itemId: %@", [self.screenData itemId]]];
    
    //some screens need to reload...
    UIViewController *theViewController;
    int selectedTab = 0;
    if([appDelegate.rootApp rootTabBarController] != nil){
        selectedTab = (int)[appDelegate.rootApp.rootTabBarController selectedIndex];
        theViewController = [[appDelegate.rootApp.rootTabBarController.viewControllers objectAtIndex:selectedTab] visibleViewController];
    }else{
        theViewController = [appDelegate.rootApp.rootNavController visibleViewController];
    }
    
    
    //if this view controller has a property named "rotating" set it to false...
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
    SEL selector = NSSelectorFromString(@"setIsRotating");
    if ([theViewController respondsToSelector:selector]) {
        [theViewController performSelector:selector];
    }
#pragma clang diagnostic pop
    
    
}


//did rotate
-(void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation{
    //[BT_debugger showIt:self message:[NSString stringWithFormat:@"didRotateFromInterfaceOrientation (BT_viewController) screen itemId: %@", [self.screenData itemId]]];
    
    //hide context menu...
    [appDelegate hideContextMenu];
    
    UIViewController *theViewController;
    int selectedTab = 0;
    if([appDelegate.rootApp rootTabBarController] != nil){
        selectedTab = (int)[appDelegate.rootApp.rootTabBarController selectedIndex];
        theViewController = [[appDelegate.rootApp.rootTabBarController.viewControllers objectAtIndex:selectedTab] visibleViewController];
    }else{
        theViewController = [appDelegate.rootApp.rootNavController visibleViewController];
    }
    
    //if this view controller has a property named "rotating" set it to false...
    SEL sel1 = NSSelectorFromString(@"rotating");
    if([theViewController respondsToSelector:sel1]) {
        [theViewController setValue:0 forKey:@"rotating"];
    }
    
    
    //some screens need to re-build their layout...If a plugin has a method called
    //"layoutScreen" we trigger it everytime the device rotates. The plugin author can
    //create this method in the UIViewController (layoutScreen) if they need something to
    //happen after rotation occurs.
    
    
    //if this view controller has a property named "rotating" set it to false...This for sure
    //is used in the Image Gallery plugin...
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
    SEL sel2 = NSSelectorFromString(@"setNotRotating");
    if ([theViewController respondsToSelector:sel2]) {
        [theViewController performSelector:sel2];
    }
#pragma clang diagnostic pop
    
    
    //some screens need to re-build their layout...If a plugin has a method called
    //"layoutScreen" we trigger it everytime the device rotates. The plugin author can
    //create this method in the UIViewController (layoutScreen) if they need something to
    //happen after rotation occurs.
    
    //if this view controller has a "layoutScreen" method, trigger it...
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
    SEL sel3 = NSSelectorFromString(@"layoutScreen");
    if([theViewController respondsToSelector:sel3]){
        [theViewController performSelector:sel3];
    }
#pragma clang diagnostic pop
    
    
}


//dealloc...
- (void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}




@end







