/*
 *	Copyright 2013, Chris Ruddell, www.churchphoneapps.com
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */

#import <UIKit/UIKit.h>
#import <Foundation/Foundation.h>
#import "JSON.h"
#import "BT_application.h"
#import "BT_strings.h"
#import "BT_viewUtilities.h"
#import "weightlosssolutions_appDelegate.h"
#import "BT_item.h"
#import "BT_debugger.h"
#import "BT_fileManager.h"
#import "BT_color.h"
#import "FF_crparsechannels.h"
#import <Parse/Parse.h>

@implementation FF_crparsechannels
@synthesize  selectorTable, parseChannels, saveAsFileName, downloader, didInit, isLoading, fontColor;

//viewDidLoad
-(void)viewDidLoad{
	[BT_debugger showIt:self theMessage:@"viewDidLoad"];
	[super viewDidLoad];
    
    //get fontColor
    fontColor = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"fontColor" defaultValue:@"#000000"];
    
    //////////////////////////////////
    //Add in header label
    UITextField *mainLabel = [[UITextField alloc]initWithFrame:CGRectMake(5, 5, self.view.frame.size.width - 10, 45)];
    mainLabel.textAlignment = NSTextAlignmentCenter;
    mainLabel.textColor = [BT_color getColorFromHexString:fontColor];
    
    NSString *mainTitle = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"mainTitle" defaultValue:@"Subscribe to Channels:"];
    mainLabel.text = mainTitle;
    [self.view addSubview:mainLabel];
    
    ////////////////////////////////////////////////////////////////////////////////////////
	//build the table that holds the menu items.
	self.selectorTable = [BT_viewUtilities getTableViewForScreen:[self screenData]];
    self.selectorTable.frame = CGRectMake(0, 55, self.view.frame.size.width, self.view.frame.size.height - 55);
	self.selectorTable.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	[self.selectorTable setDataSource:self];
	[self.selectorTable setDelegate:self];
    
    
    //prevent scrolling?
	if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"preventAllScrolling" defaultValue:@""] isEqualToString:@"1"]){
		[self.selectorTable setScrollEnabled:FALSE];
	}
	[self.view addSubview:selectorTable];
	
    
	//create adView?
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"includeAds" defaultValue:@"0"] isEqualToString:@"1"]){
	   	[self createAdBannerView];
	}

    

    
}

//view will appear
-(void)viewWillAppear:(BOOL)animated{
	[super viewWillAppear:animated];
	[BT_debugger showIt:self theMessage:@"viewWillAppear"];
	
	//flag this as the current screen
	weightlosssolutions_appDelegate *appDelegate = (weightlosssolutions_appDelegate *)[[UIApplication sharedApplication] delegate];
	appDelegate.rootApp.currentScreenData = self.screenData;
	
    //if we have not yet inited data..
	if(self.didInit == 0){
		[self performSelector:(@selector(loadData)) withObject:nil afterDelay:0.1];
		[self setDidInit:1];
	}
    else [self.selectorTable reloadData];
    
}

#pragma mark - Table view data source
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    // Return the number of rows in the section.
    // Usually the number of items in your array (the one that holds your list)
        return [parseChannels count];

}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell;
    //Where we configure the cell in each row

    NSArray *subscribedChannels = [PFInstallation currentInstallation].channels;
    
    UISwitch *switchView = [[UISwitch alloc] initWithFrame:CGRectZero];
    cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
        
        
        switchView.tag = indexPath.row;
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.accessoryView = switchView;

            
        
        [switchView addTarget:self action:@selector(switchChanged:indexPath:) forControlEvents:UIControlEventValueChanged];
    }
    
    BT_item *theChannel = [parseChannels objectAtIndex:indexPath.row];
    NSString *channel = [BT_strings getJsonPropertyValue:theChannel.jsonVars nameOfProperty:@"channelName" defaultValue:@""];
    channel = [channel stringByReplacingOccurrencesOfString:@" " withString:@""];
    channel = [channel stringByReplacingOccurrencesOfString:@"'" withString:@""];
    

    if([subscribedChannels containsObject:channel]){

        [switchView setOn:YES animated:NO];
    }else{

        [switchView setOn:NO animated:NO];
    }
    // Configure the cell... setting the text of our cell's label
    
    NSString *channelName = [BT_strings getJsonPropertyValue:theChannel.jsonVars nameOfProperty:@"channelName" defaultValue:@""];
    cell.textLabel.text = channelName;
    cell.textLabel.textColor = [BT_color getColorFromHexString:fontColor];
    
    
    return cell;
}

- (void) switchChanged:(id)sender indexPath:(int)indexPath {

    UISwitch* switchControl = sender;
    NSInteger row = switchControl.tag;

    NSLog( @"The %d switch is %@",row, switchControl.on ? @"ON" : @"OFF" );
    if(switchControl.on){
        [self subscribe:row];
    }else{
        [self unSubscribe:row];
    }
}

-(void)subscribe:(int)team{

    BT_item *theChannel = [parseChannels objectAtIndex:team];
    NSString *channel = [BT_strings getJsonPropertyValue:theChannel.jsonVars nameOfProperty:@"channelName" defaultValue:@""];
    
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"subscribing to:%@", channel]];

    
    PFInstallation *currentInstallation = [PFInstallation currentInstallation];
    [currentInstallation addUniqueObject:channel forKey:@"channels"];
    [currentInstallation saveInBackground];
}
-(void)unSubscribe:(int)team{
    BT_item *theChannel = [parseChannels objectAtIndex:team];
    NSString *channel = [BT_strings getJsonPropertyValue:theChannel.jsonVars nameOfProperty:@"channelName" defaultValue:@""];
    
    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"unsubscribing from:%@", channel]];
    
    PFInstallation *currentInstallation = [PFInstallation currentInstallation];
    [currentInstallation removeObject:channel forKey:@"channels"];
    [currentInstallation saveInBackground];
}


//allows us to check to see if we pulled-down to refresh
-(void)scrollViewWillBeginDecelerating:(UIScrollView *)scrollView{
	[self checkIsLoading];
}
-(void)checkIsLoading{
	if(isLoading){
		return;
	}else{
		//how far down did we pull?
		double down = selectorTable.contentOffset.y;
		if(down <= -65){
			if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@"1"] length] > 3){
				[self downloadData];
			}
		}
	}
}




//load data
-(void)loadData{
	[BT_debugger showIt:self theMessage:@"loadData"];
	self.isLoading = TRUE;
	
	//prevent interaction during operation
	[selectorTable setScrollEnabled:FALSE];
	[selectorTable setAllowsSelection:FALSE];
    
	/*
     Screen Data scenarios
     --------------------------------
     a)	No dataURL is provided in the screen data - use the info configured in the app's configuration file
     b)	A dataURL is provided, download now if we don't have a cache, else, download on refresh.
     */
	
	self.saveAsFileName = [NSString stringWithFormat:@"screenData_%@.txt", [self.screenData itemId]];
	
	//do we have a URL?
	BOOL haveURL = FALSE;
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""] length] > 10){
		haveURL = TRUE;
	}
	
	//start by filling the list from the configuration file, use these if we can't get anything from a URL
	if([[self.screenData jsonVars] objectForKey:@"childItems"]){
        
		//init the items array
		self.parseChannels = [[NSMutableArray alloc] init];
        
		NSArray *tmpMenuItems = [[self.screenData jsonVars] objectForKey:@"childItems"];
		for(NSDictionary *tmpMenuItem in tmpMenuItems){
			BT_item *thisMenuItem = [[BT_item alloc] init];
			thisMenuItem.itemId = [tmpMenuItem objectForKey:@"itemId"];
			thisMenuItem.itemType = [tmpMenuItem objectForKey:@"itemType"];
			thisMenuItem.jsonVars = tmpMenuItem;
			[self.parseChannels addObject:thisMenuItem];
		}
        
	}
	
	//if we have a URL, fetch..
	if(haveURL){
        
		//look for a previously cached version of this screens data...
		if([BT_fileManager doesLocalFileExist:[self saveAsFileName]]){
			[BT_debugger showIt:self theMessage:@"parsing cached version of screen data"];
			NSString *staleData = [BT_fileManager readTextFileFromCacheWithEncoding:[self saveAsFileName] encodingFlag:-1];
			[self parseScreenData:staleData];
		}else{
			[BT_debugger showIt:self theMessage:@"no cached version of this screens data available."];
			[self downloadData];
		}
        
        
	}else{
		
		//show the child items in the config data
		[BT_debugger showIt:self theMessage:@"using menu items from the screens configuration data."];
		[self layoutScreen];
		
	}
	
}

//download data
-(void)downloadData{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloading screen data from: %@", [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""]]];
	
	//flag this as the current screen
	weightlosssolutions_appDelegate *appDelegate = (weightlosssolutions_appDelegate *)[[UIApplication sharedApplication] delegate];
	appDelegate.rootApp.currentScreenData = self.screenData;
	
	//prevent interaction during operation
	[selectorTable setScrollEnabled:FALSE];
	[selectorTable setAllowsSelection:FALSE];
	
	//show progress
	[self showProgress];
	
	NSString *tmpURL = @"";
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""] length] > 3){
		
		//merge url variables
		tmpURL = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""];
        
		///merge possible variables in URL
		NSString *useURL = [BT_strings mergeBTVariablesInString:tmpURL];
		NSString *escapedUrl = [useURL stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        
		//fire downloader to fetch and results
		downloader = [[BT_downloader alloc] init];
		[downloader setSaveAsFileName:[self saveAsFileName]];
		[downloader setSaveAsFileType:@"text"];
		[downloader setUrlString:escapedUrl];
		[downloader setDelegate:self];
		[downloader downloadFile];
	}
}

//parse screen data
-(void)parseScreenData:(NSString *)theData{
	[BT_debugger showIt:self theMessage:@"parseScreenData"];
	
	//prevent interaction during operation
	[selectorTable setScrollEnabled:FALSE];
	[selectorTable setAllowsSelection:FALSE];
	
	@try {
        
		//arrays for screenData
		self.parseChannels = [[NSMutableArray alloc] init];
        
		//create dictionary from the JSON string
		SBJsonParser *parser = [SBJsonParser new];
		id jsonData = [parser objectWithString:theData];
		
		
	   	if(!jsonData){
            
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"ERROR parsing JSON: %@", parser.errorTrace]];
			[self showAlert:NSLocalizedString(@"errorTitle",@"~ Error ~") theMessage:NSLocalizedString(@"appParseError", @"There was a problem parsing some configuration data. Please make sure that it is well-formed") alertTag:0];
			[BT_fileManager deleteFile:[self saveAsFileName]];
            
		}else{
            
			if([jsonData objectForKey:@"childItems"]){
				NSArray *tmpMenuItems = [jsonData objectForKey:@"childItems"];
				for(NSDictionary *tmpMenuItem in tmpMenuItems){
					BT_item *thisMenuItem = [[BT_item alloc] init];
					thisMenuItem.itemId = [tmpMenuItem objectForKey:@"itemId"];
					thisMenuItem.itemType = [tmpMenuItem objectForKey:@"itemType"];
					thisMenuItem.jsonVars = tmpMenuItem;
					[self.parseChannels addObject:thisMenuItem];
				}
			}
            
			//layout screen
			[self layoutScreen];
            
		}
		
	}@catch (NSException * e) {
		
		//delete bogus data, show alert
		[BT_fileManager deleteFile:[self saveAsFileName]];
		[self showAlert:NSLocalizedString(@"errorTitle",@"~ Error ~") theMessage:NSLocalizedString(@"appParseError", @"There was a problem parsing some configuration data. Please make sure that it is well-formed") alertTag:0];
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"error parsing screen data: %@", e]];
        
	}
	
}

//build screen
-(void)layoutScreen{
	[BT_debugger showIt:self theMessage:@"layoutScreen"];
    
	//if we did not have any menu items...
	if(self.parseChannels.count < 1){
        
		for(int i = 0; i < 5; i++){
            
			//create a menu item from the data
			BT_item *thisMenuItemData = [[BT_item alloc] init];
			[thisMenuItemData setJsonVars:nil];
			[thisMenuItemData setItemId:@""];
			[thisMenuItemData setItemType:@"BT_menuItem"];
			[self.parseChannels addObject:thisMenuItemData];
			
		}
		
		//show message
		//[self showAlert:nil:NSLocalizedString(@"noListItems",@"This menu has no list items?"):0];
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"noListItems",@"This menu has no list items?")]];
		
	}
    
	//enable interaction again (unless owner turned it off)
	if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"preventAllScrolling" defaultValue:@""] isEqualToString:@"1"]){
		[self.selectorTable setScrollEnabled:FALSE];
	}else{
		[selectorTable setScrollEnabled:TRUE];
	}
	[selectorTable setAllowsSelection:TRUE];
	
	//reload table
	[self.selectorTable reloadData];
	
	//flag done loading
	self.isLoading = FALSE;
	
    
}





@end







