/*
 *	Copyright 2013, Chris Ruddell
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	Neither the name of David Book, or buzztouch.com nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */

#import <UIKit/UIKit.h>
#import <Foundation/Foundation.h>
#import "BT_fileManager.h"
#import "BT_color.h"
#import "BT_application.h"
#import "BT_strings.h"
#import "BT_viewUtilities.h"
#import "BT_downloader.h"
#import "BT_item.h"
#import "BT_debugger.h"
#import "Cr_menu_advanced.h"
#import "CR_menuAdvanced_cell.h"


@implementation Cr_menu_advanced
@synthesize menuItems, myTableView, headerImage;
@synthesize saveAsFileName, downloader, isLoading, didInit;
@synthesize imageHeight, imageWidth, tableHeight;
@synthesize imageFileName, imageURL, isIpad;


//viewDidLoad
-(void)viewDidLoad{
	[BT_debugger showIt:self theMessage:@"viewDidLoad"];
	[super viewDidLoad];
    
	//init screen properties
	[self setDidInit:0];
	
	//flag not loading
	[self setIsLoading:FALSE];
    
	////////////////////////////////////////////////////////////////////////////////////////
	//build the table that holds the menu items.
	self.myTableView = [Cr_menu_advanced getTableViewForScreen:[self screenData]];
	self.myTableView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	[self.myTableView setDataSource:self];
	[self.myTableView setDelegate:self];
	
	//prevent scrolling?
	if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"preventAllScrolling" defaultValue:@""] isEqualToString:@"1"]){
		[self.myTableView setScrollEnabled:FALSE];
	}
    
    
    //check to see if iPad or which size iPhone and create different frame sizes if that is the case.
    
    if (appDelegate.rootDevice.isIPad) self.isIpad = true;
    else self.isIpad = false;
    
    //if navBar is hidden, move down 20 (only iOS7)
    CGFloat theTop = 0;
    //if iOS7 or greater
    float ver = [[[UIDevice currentDevice] systemVersion] floatValue];
    if (ver >= 7.0) {
        if ([[[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@""]uppercaseString]isEqualToString:@"HIDDEN"]) theTop = 20;
    }
    
    NSString *hasImage = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"hasImage" defaultValue:@"YES"];
    if ([hasImage isEqualToString: @"YES"]){
        if([appDelegate.rootDevice isIPad]){
            imageHeight = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"imageHeightIPad" defaultValue:@"360"] intValue];
            imageWidth = 768;
            imageHeight = (int)(imageHeight/768.f * [UIScreen mainScreen].bounds.size.width);
            tableHeight = 1024 - imageHeight;
        }else {
            
            
            
            imageHeight = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"imageHeight" defaultValue:@"150"] intValue];
            imageWidth = [UIScreen mainScreen].bounds.size.width;
            imageHeight = (int)(imageHeight/320.f * [UIScreen mainScreen].bounds.size.width);
            if([UIScreen mainScreen].bounds.size.height == 568)
            {
                tableHeight = 568 -imageHeight;
            }else{
                tableHeight = 480 -imageHeight;
            }
        }
        
    }
    else {
        //header image set to no
        imageHeight = 0;
        imageWidth = [UIScreen mainScreen].bounds.size.width;
        tableHeight = [UIScreen mainScreen].bounds.size.height -imageHeight;
    }
    
    //needs to take background color from the BT_Config file?
    
    if ([hasImage isEqualToString: @"YES"]){
        //generate the frame and image for the header.
        headerImage = [[UIImageView alloc] initWithFrame:CGRectMake(0,theTop,imageWidth,imageHeight)];
        [headerImage setContentMode:UIViewContentModeScaleAspectFit];
        headerImage.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin |UIViewAutoresizingFlexibleRightMargin);
        UIImage *tmpImage = nil;
        imageFileName = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"imageFileName" defaultValue:@""];
        imageURL = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"imageURL" defaultValue:@""];
        
        
        //decided whether to get the image from project or one from a URL.  Default is from file if both possible.
        if([imageFileName length] > 0){
            //is it in the project?...
            if([BT_fileManager doesFileExistInBundle:imageFileName]){
                tmpImage = [UIImage imageNamed:imageFileName];
            }
        }else{
            if([imageURL length] > 0){
                NSData *imageData = [[NSData alloc] initWithContentsOfURL:[NSURL URLWithString:imageURL]];
                tmpImage =  [[UIImage alloc] initWithData:imageData];
            }
        }
        
        
        //sets the header image
        if(tmpImage != nil){
            [headerImage setImage:tmpImage];
            [self.view addSubview:headerImage];
        }
    }
    
    //generate the frame for tableview right below the image.
    [self.view addSubview:myTableView];
    [self.myTableView setFrame:CGRectMake(0, imageHeight + theTop, imageWidth, tableHeight)];
    
    myTableView.autoresizingMask = (UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth| UIViewAutoresizingFlexibleLeftMargin |UIViewAutoresizingFlexibleRightMargin);
    
    //handle clicks on the header image
    UITapGestureRecognizer *tapRecognizer = [[UITapGestureRecognizer alloc]
                                             initWithTarget:self action:@selector(ClickEventOnImage:)];
    //Don't forget to set the userInteractionEnabled to YES, by default It's NO.
    headerImage.userInteractionEnabled = YES;
    [headerImage addGestureRecognizer:tapRecognizer];
	
}

-(void) ClickEventOnImage:(id) sender
{
    NSLog(@"image clicked");
    
    //get possible itemId of the screen to load
    NSString *loadScreenItemId = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"headerScreenIDToLoad" defaultValue:@""];
    
    //get possible nickname of the screen to load
    NSString *loadScreenNickname = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"headerScreenNicknameToLoad" defaultValue:@""];
    
    //bail if load screen = "none"
    if([loadScreenItemId isEqualToString:@"none"]){
        return;
    }
    
    //check for loadScreenWithItemId THEN loadScreenWithNickname THEN loadScreenObject
    BT_item *screenObjectToLoad = nil;
    if([loadScreenItemId length] > 1){
        screenObjectToLoad = [appDelegate.rootApp getItemDataByItemId:loadScreenItemId];
    }else{
        if([loadScreenNickname length] > 1){
            screenObjectToLoad = [appDelegate.rootApp getItemDataByNickname:loadScreenNickname];
        }else{
            if([self.screenData.jsonVars objectForKey:@"loadScreenObject"]){
                screenObjectToLoad = [[BT_item alloc] init];
                [screenObjectToLoad setItemId:[[self.screenData.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemId"]];
                [screenObjectToLoad setItemNickname:[[self.screenData.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemNickname"]];
                [screenObjectToLoad setItemType:[[self.screenData.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemType"]];
                [screenObjectToLoad setJsonVars:[self.screenData.jsonVars objectForKey:@"loadScreenObject"]];
            }
        }
    }
    
    
    //load next screen if it's not nil
    if(screenObjectToLoad != nil){
        [self handleTapToLoadScreen:screenObjectToLoad theButtonData:self.screenData];
    }else{
        //show message
        [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"menuTapError",@"The application doesn't know how to handle this click?")]];
    }
    
}

//view will appear
-(void)viewWillAppear:(BOOL)animated{
	[super viewWillAppear:animated];
	[BT_debugger showIt:self theMessage:@"viewWillAppear"];
    
	//if we have not yet inited data..
	if(self.didInit == 0){
		[self performSelector:(@selector(loadData)) withObject:nil afterDelay:0.1];
		[self setDidInit:1];
	}
    
	
}


//load data
-(void)loadData{
	[BT_debugger showIt:self theMessage:@"loadData"];
	self.isLoading = TRUE;
	
	//prevent interaction during operation
	[myTableView setScrollEnabled:FALSE];
	[myTableView setAllowsSelection:FALSE];
    
	/*
     Screen Data scenarios
     --------------------------------
     a)	No dataURL is provided in the screen data - use the info configured in the app's configuration file
     b)	A dataURL is provided, download now if we don't have a cache, else, download on refresh.
     */
	
	self.saveAsFileName = [NSString stringWithFormat:@"screenData_%@.txt", [self.screenData itemId]];//original
    
    
	
	//do we have a URL?
	BOOL haveURL = FALSE;
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""] length] > 10){
		haveURL = TRUE;
	}
	
	//start by filling the list from the configuration file, use these if we can't get anything from a URL
	if([[self.screenData jsonVars] objectForKey:@"childItems"]){
        
		//init the items array
		self.menuItems = [[NSMutableArray alloc] init];
        
		NSArray *tmpMenuItems = [[self.screenData jsonVars] objectForKey:@"childItems"];
		for(NSDictionary *tmpMenuItem in tmpMenuItems){
            NSString *platform = [[tmpMenuItem objectForKey:@"platform"] uppercaseString];
            if (![platform isEqualToString:@"ANDROID"]) {
                BT_item *thisMenuItem = [[BT_item alloc] init];
                thisMenuItem.itemId = [tmpMenuItem objectForKey:@"itemId"];
                thisMenuItem.itemType = [tmpMenuItem objectForKey:@"itemType"];
                thisMenuItem.jsonVars = [tmpMenuItem mutableCopy];
                [self.menuItems addObject:thisMenuItem];
            }
		}
        
	}
	
	//if we have a URL, fetch..
	if(haveURL){
        
		//look for a previously cached version of this screens data...
		if([BT_fileManager doesLocalFileExist:[self saveAsFileName]]){
			[BT_debugger showIt:self theMessage:@"parsing cached version of screen data"];
			NSString *staleData = [BT_fileManager readTextFileFromCacheWithEncoding:[self saveAsFileName] encodingFlag:-1];
			[self parseScreenData:staleData];
		}else{
			[BT_debugger showIt:self theMessage:@"no cached version of this screens data available."];
			[self downloadData];
		}
        
        
	}else{
		
		//show the child items in the config data
		[BT_debugger showIt:self theMessage:@"using menu items from the screens configuration data."];
		[self layoutScreen];
		
	}
	
}

//download data
-(void)downloadData{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloading screen data from: %@", [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""]]];
	
	//flag this as the current screen
	appDelegate.rootApp.currentScreenData = self.screenData;
	
	//prevent interaction during operation
	[myTableView setScrollEnabled:FALSE];
	[myTableView setAllowsSelection:FALSE];
	
	//show progress
	[self showProgress];
	
	NSString *tmpURL = @"";
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""] length] > 3){
		
		//merge url variables
		tmpURL = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""];
        
		///merge possible variables in URL
		NSString *useURL = [BT_strings mergeBTVariablesInString:tmpURL];
		NSString *escapedUrl = [useURL stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        
		//fire downloader to fetch and results
		downloader = [[BT_downloader alloc] init];
		[downloader setSaveAsFileName:[self saveAsFileName]];
		[downloader setSaveAsFileType:@"text"];
		[downloader setUrlString:escapedUrl];
		[downloader setDelegate:self];
		[downloader downloadFile];
	}
}

-(id)convertStringToJson:(NSString*)jsonString {
    NSError *error2 = nil;
    NSData *data = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    id jsonData = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableContainers error:&error2];
    return jsonData;
}

//parse screen data
-(void)parseScreenData:(NSString *)theData{
	[BT_debugger showIt:self theMessage:@"parseScreenData"];
	
	//prevent interaction during operation
	[myTableView setScrollEnabled:FALSE];
	[myTableView setAllowsSelection:FALSE];
	
	@try {
        
		//arrays for screenData
		self.menuItems = [[NSMutableArray alloc] init];
        
		//create dictionary from the JSON string
        id jsonData = [self convertStringToJson:theData];
		
		
	   	if(!jsonData){
            
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"ERROR parsing JSON: %@", theData]];
			[self showAlert:NSLocalizedString(@"errorTitle",@"~ Error ~") theMessage:NSLocalizedString(@"appParseError", @"There was a problem parsing some configuration data. Please make sure that it is well-formed") alertTag:0];
			[BT_fileManager deleteFile:[self saveAsFileName]];
            
		}else{
            
			if([jsonData objectForKey:@"childItems"]){
				NSArray *tmpMenuItems = [jsonData objectForKey:@"childItems"];
				for(NSDictionary *tmpMenuItem in tmpMenuItems){
                    NSMutableDictionary *tmpMenuDictionary = [tmpMenuItem mutableCopy];
					NSString *platform = [[tmpMenuDictionary objectForKey:@"platform"] uppercaseString];
                    if (![platform isEqualToString:@"ANDROID"]) {
                        BT_item *thisMenuItem = [[BT_item alloc] init];
                        thisMenuItem.itemId = [tmpMenuDictionary objectForKey:@"itemId"];
                        thisMenuItem.itemType = [tmpMenuDictionary objectForKey:@"itemType"];
                        thisMenuItem.jsonVars = tmpMenuDictionary;
                        [self.menuItems addObject:thisMenuItem];
                    }
				}
			}
            
			//layout screen
			[self layoutScreen];
            
		}
		
	}@catch (NSException * e) {
		
		//delete bogus data, show alert
		[BT_fileManager deleteFile:[self saveAsFileName]];
		[self showAlert:NSLocalizedString(@"errorTitle",@"~ Error ~") theMessage:NSLocalizedString(@"appParseError", @"There was a problem parsing some configuration data. Please make sure that it is well-formed") alertTag:0];
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"error parsing screen data: %@", e]];
        
	}
	
}

//build screen
-(void)layoutScreen{
	[BT_debugger showIt:self theMessage:@"layoutScreen"];
    
	//if we did not have any menu items...
	if(self.menuItems.count < 1){
        
		for(int i = 0; i < 5; i++){
            
			//create a menu item from the data
			BT_item *thisMenuItemData = [[BT_item alloc] init];
			[thisMenuItemData setJsonVars:nil];
			[thisMenuItemData setItemId:@""];
			[thisMenuItemData setItemType:@"BT_menuItem"];
			[self.menuItems addObject:thisMenuItemData];
			
		}
		
		//show message
		//[self showAlert:nil:NSLocalizedString(@"noListItems",@"This menu has no list items?"):0];
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"noListItems",@"This menu has no list items?")]];
		
	}
    
	//enable interaction again (unless owner turned it off)
	if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"preventAllScrolling" defaultValue:@""] isEqualToString:@"1"]){
		[self.myTableView setScrollEnabled:FALSE];
	}else{
		[myTableView setScrollEnabled:TRUE];
	}
	[myTableView setAllowsSelection:TRUE];
	
	//reload table
	[self.myTableView reloadData];
	
	//flag done loading
	self.isLoading = FALSE;
	
    
}


//////////////////////////////////////////////////////////////
//UITableView delegate methods
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

// number of rows
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.menuItems count];
}

//height of row
-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    CGFloat rowHeight = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"listRowHeightSmallDevice" defaultValue:@"50"] floatValue];
    if (self.isIpad) rowHeight = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"listRowHeightLargeDevice" defaultValue:@"50"] floatValue];
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"heightForRowAtIndexPath=%f", rowHeight]];
    return rowHeight;
}

//table view cells
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	NSString *CellIdentifier = [NSString stringWithFormat:@"cell_%i", (int)indexPath.row];
	CR_menuAdvanced_cell *cell = (CR_menuAdvanced_cell *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil){
        
		//init our custom cell
		cell = [[CR_menuAdvanced_cell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
		cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
		
	}
    
	//this menu item
	BT_item *thisMenuItemData = [self.menuItems objectAtIndex:indexPath.row];
	[cell setTheMenuItemData:thisMenuItemData];
	[cell setTheParentMenuScreenData:[self screenData]];
	[cell configureCell];
	
	
	//custom background view. Must be done here so we can retain the "round" corners if this is a round table
	//this method refers to this screen's "listRowBackgroundColor" and it's position in the tap. Top and
	//bottom rows may need to be rounded if this is screen uses "listStyle":"round"
    cell.backgroundColor = [UIColor clearColor];
    
	[cell setBackgroundView:[Cr_menu_advanced getCellBackgroundForListRow:[self screenData] theIndexPath:indexPath numRows:(int)[self.menuItems count]]];
	
	//return
	return cell;
    
}

//on row select
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"didSelectRowAtIndexPath: Selected Row: %li", (long)indexPath.row]];
	
	//pass this menu item to the tapForMenuItem method
	BT_item *thisMenuItem = [self.menuItems objectAtIndex:indexPath.row];
	if([thisMenuItem jsonVars] != nil){
        
        
		//get possible itemId of the screen to load
		NSString *loadScreenItemId = [BT_strings getJsonPropertyValue:thisMenuItem.jsonVars nameOfProperty:@"loadScreenWithItemId" defaultValue:@""];
		
		//get possible nickname of the screen to load
		NSString *loadScreenNickname = [BT_strings getJsonPropertyValue:thisMenuItem.jsonVars nameOfProperty:@"loadScreenWithNickname" defaultValue:@""];
        
		//bail if load screen = "none"
		if([loadScreenItemId isEqualToString:@"none"]){
			return;
		}
		
		//check for loadScreenWithItemId THEN loadScreenWithNickname THEN loadScreenObject
		BT_item *screenObjectToLoad = nil;
		if([loadScreenItemId length] > 1){
			screenObjectToLoad = [appDelegate.rootApp getItemDataByItemId:loadScreenItemId];
		}else{
			if([loadScreenNickname length] > 1){
				screenObjectToLoad = [appDelegate.rootApp getItemDataByNickname:loadScreenNickname];
			}else{
				if([thisMenuItem.jsonVars objectForKey:@"loadScreenObject"]){
					screenObjectToLoad = [[BT_item alloc] init];
					[screenObjectToLoad setItemId:[[thisMenuItem.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemId"]];
					[screenObjectToLoad setItemNickname:[[thisMenuItem.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemNickname"]];
					[screenObjectToLoad setItemType:[[thisMenuItem.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemType"]];
					[screenObjectToLoad setJsonVars:[thisMenuItem.jsonVars objectForKey:@"loadScreenObject"]];
				}
			}
		}
        
		
		//load next screen if it's not nil
		if(screenObjectToLoad != nil){
            [self handleTapToLoadScreen:screenObjectToLoad theButtonData:thisMenuItem];

		}else{
			//show message
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"menuTapError",@"The application doesn't know how to handle this click?")]];
		}
		
	}else{
        
		//show message
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"menuTapError",@"The application doesn't know how to handle this action?")]];
        
	}
	
}

//on accessory view tap (details arrow tap)
- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"accessoryButtonTappedForRowWithIndexPath: Selected Row: %i", (int)indexPath.row]];
    
	//pass this menu item to the tapForMenuItem method
	BT_item *thisMenuItem = [self.menuItems objectAtIndex:indexPath.row];
	if([thisMenuItem jsonVars] != nil){
        
		//get possible itemId of the screen to load
		NSString *loadScreenItemId = [BT_strings getJsonPropertyValue:thisMenuItem.jsonVars nameOfProperty:@"loadScreenWithItemId" defaultValue:@""];
		
		//get possible nickname of the screen to load
		NSString *loadScreenNickname = [BT_strings getJsonPropertyValue:thisMenuItem.jsonVars nameOfProperty:@"loadScreenWithNickname" defaultValue:@""];
        
		//bail if load screen = "none"
		if([loadScreenItemId isEqualToString:@"none"]){
			return;
		}
		
		//check for loadScreenWithItemId THEN loadScreenWithNickname THEN loadScreenObject
		BT_item *screenObjectToLoad = nil;
		if([loadScreenItemId length] > 1){
			screenObjectToLoad = [appDelegate.rootApp getItemDataByItemId:loadScreenItemId];
		}else{
			if([loadScreenNickname length] > 1){
				screenObjectToLoad = [appDelegate.rootApp getItemDataByNickname:loadScreenNickname];
			}else{
				if([thisMenuItem.jsonVars objectForKey:@"loadScreenObject"]){
					screenObjectToLoad = [[BT_item alloc] init];
					[screenObjectToLoad setItemId:[[thisMenuItem.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemId"]];
					[screenObjectToLoad setItemNickname:[[thisMenuItem.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemNickname"]];
					[screenObjectToLoad setItemType:[[thisMenuItem.jsonVars objectForKey:@"loadScreenObject"] objectForKey:@"itemType"]];
					[screenObjectToLoad setJsonVars:[thisMenuItem.jsonVars objectForKey:@"loadScreenObject"]];
				}
			}
		}
        
		
		//load next screen if it's not nil
		if(screenObjectToLoad != nil){
            [self handleTapToLoadScreen:screenObjectToLoad theButtonData:thisMenuItem];

		}else{
			//show error alert
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"menuTapError",@"The application doesn't know how to handle this action?")]];
		}
		
	}else{
		//show error alert
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"menuTapError",@"The application doesn't know how to handle this action?")]];
	}
    
}





//////////////////////////////////////////////////////////////////////////////////////////////////
//downloader delegate methods
-(void)downloadFileStarted:(NSString *)message{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloadFileStarted: %@", message]];
}
-(void)downloadFileInProgress:(NSString *)message{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloadFileInProgress: %@", message]];
	
}

-(void)downloadFileCompleted:(NSString *)message{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloadFileCompleted: %@", message]];
	[self hideProgress];
	
	//if message contains "error", look for previously cached data...
	if([message rangeOfString:@"ERROR-1968" options:NSCaseInsensitiveSearch].location != NSNotFound){
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"download error: There was a problem downloading data from the internet.%@", message]];
		//NSLog(@"Message: %@", message);
		
		//show alert
		[self showAlert:nil theMessage:NSLocalizedString(@"downloadError", @"There was a problem downloading some data. Check your internet connection then try again.") alertTag:0];
        
		//show local data if it exists
		if([BT_fileManager doesLocalFileExist:[self saveAsFileName]]){
            
			//use stale data if we have it
			NSString *staleData = [BT_fileManager readTextFileFromCacheWithEncoding:self.saveAsFileName encodingFlag:-1];
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"building screen from stale configuration data: %@", [self saveAsFileName]]];
			[self parseScreenData:staleData];
			
		}else{
            
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"There is no local data availalbe for this screen?%@", @""]];
			
			//if we have items... else.. show alert
			if(self.menuItems.count > 0){
				[self layoutScreen];
			}
			
		}
		
		
	}else{
        
		//parse previously saved data
		if([BT_fileManager doesLocalFileExist:[self saveAsFileName]]){
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"parsing downloaded screen data.%@", @""]];
			NSString *downloadedData = [BT_fileManager readTextFileFromCacheWithEncoding:[self saveAsFileName] encodingFlag:-1];
			[self parseScreenData:downloadedData];
            
		}else{
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"Error caching downloaded file: %@", [self saveAsFileName]]];
			[self layoutScreen];
            
			//show alert
			[self showAlert:nil theMessage:NSLocalizedString(@"appDownloadError", @"There was a problem saving some data downloaded from the internet.") alertTag:0];
            
		}
		
	}
    
}

//allows us to check to see if we pulled-down to refresh
-(void)scrollViewWillBeginDecelerating:(UIScrollView *)scrollView{
	[self checkIsLoading];
}
-(void)checkIsLoading{
	if(isLoading){
		return;
	}else{
		//how far down did we pull?
		double down = myTableView.contentOffset.y;
		if(down <= -65){
			if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@"1"] length] > 3){
				[self downloadData];
			}
		}
	}
}



/*
 this method returns a UITable view for a screen. It uses the screen data to configure options such as
 style, header/footer height, colors, background if the global theme values are over-ridden in the screen data.
 Note: lists contain cells, cells also have styles applied. This can get confusing when working with
 backgrounds and colors.
 */
+(UITableView *)getTableViewForScreen:(BT_item *)theScreenData{
    if([[theScreenData.jsonVars objectForKey:@"itemNickname"] length] > 1){
        [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"getTableViewForScreen with nickname: \"%@\" and itemId: %@ and type: %@", [theScreenData.jsonVars objectForKey:@"itemNickname"], [theScreenData.jsonVars objectForKey:@"itemId"], [theScreenData.jsonVars objectForKey:@"itemType"]]];
    }else{
        [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"getTableViewForScreen with nickname: \"%@\" and itemId: %@ and type: %@", @"no nickname?", [theScreenData.jsonVars objectForKey:@"itemId"], [theScreenData.jsonVars objectForKey:@"itemType"]]];
    }
    
    //appDelegate
    
    //assume we are building a standard, "square" table
    UITableViewStyle tmpTableStyle = UITableViewStylePlain;
    
    //if the global theme or the screen data want a round style table
    if([[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listStyle" defaultValue:@""] isEqualToString:@"round"]){
        tmpTableStyle = UITableViewStyleGrouped;
    }
    
    //default values, may be over-ridden in global theme or screen data
    int tableRowHeight = 50;
    int tableHeaderHeight = 10;
    int tableFooterHeight = 50;
    UIColor *tableBackgroundColor = [UIColor whiteColor];
    
    //table background color
    tableBackgroundColor = [BT_color getColorFromHexString:[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listBackgroundColor" defaultValue:@"clear"]];
    
    /*
     Some styles depend on the device. Use global theme settings first, then screen-data if over-ridden
     */
    if([appDelegate.rootDevice isIPad]){
        
        //use large device settings
        tableRowHeight = [[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listRowHeightLargeDevice" defaultValue:@"50"] intValue];
        tableHeaderHeight = [[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listHeaderHeightLargeDevice" defaultValue:@"10"] intValue];
        tableFooterHeight = [[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listFooterHeightLargeDevice" defaultValue:@"50"] intValue];
        
    }else{
        
        //use small device settings
        tableRowHeight = [[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listRowHeightSmallDevice" defaultValue:@"50"] intValue];
        tableHeaderHeight = [[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listHeaderHeightSmallDevice" defaultValue:@"10"] intValue];
        tableFooterHeight = [[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listFooterHeightSmallDevice" defaultValue:@"50"] intValue];
        
    }
    
    UITableView *tmpTable = [[UITableView alloc] initWithFrame:[[UIScreen mainScreen] bounds] style:tmpTableStyle];
    [tmpTable setRowHeight:tableRowHeight];
    [tmpTable setBackgroundColor:tableBackgroundColor];
    
    //this is a hack because iPad does not recognize UITableView background color...
    if([tmpTable respondsToSelector:@selector(backgroundView)]){
        tmpTable.backgroundView = nil;
    }
    
    [tmpTable setSectionHeaderHeight:tableHeaderHeight];
    [tmpTable setSectionFooterHeight:tableFooterHeight];
    [tmpTable setShowsVerticalScrollIndicator:FALSE];
    [tmpTable setShowsHorizontalScrollIndicator:FALSE];
    tmpTable.autoresizingMask = (UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth);
    
    //separator color is set to clear here. BT_viewUtilities.getCellBackgroundForListRow handles the separator color
    [tmpTable setSeparatorStyle:UITableViewCellSeparatorStyleNone];
    
    //we may want to prevent scrolling. This is useful if a header or footer image is used.
    if([[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listPreventScroll" defaultValue:@"0"] isEqualToString:@"1"]){
        [tmpTable setScrollEnabled:FALSE];
        
    }else{
        
        //if we are not preventing scrolling, add a table footer view so the user can always scroll to the
        //last item. This is helpful in tabbed apps where the last item doesn't quite scroll up high enough.
        UIView *tmpFooterView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 50, 50)];
        [tmpFooterView setBackgroundColor:[UIColor clearColor]];
        [tmpTable setTableFooterView:tmpFooterView];
        
    }
    
    //return
    return tmpTable;
    
}

/*
 This method returns a custom view for a table view cell's background. It's the only way to add a background
 color without cutting off the rounded corners on a "round" style table.
 */
+(UIView *)getCellBackgroundForListRow:(BT_item *)theScreenData theIndexPath:(NSIndexPath *)theIndexPath numRows:(int)numRows{
    //[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"getCellBackgroundForListRow for screen with itemId: %@", [theScreenData itemId]]];
    
    CR_cell_backgroundView *bgView = [[CR_cell_backgroundView alloc] initWithFrame:CGRectZero];
    UIColor *borderColor = [UIColor grayColor];
    UIColor *backgroundColor = [UIColor clearColor];
    BOOL isRoundTable = FALSE;
    
    //if the global theme or the screen data want a round style table
    if([[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listStyle" defaultValue:@""] isEqualToString:@"round"]){
        isRoundTable = TRUE;
    }
    
    //cell background color
    if([[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listRowBackgroundColor" defaultValue:@""] length] > 0){
        backgroundColor = [BT_color getColorFromHexString:[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listRowBackgroundColor" defaultValue:@""]];
    }
    
    //cell border color
    if([[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listRowSeparatorColor" defaultValue:@""] length] > 0){
        borderColor = [BT_color getColorFromHexString:[BT_strings getStyleValueForScreen:theScreenData nameOfProperty:@"listRowSeparatorColor" defaultValue:@""]];
    }
    
    //set colors
    [bgView setFillColor:backgroundColor];
    [bgView setBorderColor:borderColor];
    
    
    //position is important to maintain rounded corners if this is a "round" table.
    if(isRoundTable == FALSE){
       	bgView.position = CustomCellBackgroundViewPositionMiddle;
    }else{
        if(theIndexPath.row == 0){
            bgView.position = CustomCellBackgroundViewPositionTop;
        }else if(theIndexPath.row == numRows - 1){
            bgView.position = CustomCellBackgroundViewPositionBottom;
        }else{
            bgView.position = CustomCellBackgroundViewPositionMiddle;
        }
    }
    
    //return 
    return bgView;
    
}

@end

