/*
 *	Copyright, Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the
 *	following disclaimer.
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list
 *	of conditions and the following disclaimer in the documentation and/or other materials
 *	provided with the distribution.
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 *	OF SUCH DAMAGE.
 */

#import "BT_loading.h"

@implementation BT_loading

//viewDidLoad
-(void)viewDidLoad{
    [super viewDidLoad];
    [BT_debugger showIt:self theMessage:@"viewDidLoad"];
    
    //set background color for loading view...
    [self.view setBackgroundColor:[UIColor clearColor]];
    
    //add a spinner to the center of the view...
    UIActivityIndicatorView *spinner = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
    [spinner setCenter:self.view.center];
    [self.view addSubview:spinner];
    [spinner startAnimating];
    
}


//prefersStatusBarHidden...
-(BOOL)prefersStatusBarHidden{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"prefersStatusBarHidden%@", @""]];
    return YES;
}


//viewDidAppear...
-(void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    [BT_debugger showIt:self theMessage:@"viewDidAppear"];
    
    
    ////////////////////////////////////////////////
    //nullify core objects if they exist...
    if([appDelegate rootDatabase] != nil){
        [appDelegate setRootDatabase:nil];
    }
    if([appDelegate rootDevice] != nil){
        [appDelegate setRootDevice:nil];
    }
    if([appDelegate rootUser] != nil){
        [appDelegate setRootUser:nil];
    }
    if([appDelegate rootApp] != nil){
        [appDelegate setRootApp:nil];
    }
    if([appDelegate rootSoundEffectPlayer] != nil){
        [appDelegate setRootSoundEffectPlayer:nil];
    }
    if([appDelegate rootNetworkMonitor] != nil){
        [appDelegate setRootNetworkMonitor:nil];
    }
    if([appDelegate rootLocationManager] != nil){
        [appDelegate setRootLocationManager:nil];
    }
    if([appDelegate contextMenu] != nil){
        [appDelegate setContextMenu:nil];
    }
    
    
    ////////////////////////////////////////////////
    //init core objects...
    
    //init the root database...
    [appDelegate setRootDatabase:[[BT_database alloc] initWithDatabaseFilename:@"BT_database.sqlite"]];
    
    //init the root device...
    [appDelegate setRootDevice:[[BT_device alloc] init]];
    
    //init the root user...
    [appDelegate setRootUser:[[BT_user alloc] init]];
    
    //init the root app...
    [appDelegate setRootApp:[[BT_application alloc] init]];
    
    //start monitoring network connection...
    [appDelegate setRootNetworkMonitor:[BT_reachability reachabilityWithHostname:@"google.com"]];
    [appDelegate.rootNetworkMonitor setReachableOnWWAN:YES];
    
    //init the root locationMonitor, this may or may not track any locations...
    [appDelegate setRootLocationManager:[[BT_locationManager alloc] init]];
    
    //init the root soundEffectPlayer...
    [appDelegate setRootSoundEffectPlayer:[[BT_soundEffectPlayer alloc] init]];
    [appDelegate.rootSoundEffectPlayer loadSoundEffects];
    
    //load app data...
    [self loadAppData];
    
}

//loadAppData...
-(void)loadAppData{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadAppData%@", @""]];
    
    /*
        App Config data is stored in the local SqlLite database...scenarios...
        -----------------------------------------------------------------
        1)  We have data in the SqlLite database, on the device, use that. This is much
            faster and provides much better performance than parsing the potentially very large
            BT_config file included in the Xcode project everytime the app launches.
     
        2)  If we do not have any data in the local SqlLite database then we parse the BT_config
            file included in the Xcode project, save it in the database and move on.
     
        NOTE: If the app uses a dataURL it will NOT download data on the app's first launch. This 
        means you must be sure the BT_config data included in the Xcode project is the most up to date
        possible before distributing the app in an App Store.
     
    */
    
    //block safe variables used if we download data from a URL...
    __block NSString *jsonConfigInXcode = nil;
    __block BOOL didDownload = false;
    __block NSString *response = nil;
    __block NSString *useDataURL = nil;
    
    //do we have any data in the local SqlLite database?...
    BT_item *appCoreProperties = [appDelegate.rootDatabase getBtItemWithId:@"appCoreProperties"];
    if(appCoreProperties != nil){
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadAppData, parsing saved \"appCoreProperties\"%@", @""]];

        if([appCoreProperties.jsonVars objectForKey:@"dataURL"]){
            useDataURL = [appCoreProperties.jsonVars objectForKey:@"dataURL"];
        }
        
    }
    
    //read local data in Xcode project if we don't have a dataURL...
    if(appCoreProperties == nil || [useDataURL length] < 1){
        
        //load BT_config_en.txt OR a localized version...
        NSString *useConfigFileName = @"BT_config_en.txt";
        NSString *localizedConfigFileName = [NSString stringWithFormat:@"BT_config_%@.txt", [appDelegate.rootDevice deviceLangCode]];
        
        if([BT_fileManager doesFileExistInBundle:localizedConfigFileName]){
            useConfigFileName = localizedConfigFileName;
        }
        [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadAppData, FIRST LAUNCH, loading \"%@\" from Xcode project", useConfigFileName]];
        jsonConfigInXcode = [BT_fileManager readTextFileFromBundleWithEncoding:useConfigFileName encodingFlag:-1];
        
    }

    //process in block, may be long depending on download time, JSON size, etc...
    dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0ul);
    dispatch_async(queue, ^{
        
        //download if we have a URL...
        if([useDataURL length] > 1){

            //merge variables in useDataURL...
            useDataURL = [BT_strings mergeBTVariablesInString:useDataURL];
            
             //download...
            NSURL *escapedURL = [NSURL URLWithString:[useDataURL stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
            [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadAppData: Downloading from: %@", [escapedURL absoluteString]]];
            
            NSError* error;
            response = [NSString stringWithContentsOfURL:escapedURL encoding:NSASCIIStringEncoding error:&error];
            if(error != nil){
                [BT_debugger showIt:self message:[NSString stringWithFormat:@"loadAppData: ERROR downloading: %@", [error description]]];
            }else{
                didDownload = true;
            }
        
        }
        
        //back to main thread...
        dispatch_sync(dispatch_get_main_queue(), ^{
            
            //if we downloaded data
            if(didDownload && [response length] > 1){
                
                //update database with newly downloaded data...
                [appDelegate.rootDatabase updateConfigDataInDatabase:response];
            
            }else{
                
                //don't overwrite data if we tried to download but failed...
                if([useDataURL length] < 1){

                    //update database with newly downloaded data...
                    [appDelegate.rootDatabase updateConfigDataInDatabase:jsonConfigInXcode];
                
                }
                
            }
            
            //set the app's core properties (in memory properties, from appCoreProperties in database)...
            BT_item *appCoreProperties = [appDelegate.rootDatabase getBtItemWithId:@"appCoreProperties"];
            if(appCoreProperties != nil){
                [self setAppCorePropertiesFromBtItem:appCoreProperties];
            }
            
            //set the rootTheme if we have one..
            if([appDelegate.rootApp.themeItemId length] > 0){
                
                NSLog(@"HAVE ROOT THEME ID: %@", [appDelegate.rootApp themeItemId]);
                
                
                [appDelegate.rootApp setRootTheme:[appDelegate.rootApp getItemDataByItemId:appDelegate.rootApp.themeItemId]];
            }
            
            //promptForPushNotifications after a short delay (possible)...
            if([[appDelegate.rootApp promptForPushNotifications] isEqualToString:@"1"]){
                [appDelegate performSelector:NSSelectorFromString(@"promptforPushNotificationsAfterDelay") withObject:self afterDelay:3.0];
            }
            
            //promptForLocationAccessAfterDelay after a short delay (possible)...
            if([appDelegate.rootDevice canReportLocation]){
                if((int)[appDelegate.rootApp gpsUpdateIntervalSeconds] > 0 && appDelegate.rootLocationManager != nil){
                    [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"will prompt for location access after short delay%@",@""]];
                    [appDelegate performSelector:NSSelectorFromString(@"promptForLocationAccessAfterDelay") withObject:self afterDelay:3.0];
                }
            }
            
            //ask app to build it's interface...
            [appDelegate.rootApp buildInterface];
            
            //init context menu (now that we have a UI)...
            [appDelegate setContextMenu:[[BT_contextMenu alloc] initWithMenuData:nil]];
            [appDelegate.contextMenu.view setTag:888];
            
            //flag UI as visible...
            [appDelegate setUiIsVisible:TRUE];
            
            //fade out the current view (fadeOutStartView happens after this completes)...
            [self fadeOutLoadView];
            
            //reportToCloud (possible)...
            if([[appDelegate.rootApp reportToCloudURL] length] > 0){
                [appDelegate reportToCloud];
            }
            
            //flag as not refreshing anymore...
            [appDelegate setIsRefreshing:FALSE];
            
        });
        
    });
        
  
}



//setAppCorePropertiesFromBtItem...
-(void)setAppCorePropertiesFromBtItem:(BT_item *)propData{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"setAppCorePropertiesFromBtItem%@", @""]];
    
     //buzztouchAppId...
     if([propData.jsonVars objectForKey:@"buzztouchAppId"]) [appDelegate.rootApp setBuzztouchAppId:[propData.jsonVars objectForKey:@"buzztouchAppId"]];
    
     //buzztouchAPIKey...
     if([propData.jsonVars objectForKey:@"buzztouchAPIKey"]) [appDelegate.rootApp setBuzztouchAPIKey:[propData.jsonVars objectForKey:@"buzztouchAPIKey"]];
    
     //themeItemId...
     if([propData.jsonVars objectForKey:@"themeItemId"]) [appDelegate.rootApp setThemeItemId:[propData.jsonVars objectForKey:@"themeItemId"]];

     //homeScreenItemId...
     if([propData.jsonVars objectForKey:@"homeScreenItemId"]) [appDelegate.rootApp setHomeScreenItemId:[propData.jsonVars objectForKey:@"homeScreenItemId"]];

     //splashScreenItemId...
     if([propData.jsonVars objectForKey:@"splashScreenItemId"]) [appDelegate.rootApp setSplashScreenItemId:[propData.jsonVars objectForKey:@"splashScreenItemId"]];

     //registrationScreenItemId...
     if([propData.jsonVars objectForKey:@"registrationScreenItemId"]) [appDelegate.rootApp setRegistrationScreenItemId:[propData.jsonVars objectForKey:@"registrationScreenItemId"]];

     //registrationRequired...
     if([propData.jsonVars objectForKey:@"registrationRequired"]) [appDelegate.rootApp setRegistrationScreenItemId:[propData.jsonVars objectForKey:@"registrationRequired"]];
    
     //dataURL...
     if([propData.jsonVars objectForKey:@"dataURL"]) [appDelegate.rootApp setDataURL:[propData.jsonVars objectForKey:@"dataURL"]];
    
     //reportToCloudURL...
     if([propData.jsonVars objectForKey:@"reportToCloudURL"]) [appDelegate.rootApp setReportToCloudURL:[propData.jsonVars objectForKey:@"reportToCloudURL"]];

     //registerForPushURL...
     if([propData.jsonVars objectForKey:@"registerForPushURL"]) [appDelegate.rootApp setRegisterForPushURL:[propData.jsonVars objectForKey:@"registerForPushURL"]];
 
     //allowRotation...
     if([propData.jsonVars objectForKey:@"allowRotation"]) [appDelegate.rootApp setAllowRotation:[propData.jsonVars objectForKey:@"allowRotation"]];
    
     //gpsUpdateIntervalSeconds...
     if([propData.jsonVars objectForKey:@"gpsUpdateIntervalSeconds"]) [appDelegate.rootApp setGpsUpdateIntervalSeconds:[[propData.jsonVars objectForKey:@"gpsUpdateIntervalSeconds"] intValue]];

     //promptForPushNotifications...
     if([propData.jsonVars objectForKey:@"promptForPushNotifications"]) [appDelegate.rootApp setPromptForPushNotifications:[propData.jsonVars objectForKey:@"promptForPushNotifications"]];
    
     //currentMode (live or design)...
     if([propData.jsonVars objectForKey:@"currentMode"]){
         if([[propData.jsonVars objectForKey:@"currentMode"] length] > 0){
             [appDelegate setCurrentMode:[propData.jsonVars objectForKey:@"currentMode"]];
         }else{
             [appDelegate setCurrentMode:@"design"];
         }
     }else{
         [appDelegate setCurrentMode:@"design"];
     }
    
}


//fadeOutLoadView...
-(void)fadeOutLoadView{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"fadeOutLoadView%@", @""]];
    
    [self.view setAlpha:1.0];
    [UIView animateWithDuration:0.5
                     animations:^{
                         self.view.alpha = 0.0;
                     }
                     completion:^(BOOL finished){
     
                         //fade in start view...
                         [self fadeInStartView];
     
                     }];
    
}


//fadeInStartView...
-(void)fadeInStartView{
    [BT_debugger showIt:self message:[NSString stringWithFormat:@"fadeInStartView%@", @""]];
    
    [appDelegate.window.rootViewController.view setAlpha:1.0];
    
    //set navigation controller or tab controller for this storyboard / UINavigation controller...
    if([appDelegate.rootApp rootTabBarController] != nil){
     
        //set the windows rootViewController...
        [appDelegate.window setRootViewController:appDelegate.rootApp.rootTabBarController];
        [appDelegate.window bringSubviewToFront:[appDelegate.rootApp.rootTabBarController view]];
        [appDelegate.window.rootViewController.view setAlpha:0.0];
     
    }else{
     
        //set the windows rootViewController...
        [appDelegate.window setRootViewController:appDelegate.rootApp.rootNavController];
        [appDelegate.window bringSubviewToFront:[appDelegate.rootApp.rootNavController view]];
        [appDelegate.window.rootViewController.view setAlpha:0.0];
     
    }
    
    //fade in "home screen" view...
    [self.view setAlpha:0.0];
    [appDelegate.window.rootViewController.view setAlpha:0.0];
    [UIView animateWithDuration:0.5
         animations:^{
             [appDelegate.window.rootViewController.view setAlpha:1.0];
         }
         completion:^(BOOL finished){
             //ignore...
         }];
    
}




@end







