/*
 *	Copyright Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */


#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

//class
@class BT_quiz_photo;

//delegate
@protocol BT_Quiz_PhotoDelegate <NSObject>
- (void)photoDidFinishLoading:(BT_quiz_photo *)photo;
- (void)photoDidFailToLoad:(BT_quiz_photo *)photo;
@end

//BT_photo
@interface BT_quiz_photo : NSObject{
	
	//image
	NSString *imageName;
	NSString *photoPath;
	NSURL *photoURL;
	UIImage *photoImage;
	
	// Flags
	BOOL workingInBackground;
	
}

//class
+(BT_quiz_photo *)photoWithImage:(UIImage *)image;
+(BT_quiz_photo *)photoWithFilePath:(NSString *)path;
+(BT_quiz_photo *)photoWithURL:(NSURL *)url;

//init
-(id)initWithImage:(UIImage *)image;
-(id)initWithFilePath:(NSString *)path;
-(id)initWithURL:(NSURL *)url;

//public methods
-(BOOL)isImageAvailable;
-(UIImage *)image;
-(UIImage *)obtainImage;
-(void)obtainImageInBackgroundAndNotify:(id <BT_Quiz_PhotoDelegate>)notifyDelegate;
-(void)releasePhoto;

@end






