/*
 *	Copyright Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */

#import "BT_quiz_photo.h"
#import "BT_screen_quiz.h"

@implementation BT_screen_quiz
@synthesize saveAsFileName, downloader, didInit, ver;

/* quiz properties */
@synthesize quizQuestions, quizShowTimer, quizQuestionDelay, quizPointsPerAnswer, quizRandomizeQuestions;
@synthesize quizButtonColorAnswers, quizButtonColorCorrect, quizButtonColorIncorrect, quizImageCornerRadius;
@synthesize quizSoundEffectFileNameCorrect, quizSoundEffectFileNameIncorrect, quizSoundEffectFileNameFinished;
@synthesize quizNumberOfQuestions, quizRewardIfPointsOver, quizShowCorrectAnswers, quizFontColorQuestions, quizFontColorAnswers;
@synthesize quizQuestionTransitionType, quizAnswersTransitionType, rotateMessageView, rotateMessageLabel;
@synthesize quizQuestionFontSizeSmallDevice, quizQuestionFontSizeLargeDevice, didFinishOnce, spinner;
@synthesize quizButtonFontSizeSmallDevice, quizButtonFontSizeLargeDevice, finishedButtons;

/* quiz runtime properties */
@synthesize quizRunning, numberCorrect, numberIncorrect, streak, totalPoints, totalSeconds;
@synthesize currentQuestionIndex, currentQuestionObject, quizDidEnd;

/* quiz controls */
@synthesize startButtonBox, startButton, questionBox, answerButtonBox, paddingTop;
@synthesize answerButton1, answerButton2, answerButton3, answerButton4;
@synthesize answerLabel1, answerLabel2, answerLabel3, answerLabel4;
@synthesize countdownToStartTimer, countdownToStartLabel, currentPointsView, currentPointsImageView;
@synthesize currentPointsLabel, bonusImageView, quizTimeLabel, quizTimer, questionTransitionTimer;
@synthesize questionText, questionImageView, quizToolbar;

//viewDidLoad
-(void)viewDidLoad{
	[BT_debugger showIt:self theMessage:@"viewDidLoad"];
	[super viewDidLoad];
    
	//init screen properties
	[self setDidInit:0];
	[self setDidFinishOnce:0];
	[self setQuizDidEnd:0];
	[self setPaddingTop:0];
    
    
	////////////////////////////////////
	//init quiz properties...
	quizShowCorrectAnswers = TRUE;
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizShowCorrectAnswers" defaultValue:@"1"] isEqualToString:@"0"]){
		quizShowCorrectAnswers = FALSE;
	}
	quizShowTimer = TRUE;
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizShowTimer" defaultValue:@"1"] isEqualToString:@"0"]){
		quizShowTimer = FALSE;
	}		
	quizRandomizeQuestions = TRUE;
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizRandomizeQuestions" defaultValue:@"1"] isEqualToString:@"0"]){
		quizRandomizeQuestions = FALSE;
	}		
	
	//font sizes
	quizQuestionFontSizeSmallDevice = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizQuestionFontSizeSmallDevice" defaultValue:@"25"] intValue];
	quizQuestionFontSizeLargeDevice = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizQuestionFontSizeLargeDevice" defaultValue:@"55"] intValue];
	quizButtonFontSizeSmallDevice = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizButtonFontSizeSmallDevice" defaultValue:@"14"] intValue];
	quizButtonFontSizeLargeDevice = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizButtonFontSizeLargeDevice" defaultValue:@"40"] intValue];
	quizImageCornerRadius = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizImageCornerRadius" defaultValue:@"8"] intValue];
	quizQuestionDelay = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizQuestionDelay" defaultValue:@"2"] intValue];
	quizPointsPerAnswer = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizPointsPerAnswer" defaultValue:@"150"] intValue];
	quizNumberOfQuestions = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizNumberOfQuestions" defaultValue:@"0"] intValue];
	quizRewardIfPointsOver = [[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizRewardIfPointsOver" defaultValue:@"0"] intValue];
	
	//question, button, font colors.
	quizFontColorQuestions = [BT_color getColorFromHexString:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizFontColorQuestions" defaultValue:@"#000000"]];
	quizFontColorAnswers = [BT_color getColorFromHexString:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizFontColorAnswers" defaultValue:@"#FFFFFF"]];
	quizButtonColorAnswers = [BT_color getColorFromHexString:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizButtonColorAnswers" defaultValue:@"#000000"]];
	
	//sound effects.
	quizSoundEffectFileNameCorrect = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameCorrect" defaultValue:@""];
	quizSoundEffectFileNameIncorrect = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameIncorrect" defaultValue:@""];
	quizSoundEffectFileNameFinished = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameFinished" defaultValue:@""];
	
	//transition types
	quizQuestionTransitionType = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizQuestionTransitionType" defaultValue:@"flip"];
	quizAnswersTransitionType = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizAnswersTransitionType" defaultValue:@"flip"];
    
	//if we have a transparent nav bar, move everything down...
	if([[BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@""] isEqualToString:@"transparent"]){
		paddingTop = 44;
	}

	////////////////////////////////////
	//init quiz controls...
	
	double deviceWidth = self.view.bounds.size.width;
	double deviceHeight = self.view.bounds.size.height;
	
 	//frame for image is different for small / large devices
	UIImage *defaultQuestionImage = [UIImage imageNamed:@"bt_quiz_bg_small.png"];
	int margin = 10;
	int questionBoxTop = 5;
	int questionBoxWidth = deviceWidth - 20;
	int questionBoxHeight = 150;
	int answerButtonBoxTop = questionBoxHeight + 5;
	int answerButtonBoxWidth = deviceWidth - 20;
	int answerButtonBoxHeight = deviceHeight - (160 + 88);
    //if iOS7+, adjust answerButtonBoxHeight
    ver = [[[UIDevice currentDevice] systemVersion] floatValue];
    if (ver >= 7.0) {
        //get navbar style
        NSString *navBarStyle = [BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarStyle" defaultValue:@"solid"];
        if ([[navBarStyle uppercaseString] isEqualToString:@"SOLID"]) answerButtonBoxHeight -= 44;
    }
    
	int buttonHeight = (answerButtonBoxHeight / 4.5);		
	int buttonPadding = 6;
	int questionFontSize = quizQuestionFontSizeSmallDevice;
	int buttonFontSize = quizButtonFontSizeSmallDevice;
	if([appDelegate.rootDevice isIPad]){
		defaultQuestionImage = [UIImage imageNamed:@"bt_quiz_bg_large.png"];
		questionBoxHeight = 350;
		answerButtonBoxTop = questionBoxHeight + 12;
		answerButtonBoxHeight = deviceHeight - (360 + 88);
		buttonHeight = (answerButtonBoxHeight / 4.3);
		questionFontSize = quizQuestionFontSizeLargeDevice;
		buttonFontSize = quizButtonFontSizeLargeDevice;
	}

	int buttonWidth = answerButtonBoxWidth;
	
	//add the top padding (changes if we have a transparent nav bar)
	questionBoxTop += paddingTop;
	answerButtonBoxTop += paddingTop;
	//start button box is a view with two subviews. Start button is initially hidden before until the parseData method completes.
	startButtonBox = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
	startButtonBox.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	[startButtonBox setAlpha:0.0];
    
    //transparent mask
    UIView *startMask = [[UIView alloc] initWithFrame:CGRectMake(0, -50, 1500, 1500)];
    [startMask setBackgroundColor:[UIColor blackColor]];
    
    //check if we have a backgroundColor set
    NSString *backgroundColor = [BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"backgroundColor" defaultValue:@""];
    if (backgroundColor.length>0) [startMask setBackgroundColor:[UIColor clearColor]];
    
    [startMask setAlpha:.60];
    [startButtonBox addSubview:startMask];
    
    //create rounded rectangle start button
    startButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    [startButton setFrame:CGRectMake(margin, answerButtonBoxTop + 5, answerButtonBoxWidth, buttonHeight)];
    startButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [startButton addTarget:self action:@selector(startQuiz) forControlEvents:UIControlEventTouchUpInside];
    [startButton.titleLabel setFont:[UIFont boldSystemFontOfSize:30]];
    [startButton setTitle:NSLocalizedString(@"quizStart",@"start quiz") forState:UIControlStateNormal];
    
    //countdown to start label is initially hidden
    countdownToStartLabel = [[UILabel alloc] initWithFrame:CGRectMake(margin, answerButtonBoxTop + 20, answerButtonBoxWidth, 125)];
    countdownToStartLabel.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [countdownToStartLabel setFont:[UIFont boldSystemFontOfSize:100]];
    [countdownToStartLabel setBackgroundColor:[UIColor clearColor]];
    [countdownToStartLabel setTextColor:[UIColor whiteColor]];
    [countdownToStartLabel setNumberOfLines:1];
    [countdownToStartLabel setText:@""];
    [countdownToStartLabel setTextAlignment:NSTextAlignmentCenter];
    
    //add the countdown label, then the start button on top
    [startButtonBox addSubview:countdownToStartLabel];
    [startButtonBox addSubview:startButton];
    
	//do we round the question images?
	if(quizImageCornerRadius > 0){
        defaultQuestionImage = [self roundedCornerImage:defaultQuestionImage withCornerSize:8 borderSize:1];
	}

	//position controls
	questionBox = [[UIView alloc] initWithFrame:CGRectMake(margin, questionBoxTop, questionBoxWidth, questionBoxHeight)];
	questionBox.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
	
    //question image view
    questionImageView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, questionBoxWidth, questionBoxHeight)];
    [questionImageView setContentMode:UIViewContentModeScaleToFill];
    [questionImageView setClipsToBounds:TRUE];
    [questionImageView setImage:defaultQuestionImage];
    [questionBox addSubview:questionImageView];
    
    //spinner on top of question.
    spinner = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhite];		
    spinner.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [spinner setCenter:[questionImageView center]];
    [spinner setHidesWhenStopped:TRUE];
    [questionBox addSubview:spinner];		
    
    //question text
    questionText = [[UITextView alloc] initWithFrame:CGRectMake(0, 0, questionBoxWidth, questionBoxHeight)];
    [questionText setFont:[UIFont boldSystemFontOfSize:questionFontSize]];
    [questionText setBackgroundColor:[UIColor clearColor]];
    [questionText setTextColor:quizFontColorQuestions];
    [questionText setUserInteractionEnabled:TRUE];
    [questionText setEditable:FALSE];
    [questionText setShowsVerticalScrollIndicator:FALSE];
    [questionText setShowsHorizontalScrollIndicator:FALSE];
    [questionText setText:@""];
    if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizQuestionFontJustify" defaultValue:@""] isEqualToString:@"left"]){
        [questionText setTextAlignment:NSTextAlignmentLeft];
    }else{
        [questionText setTextAlignment:NSTextAlignmentCenter];
    }
    [questionBox addSubview:questionText];
    
	//add the question box...
	[self.view addSubview:questionBox];
    
	//answer buttons are in a box. Initially hidden until the parseData method is complete.
	answerButtonBox = [[UIView alloc] initWithFrame:CGRectMake(10, answerButtonBoxTop, answerButtonBoxWidth, answerButtonBoxHeight)];
	[answerButtonBox setBackgroundColor:[UIColor clearColor]];
	answerButtonBox.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
	[answerButtonBox setAlpha:0.0];
	

	//answer button 1
    answerButton1 = [self getButtonForQuizWithFrame:CGRectMake(4, buttonPadding, buttonWidth - 8, buttonHeight) theTag:1 buttonColor:quizButtonColorAnswers];
	[answerButtonBox addSubview:answerButton1];
	answerLabel1 = [BT_viewUtilities getLabelForQuizButton:CGRectMake(5, buttonPadding, buttonWidth - 10, buttonHeight)
                                                  fontSize:buttonFontSize
                                                 fontColor:quizFontColorAnswers];
	[answerButtonBox addSubview:answerLabel1];	
    
	//answer button 2
    answerButton2 = [self getButtonForQuizWithFrame:CGRectMake(4, (1 * buttonHeight) + (2 * buttonPadding), buttonWidth - 8, buttonHeight) theTag:2 buttonColor:quizButtonColorAnswers];
	[answerButtonBox addSubview:answerButton2];
	answerLabel2 = [BT_viewUtilities getLabelForQuizButton:CGRectMake(5, (1 * buttonHeight) + (2 * buttonPadding), buttonWidth - 10, buttonHeight)fontSize:buttonFontSize fontColor:quizFontColorAnswers];
	[answerButtonBox addSubview:answerLabel2];	
    
	//answer button 3
    answerButton3 = [self getButtonForQuizWithFrame:CGRectMake(4, (2 * buttonHeight) + (3 * buttonPadding), buttonWidth - 8, buttonHeight) theTag:3 buttonColor:quizButtonColorAnswers];
	[answerButtonBox addSubview:answerButton3];
	answerLabel3 = [BT_viewUtilities getLabelForQuizButton:CGRectMake(5, (2 * buttonHeight) + (3 * buttonPadding), buttonWidth - 10, buttonHeight) fontSize:buttonFontSize fontColor:quizFontColorAnswers];
	[answerButtonBox addSubview:answerLabel3];	
    
	//answer button 4
    answerButton4 = [self getButtonForQuizWithFrame:CGRectMake(4, (3 * buttonHeight) + (4 * buttonPadding), buttonWidth - 8, buttonHeight) theTag:4 buttonColor:quizButtonColorAnswers];
	[answerButtonBox addSubview:answerButton4];
	answerLabel4 = [BT_viewUtilities getLabelForQuizButton:CGRectMake(5, (3 * buttonHeight) + (4 * buttonPadding), buttonWidth - 10, buttonHeight) fontSize:buttonFontSize fontColor:quizFontColorAnswers];
	[answerButtonBox addSubview:answerLabel4];	
    
	//add button box..
	[self.view addSubview:answerButtonBox];

	//current points view holds a label and image (starts out hidden then animates when we show it)
	currentPointsView = [[UIView alloc] initWithFrame:CGRectMake(0, answerButtonBoxTop - 40, 75, 75)];
	currentPointsView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
	[currentPointsView setBackgroundColor:[UIColor clearColor]];
	[currentPointsView setAlpha:0.0];			
    
    //image view for corrent / incorrect images
    currentPointsImageView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"bt_quiz_goldStar.png"]];
    [currentPointsView addSubview:currentPointsImageView];
    
    //label for current points
    currentPointsLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 30, 75, 20)];
    [currentPointsLabel setFont:[UIFont boldSystemFontOfSize:15]];
    [currentPointsLabel setBackgroundColor:[UIColor clearColor]];
    [currentPointsLabel setTextColor:[UIColor blackColor]];
    [currentPointsLabel setNumberOfLines:1];
    [currentPointsLabel setText:@""];
    [currentPointsLabel setTextAlignment:NSTextAlignmentCenter];
    [currentPointsView addSubview:currentPointsLabel];
    
    //image view for bonus images
    bonusImageView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 75, 30)];
    [bonusImageView setImage:[UIImage imageNamed:@"bt_blank.png"]];
    [bonusImageView setHidden:TRUE];
    bonusImageView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
    [currentPointsView addSubview:bonusImageView];
    
	//add to sub-view
	[self.view addSubview:currentPointsView];
	
	//add start button box on top of everything as a mask
	[self.view addSubview:startButtonBox];

	//quizToolbar
    //make sure navBarBackgroundColor is set first.  Necessary so we don't pass a null value to setTintcolor
    NSString *navBarBackgroundColor = [BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"navBarBackgroundColor" defaultValue:@"#FFF"];
    BT_item *thisQuizData = self.screenData;
    NSMutableDictionary *theJson = [self.screenData.jsonVars mutableCopy];
    [theJson setObject:navBarBackgroundColor forKey:@"navBarBackgroundColor"];
    thisQuizData.jsonVars = theJson;
    
	quizToolbar = [BT_viewUtilities getQuizToolBarForScreen:self theScreenData:thisQuizData];
    quizToolbar.translucent = false;
    if (ver>=7)
        quizToolbar.barTintColor = [BT_color getColorFromHexString:navBarBackgroundColor];
    else
        quizToolbar.tintColor = [BT_color getColorFromHexString:navBarBackgroundColor];
    [self.view addSubview:quizToolbar];
    
	//message view shows when device is in landscape mode (quizzes do not rotate)
	rotateMessageView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, deviceWidth, deviceHeight - 44)];
	rotateMessageView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	[rotateMessageView setBackgroundColor:[UIColor blackColor]];
    
    //label goes on top
    rotateMessageLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 75, deviceWidth, 50)];
    rotateMessageLabel.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [rotateMessageLabel setFont:[UIFont boldSystemFontOfSize:20]];
    [rotateMessageLabel setBackgroundColor:[UIColor clearColor]];
    [rotateMessageLabel setTextColor:[UIColor whiteColor]];
    [rotateMessageLabel setNumberOfLines:1];
    [rotateMessageLabel setText:[NSString stringWithFormat:@"%@", NSLocalizedString(@"quizRotateMessage", @"Please rotate your device.")]];
    [rotateMessageLabel setTextAlignment:NSTextAlignmentCenter];
	[self.view addSubview:rotateMessageView];
	[self.view addSubview:rotateMessageLabel];
	
	//if landscape...show the warning..
    if(UIDeviceOrientationIsPortrait([UIDevice currentDevice].orientation)){
		[rotateMessageView setHidden:TRUE];
		[rotateMessageLabel setHidden:TRUE];
		[startButton setHidden:FALSE];
	}else{
		[rotateMessageView setHidden:FALSE];
		[rotateMessageLabel setHidden:FALSE];
		[startButton setHidden:TRUE];
	}
    
    
	//find the quiz timer label in the toolbar.
	quizTimeLabel = (UILabel *)[quizToolbar viewWithTag:105];
	
	//quizRunningLoop runs in background and increments seconds..
	quizTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(quizRunningLoop) userInfo:nil repeats:YES]; 
	[[NSRunLoop currentRunLoop] addTimer:quizTimer forMode:NSRunLoopCommonModes];		
	
    
}

/*
 This method build a UISegementedControl that we use as a "button" so we can show "selected states"
 */
-(UIButton *)getButtonForQuizWithFrame:(CGRect)theFrame theTag:(int)theTag buttonColor:(UIColor *)buttonColor{
	//[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"getButtonForQuiz%@", @""]];
    
    SEL sel15 = NSSelectorFromString(@"answerClick:");
    
    UIButton *tmpBtn = [[UIButton alloc] initWithFrame:theFrame];

	[tmpBtn setTintColor:buttonColor];
    [tmpBtn setBackgroundColor:buttonColor];
    tmpBtn.layer.cornerRadius = 10;
    tmpBtn.layer.borderWidth = 1;
    tmpBtn.layer.borderColor = [UIColor blackColor].CGColor;
    [tmpBtn addTarget:self action:sel15 forControlEvents:UIControlEventTouchUpInside];
	[tmpBtn setTag:theTag];
	return tmpBtn;
    
}


//view will appear
-(void)viewWillAppear:(BOOL)animated{
	[super viewWillAppear:animated];
	[BT_debugger showIt:self theMessage:@"viewWillAppear"];
    self.view.backgroundColor = [UIColor lightGrayColor];
    
    //check if we have a backgroundColor set
    NSString *backgroundColor = [BT_strings getStyleValueForScreen:self.screenData nameOfProperty:@"backgroundColor" defaultValue:@""];
    if (backgroundColor.length>0) [self.view setBackgroundColor:[BT_color getColorFromHexString:backgroundColor]];
}


//view did appear
-(void)viewDidAppear:(BOOL)animated{
	[super viewWillAppear:animated];
	[BT_debugger showIt:self theMessage:@"viewDidAppear"];
	
	//ignore if we already inited
	if(self.didInit < 1){
		[self setDidInit:1];
		[self setQuizDidEnd:0];
		[self performSelector:(@selector(loadData)) withObject:nil afterDelay:0.1];
	}else{
		//we must be coming back from another screen, trigger quiz ended method if
		//we have already taken the quiz. If we have not taken a quiz yet, do nothing
		if(self.quizDidEnd > 0){
			[self quizEnded];
		}else{
			[self startQuiz];
		}
	}
    
}

- (UIImage *)roundedCornerImage:(UIImage*)image withCornerSize:(NSInteger)cornerSize borderSize:(NSInteger)borderSize {
    
    UIGraphicsBeginImageContextWithOptions(image.size, NO, image.scale); {
        CGRect imageRect = (CGRect){ CGPointZero, image.size };
        CGRect borderRect = CGRectInset(imageRect, borderSize, borderSize);
        UIBezierPath *path = [UIBezierPath bezierPathWithRoundedRect:borderRect
                                                   byRoundingCorners:UIRectCornerAllCorners
                                                         cornerRadii:CGSizeMake(cornerSize, cornerSize)];
        [path addClip];
        [image drawAtPoint:CGPointZero];
    }
    UIImage *roundedImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return roundedImage;
}

//load data
-(void)loadData{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"loadData%@", @""]];
	
	/*
     Screen Data scenarios
     --------------------------------
     a)	No dataURL is provided in the screen data - use the info configured in the app's configuration file
     b)	A dataURL is provided, download now if we don't have a cache, else, download on refresh.
     */
	
	self.saveAsFileName = [NSString stringWithFormat:@"screenData_%@.txt", [self.screenData itemId]];
	
	//do we have a URL?
	BOOL haveURL = FALSE;
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""] length] > 10){
		haveURL = TRUE;
	}
	
	//start by filling the list from the configuration file, use these if we can't get anything from a URL
	if([[self.screenData jsonVars] objectForKey:@"childItems"]){
        
		//init the items array
		self.quizQuestions = [[NSMutableArray alloc] init];
        
		NSArray *tmpQuestions = [[self.screenData jsonVars] objectForKey:@"childItems"];
		for(NSMutableDictionary *tmpQuestion in tmpQuestions){
			BT_item *thisQuestion = [[BT_item alloc] init];
			thisQuestion.itemId = [tmpQuestion objectForKey:@"itemId"];
			thisQuestion.itemType = [tmpQuestion objectForKey:@"itemType"];
			thisQuestion.jsonVars = tmpQuestion;
			[self.quizQuestions addObject:thisQuestion];
		}
        
	}else{
		[BT_debugger showIt:self theMessage:@"the quiz screen does not have a childItems[] array"];
	}
	
	//if we have a URL, fetch..
	if(haveURL){
        
		//look for a previously cached version of this screens data...
		if([BT_fileManager doesLocalFileExist:[self saveAsFileName]]){
			[BT_debugger showIt:self theMessage:@"parsing previsouly cached quiz questions"];
			NSString *staleData = [BT_fileManager readTextFileFromCacheWithEncoding:self.saveAsFileName encodingFlag:-1];
			[self parseScreenData:staleData];
		}else{
			[BT_debugger showIt:self theMessage:@"no cached version of the quiz questions found"];
			[self downloadData];
		}
        
        
	}else{
		
		//show the child items in the config data
		[BT_debugger showIt:self theMessage:@"no dataURL found for this quiz screen"];
		[self layoutScreen];
		
	}
    
	
}

//refresh data
-(void)refreshData{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"refreshData%@", @""]];
	[self downloadData];
}


//download data
-(void)downloadData{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloading screen data and saving as: %@", [self saveAsFileName]]];
	
	//flag this as the current screen
	appDelegate.rootApp.currentScreenData = self.screenData;
	
	//hide the answer buttons
	[answerButtonBox setAlpha:0.0];
    
	//hide the start button
	[startButton setHidden:TRUE];
	
	//show progress
	[self showProgress];
	
	NSString *tmpURL = @"";
	if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""] length] > 3){
		
		//merge url variables
		tmpURL = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"dataURL" defaultValue:@""];
		tmpURL = [tmpURL stringByReplacingOccurrencesOfString:@"[screenId]" withString:[self.screenData itemId]];
        
		///merge possible variables in URL
		NSString *useURL = [BT_strings mergeBTVariablesInString:tmpURL];
		NSString *escapedUrl = [useURL stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        
		//fire downloader to fetch and results
		downloader = [[BT_downloader alloc] init];
		[downloader setSaveAsFileName:[self saveAsFileName]];
		[downloader setSaveAsFileType:@"text"];
		[downloader setUrlString:escapedUrl];
		[downloader setDelegate:self];
		[downloader downloadFile];	
	}
}

//parse screen data
-(void)parseScreenData:(NSString *)theData{
	[BT_debugger showIt:self theMessage:@"parseScreenData"];
	
	@try {	
        
		//arrays for screenData
		self.quizQuestions = [[NSMutableArray alloc] init];
        
        
        //create dictionary from the JSON string
        NSError *jsonError = nil;
        NSData *jsonDataFromString = [theData dataUsingEncoding:NSUTF8StringEncoding];
        NSDictionary *jsonData = [NSJSONSerialization JSONObjectWithData:jsonDataFromString options:0 error:&jsonError];
        if(jsonError != nil){
        
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"ERROR parsing JSON: %@", jsonError.description]];
			[self showAlert:NSLocalizedString(@"errorTitle",@"~ Error ~") theMessage:NSLocalizedString(@"appParseError", @"There was a problem parsing some configuration data. Please make sure that it is well-formed") alertTag:0];
			[BT_fileManager deleteFile:[self saveAsFileName]];
            
		}else{
			
			if([jsonData objectForKey:@"childItems"]){
				NSArray *tmpQuestions = [jsonData objectForKey:@"childItems"];
				for(NSMutableDictionary *tmpQuestion in tmpQuestions){
					BT_item *thisQuestion = [[BT_item alloc] init];
					thisQuestion.itemId = [tmpQuestion objectForKey:@"itemId"];
					thisQuestion.itemType = [tmpQuestion objectForKey:@"itemType"];
					thisQuestion.jsonVars = tmpQuestion;
					[self.quizQuestions addObject:thisQuestion];
				}
			}
			
			//layout screen
			[self layoutScreen];		
            
		}
		
	}@catch (NSException * e) {
        
		//delete bogus data, show alert
		[BT_fileManager deleteFile:[self saveAsFileName]];
		[self showAlert:NSLocalizedString(@"errorTitle",@"~ Error ~") theMessage:NSLocalizedString(@"appParseError", @"There was a problem parsing some configuration data. Please make sure that it is well-formed") alertTag:0];
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"error parsing screen data: %@", e]];
        
	} 	
	
    
}

//build screen (drops pins)
-(void)layoutScreen{
	[BT_debugger showIt:self theMessage:@"layoutScreen"];
	
	//show cover-view if we are in landscape mode. Quizzes only work in portrait
    if(UIDeviceOrientationIsPortrait([UIDevice currentDevice].orientation)){
		[rotateMessageView setHidden:TRUE];
		[rotateMessageLabel setHidden:TRUE];
		[questionBox setAlpha:1.0];
		if(!quizRunning){
			[startButton setHidden:FALSE];
		}else{
			[answerButtonBox setAlpha:1.0];
		}
	}else{
		[rotateMessageView setHidden:FALSE];
		[rotateMessageLabel setHidden:FALSE];
		[answerButtonBox setAlpha:0.0];
		[questionBox setAlpha:0.0];
		[startButton setHidden:TRUE];
	}
	
	//this method is called after parsing the screens data, AND after rotations...
	if(!quizRunning){
		if([quizQuestions count] > 0){
            
			//hide the answer buttons
			[answerButtonBox setAlpha:0.0];
            
			//show the start button
			[startButtonBox setAlpha:1.0];
            
		}else{
			[BT_debugger showIt:self theMessage:@"This quiz screen does not have any questions?"];
		}
	}
    
}


//////////////////////////////////////////////////////////////////////////////////////////////////
//quiz methods

//start quiz
-(void)startQuiz{
	[BT_debugger showIt:self theMessage:@"startQuiz"];
	
	//reset timer label
	[quizTimeLabel setText:@""];
	[self setQuizRunning:FALSE];
    
	//don't start unless we are in portrait mode
    if(UIDeviceOrientationIsPortrait([UIDevice currentDevice].orientation)){
        
		//hide the start button
		[startButton setHidden:TRUE];
		[startButtonBox setAlpha:1.0];
		[countdownToStartLabel setHidden:FALSE];
		[answerButtonBox setHidden:TRUE];
		[questionText setText:@""];
		[quizTimeLabel setText:@""];
		[self setDidFinishOnce:0];
		[self setTotalSeconds:0];
        
		//set the initial label
		[countdownToStartLabel setText:@"3"];
        if (!quizShowTimer) {
            [countdownToStartLabel setText:@""];
            //kill the timer..
            [countdownToStartTimer invalidate];
            
            //triger start..
            [self countdownToStartDone];
        }
        
		//trigger the countdown timer to run every 1 second
		countdownToStartTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(countdownToStartUpdate) userInfo:nil repeats:YES]; 
        
	}//portrait mode
}

//udpates on timer ticks during quiz start countdown
-(void)countdownToStartUpdate{
    
    
	//ignore if we are not in portrait mode
	//don't start unless we are in portrait mode
    if(UIDeviceOrientationIsPortrait([UIDevice currentDevice].orientation)){
        
		//change countdown label until done...
		if([countdownToStartLabel.text isEqualToString:@"3"]){
			[countdownToStartLabel setText:@"2"];
		}else if([countdownToStartLabel.text isEqualToString:@"2"]){
			[countdownToStartLabel setText:@"1"];
		}else if([countdownToStartLabel.text isEqualToString:@"1"]){
			[countdownToStartLabel setText:NSLocalizedString(@"quizGo", "go!")];
		}else if([countdownToStartLabel.text isEqualToString:NSLocalizedString(@"quizGo", "go!")]){
			
			//kill the timer..
			[countdownToStartTimer invalidate];
			
			//triger start..
			[self countdownToStartDone];		
            
		}
		
	}else{
		[countdownToStartLabel setText:@""];
	}
	
}

//countdown to start done..
-(void)countdownToStartDone{
	[BT_debugger showIt:self theMessage:@"countdownToStartDone"];
	
	//ignore this if we are not in portrait
    if(UIDeviceOrientationIsPortrait([UIDevice currentDevice].orientation)){
        
		//hide the start button and the countdown label
		[startButton setHidden:TRUE];
		[countdownToStartLabel setHidden:TRUE];
		[answerButtonBox setHidden:FALSE];
        
		//animate "mask" view away...
		[UIView beginAnimations:@"fadeMask" context:nil];
		[startButtonBox setAlpha:1.0];
		[startButtonBox setAlpha:0.0];
		[UIView setAnimationDuration:0.75];
		[UIView commitAnimations];
        
		//init quiz run-time vars
		[self setQuizRunning:TRUE];
		[self setTotalSeconds:0];
		[self setNumberCorrect:0];
		[self setNumberIncorrect:0];
		[self setStreak:0];
		[self setTotalPoints:0];
		[self setCurrentQuestionIndex:0];
		[currentPointsView setAlpha:0.0];
		
		//make sure we have enough questions...
		if(([quizQuestions count] <= quizNumberOfQuestions) || quizNumberOfQuestions == 0 ){
			quizNumberOfQuestions = (int)[quizQuestions count];
		}
		
		//randomize questions if "quizRandomizeQuestions"
		if(quizRandomizeQuestions){
			NSUInteger firstQuestion = 0;
			for (int i = 0; i < [self.quizQuestions count]; i++){
				NSUInteger randomIndex = arc4random() % [self.quizQuestions count];
				[self.quizQuestions exchangeObjectAtIndex:firstQuestion withObjectAtIndex:randomIndex];
				firstQuestion +=1;
			}
		}					
        
		//transition question
		[self transitionQuestion];
        
	}//not in portrait mode
    
}

//quizRunningLoop
-(void)quizRunningLoop{
	//if we are showing the timer
	if(quizShowTimer){
		[quizTimeLabel setHidden:FALSE];
		//setup text in footer label
		if(quizRunning){
			
			//increment seconds
			[self setTotalSeconds:totalSeconds + 1];
			NSString *formatted = [NSString stringWithFormat:@"%d", totalSeconds];
			[quizTimeLabel setText:[BT_strings formatTimeFromSeconds:formatted]];
			
		}
		
	}else{
		[quizTimeLabel setHidden:TRUE];
	}
	
}

//transitionQuestion 
-(void)transitionQuestion{
	[BT_debugger showIt:self theMessage:@"transitionQuestion"];
    
	//be sure answers are showing..
	[answerButtonBox setAlpha:1.0];
	
	//prepare question animation block
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.75];
	if([quizQuestionTransitionType isEqualToString:@"curl"]) [UIView setAnimationTransition:UIViewAnimationTransitionCurlUp forView:questionBox cache:NO];
	if([quizQuestionTransitionType isEqualToString:@"flip"]) [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromRight forView:questionBox cache:NO];
	[UIView commitAnimations];
    
	//prepare answer animation block
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.75];
	if([quizAnswersTransitionType isEqualToString:@"curl"]) [UIView setAnimationTransition:UIViewAnimationTransitionCurlUp forView:answerButtonBox cache:NO];
	if([quizAnswersTransitionType isEqualToString:@"flip"]) [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:answerButtonBox cache:NO];
	[UIView commitAnimations];
	
	//get the current question
	BT_item *tmpQuestion = [self.quizQuestions objectAtIndex:currentQuestionIndex];
    
	//remember this question
	[self setCurrentQuestionObject:tmpQuestion];
    
	//properties for this question
	NSString *tmpQuestionText = [BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"questionText" defaultValue:@""];
	NSString *tmpImageName = [BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"imageNameSmallDevice" defaultValue:@"bt_blank.png"];
	NSString *tmpImageURL = [BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"imageURLSmallDevice" defaultValue:@"bt_blank.png"];
	
	//should we use the large image instead?
	if([appDelegate.rootDevice isIPad]){
		tmpImageName = [BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"imageNameLargeDevice" defaultValue:@"bt_blank.png"];
		tmpImageURL = [BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"imageURLLargeDevice" defaultValue:@"bt_blank.png"];
	}
	
	//if we have an imageURL, and no imageName, figure out a name to use...
	if(tmpImageName.length < 3 && tmpImageURL.length > 3){
		tmpImageName = [BT_strings getFileNameFromURL:tmpImageURL];
	}
    
	//if we don't have an image, "dim" the default image...
	if([tmpImageName length] < 3){
		[spinner stopAnimating];
        
		UIImage *defaultQuestionImage = [UIImage imageNamed:@"bt_quiz_bg_small.png"];
		if([appDelegate.rootDevice isIPad]){
			defaultQuestionImage = [UIImage imageNamed:@"bt_quiz_bg_large.png"];
		}
		[self setQuestionImage:defaultQuestionImage];
		[questionImageView setAlpha:0.1];
        
	}else{
        
		//show this questions image...
		[self setQuestionImage:[UIImage imageNamed:@"bt_blank.png"]];
		[questionImageView setAlpha:1.0];
		[self.spinner startAnimating];
		
		//photo could be in the bundle, in the cache, or at a URL
		UIImage *tmpImage = nil;
		if([BT_fileManager doesFileExistInBundle:tmpImageName]){
			tmpImage = [UIImage imageNamed:tmpImageName];
			[self setQuestionImage:tmpImage];
		}else{
			if([BT_fileManager doesLocalFileExist:tmpImageName]){
				tmpImage = [BT_fileManager getImageFromFile:tmpImageName];
				[self setQuestionImage:tmpImage];
			}else{
				if([tmpImageURL length] > 3){
                    
					//custom image downloader thing-a-ma-jig
					BT_quiz_photo *bt_quiz_photo = [BT_quiz_photo photoWithURL:[NSURL URLWithString:tmpImageURL]];
					if([bt_quiz_photo isImageAvailable]){
						[self setQuestionImage:[bt_quiz_photo image]];
					}else{
						[bt_quiz_photo obtainImageInBackgroundAndNotify:self];
					}
					
				}else{
					
					//image does not exist in bundle..and.. not imageURL was provided
					[self.spinner stopAnimating];
					[self setQuestionImage:[UIImage imageNamed:@"bt_blank.png"]];
                    
				}
				
				
			}
		}
        
	}//tmpImageName
	
    
	//set question text - blank if "none" is used.
	if([tmpQuestionText caseInsensitiveCompare:@"none"] == NSOrderedSame ){
		[self.questionText setText:@""];
	}else{
		[self.questionText setText:tmpQuestionText];
	}
	
	//tmp array of answers..3 are incorrect, 1 is correct
	NSMutableArray *tmpAnswers = [[NSMutableArray alloc] init];
	[tmpAnswers addObject:[BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"incorrectText1" defaultValue:@""]];
	[tmpAnswers addObject:[BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"incorrectText2" defaultValue:@""]];
	[tmpAnswers addObject:[BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"incorrectText3" defaultValue:@""]];
	[tmpAnswers addObject:[BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"correctAnswerText" defaultValue:@""]];
	
	//randomize the answers..	
	NSUInteger firstAnswer = 0;
	for (int i = 0; i < [tmpAnswers count]; i++){
		NSUInteger randomIndex = random() % [tmpAnswers count];
		[tmpAnswers exchangeObjectAtIndex:firstAnswer withObjectAtIndex:randomIndex];
		firstAnswer +=1;
	}
	
	//set the answers text...
	[answerLabel1 setText:[tmpAnswers objectAtIndex:0]];
	[answerLabel2 setText:[tmpAnswers objectAtIndex:1]];
	[answerLabel3 setText:[tmpAnswers objectAtIndex:2]];
	[answerLabel4 setText:[tmpAnswers objectAtIndex:3]];
	
	//clean up
	tmpAnswers = nil;
	
	//change button colors if "quizButtonColorAnswers" 
	if(quizShowCorrectAnswers){
        
		//button colors
		quizButtonColorAnswers = [BT_color getColorFromHexString:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizButtonColorAnswers" defaultValue:@"#000000"]];
        
		[answerButton1 setBackgroundColor:quizButtonColorAnswers];
		[answerButton2 setBackgroundColor:quizButtonColorAnswers];
		[answerButton3 setBackgroundColor:quizButtonColorAnswers];
		[answerButton4 setBackgroundColor:quizButtonColorAnswers];
        
	}
	
	//allow answer selection
	[self enableButtons];	
    
    
}

//transitions correct
-(void)transitionPoints{
	[BT_debugger showIt:self theMessage:@"transitionPoints"];
    
	//fade in
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:1.0];
	[currentPointsView setAlpha:0.0];
	[currentPointsView setAlpha:1.0];
	[UIView commitAnimations];
	
	//fade out
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:2.5];
	[currentPointsView setAlpha:1.0];
	[currentPointsView setAlpha:0.0];
	[UIView commitAnimations];
    
	
}

//wobbleImage...
-(void)wobbleImage{
    [BT_debugger showIt:self theMessage:@"wobbleImage"];
    
    //animate the image...
#define RADIANS(degrees) ((degrees * M_PI) / 180.0)
    //You can mess around with these numbers to adjust the wobble
    CGAffineTransform leftWobble = CGAffineTransformRotate(CGAffineTransformIdentity, RADIANS(-2.0));
    CGAffineTransform rightWobble = CGAffineTransformRotate(CGAffineTransformIdentity, RADIANS(2.0));
    answerButtonBox.transform = leftWobble;
    
    [UIView beginAnimations:@"wobble" context:nil];
    [UIView setAnimationRepeatAutoreverses:YES];
    [UIView setAnimationRepeatCount:1];
    [UIView setAnimationDuration:0.14];
    [UIView setAnimationDelegate:self];
    [UIView setAnimationDidStopSelector:@selector(wobbleEnded)];
    answerButtonBox.transform = rightWobble;
    answerButtonBox.transform = leftWobble;
    [UIView commitAnimations];
    
    
}

//wobbleEnded...
-(void)wobbleEnded{
    [BT_debugger showIt:self theMessage:@"wobbleEnded"];
    answerButtonBox.transform = CGAffineTransformIdentity; 
    
}

//answer click
-(void)answerClick:(id)sender{
	[BT_debugger showIt:self theMessage:@"startQuiz"];
    
	//prevent another answer selection
	[self disableButtons];	
	
	//holds the answer
	NSString *selAnswer = @"";	
	
	//the index of the answer clicked...
	int selIndex = (int)[sender tag];
	
	//get the text from the appropriate answer label
	if(selIndex == 1) selAnswer = [answerLabel1 text];
	if(selIndex == 2) selAnswer = [answerLabel2 text];
	if(selIndex == 3) selAnswer = [answerLabel3 text];
	if(selIndex == 4) selAnswer = [answerLabel4 text];
	
	//get the current question...
	BT_item *tmpQuestion = [quizQuestions objectAtIndex:currentQuestionIndex];
	NSString *tmpAnswer = [BT_strings getJsonPropertyValue:tmpQuestion.jsonVars nameOfProperty:@"correctAnswerText" defaultValue:@""];
	
	//increment the question
	currentQuestionIndex = (currentQuestionIndex + 1);
    
	//change button colors if "quizShowCorrectAnswers" 
	if(quizShowCorrectAnswers){
        
		//correct button color
		quizButtonColorCorrect = [BT_color getColorFromHexString:@"#336600"];
		if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizButtonColorCorrect" defaultValue:@""] length] > 3){
			quizButtonColorCorrect = [BT_color getColorFromHexString:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizButtonColorCorrect" defaultValue:@""]];
		}
		//incorrect button color
		quizButtonColorIncorrect = [BT_color getColorFromHexString:@"#990000"];
		if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizButtonColorIncorrect" defaultValue:@""] length] > 3){
			quizButtonColorIncorrect = [BT_color getColorFromHexString:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizButtonColorIncorrect" defaultValue:@""]];
		}		
        
		//color each button...
		if([answerLabel1.text isEqualToString:tmpAnswer]){
			[answerButton1 setBackgroundColor:quizButtonColorCorrect];
		}else{
			[answerButton1 setBackgroundColor:quizButtonColorIncorrect];
		}
		if([answerLabel2.text isEqualToString:tmpAnswer]){
			[answerButton2 setBackgroundColor:quizButtonColorCorrect];
		}else{
			[answerButton2 setBackgroundColor:quizButtonColorIncorrect];
		}
		if([answerLabel3.text isEqualToString:tmpAnswer]){
			[answerButton3 setBackgroundColor:quizButtonColorCorrect];
		}else{
			[answerButton3 setBackgroundColor:quizButtonColorIncorrect];
		}
		if([answerLabel4.text isEqualToString:tmpAnswer]){
			[answerButton4 setBackgroundColor:quizButtonColorCorrect];
		}else{
			[answerButton4 setBackgroundColor:quizButtonColorIncorrect];
		}
	}
    
	//correct or incorrect?
	if([tmpAnswer isEqualToString:selAnswer]){
        
		//answer was correct...
		
		//do we play sound effect?
		if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameCorrect" defaultValue:@""] length] > 3){
			[appDelegate playSoundEffect:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameCorrect" defaultValue:@""]];
		}
        
        //do we wobble?
        if ([[[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"wobbleOnCorrect" defaultValue:@"NO"]uppercaseString]isEqualToString:@"YES"]) [self wobbleImage];
        
		//flags / points...
		numberCorrect = (numberCorrect + 1);
		streak = (streak + 1);
		
		//determine how many points we earned for this answer
		int pointsPerRight = quizPointsPerAnswer;
		
		//assume we are not in bonus..
		UIImage *tmpBonusImg = [UIImage imageNamed:@"bt_blank.png"];
        
		//bonusImage for right "in a row"
		if(streak > 9){
			pointsPerRight = (pointsPerRight * 10);
			tmpBonusImg = [UIImage imageNamed:@"bt_quiz_10X.png"];
		}else if (streak > 8){
			pointsPerRight = (pointsPerRight * 9);
			tmpBonusImg = [UIImage imageNamed:@"bt_quiz_9X.png"];
		}else if (streak > 7){
			pointsPerRight = (pointsPerRight * 8);
			tmpBonusImg = [UIImage imageNamed:@"bt_quiz_8X.png"];
		}else if (streak > 6){
			pointsPerRight = (pointsPerRight * 7);
			tmpBonusImg = [UIImage imageNamed:@"bt_quiz_7X.png"];
		}else if (streak > 5){
			pointsPerRight = (pointsPerRight * 6);
			tmpBonusImg = [UIImage imageNamed:@"bt_quiz_6X.png"];
		}else if (streak > 4){
			pointsPerRight = (pointsPerRight * 5);
			tmpBonusImg = [UIImage imageNamed:@"bt_quiz_5X.png"];
		}else if (streak > 3){
			pointsPerRight = (pointsPerRight * 4);
			tmpBonusImg = [UIImage imageNamed:@"bt_quiz_4X.png"];
		}else if (streak > 2){
			pointsPerRight = (pointsPerRight * 3);
			tmpBonusImg = [UIImage imageNamed:@"bt_quiz_3X.png"];
		}
		
		//set the bonus image
		[self.bonusImageView setImage:tmpBonusImg];
		[bonusImageView setHidden:FALSE];
        
		//figure out totalPoints
		totalPoints = (totalPoints + pointsPerRight);
		
		//set points label
		[currentPointsLabel setText:[NSString stringWithFormat:@"%d", totalPoints]];
		
		//animate
		[self showCorrect];
		
		
	}else{
		
		//answer was incorrect
        
		//do we play sound effect?
		if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameIncorrect" defaultValue:@""] length] > 3){
            [appDelegate playSoundEffect:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameIncorrect" defaultValue:@""]];
		}
        
        //do we wobble?
        if ([[[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"wobbleOnIncorrect" defaultValue:@"NO"]uppercaseString]isEqualToString:@"YES"]) [self wobbleImage];
        
		//flags
		numberIncorrect = (numberIncorrect + 1);
		streak = 0;
        
		//set points label
		[currentPointsLabel setText:@""];
		
		//hide bonus image
		[bonusImageView setHidden:TRUE];
		
		//animate
		[self showIncorrect];
	}
    
}

//show correct
-(void)showCorrect{
	[BT_debugger showIt:self theMessage:@"showCorrect"];
	
	//change image
	[currentPointsImageView setImage:[UIImage imageNamed:@"bt_quiz_goldStar.png"]];
	
	//play possible sound effect...
	if([quizSoundEffectFileNameCorrect length] > 3){
		[appDelegate playSoundEffect:quizSoundEffectFileNameCorrect];
	}
	
	//figure out points...
	[self transitionPoints];
	
	//move to next question or end?
	if(currentQuestionIndex >= quizNumberOfQuestions){
        
		//end after delay
		[self setQuizDidEnd:1];
		[self performSelector:@selector(quizEnded) withObject:nil afterDelay:quizQuestionDelay];
        
	}else{
        
		//go to next question after delay
		[self performSelector:@selector(transitionQuestion) withObject:nil afterDelay:quizQuestionDelay];
        
	}
    
}

//show incorrect
-(void)showIncorrect{
	[BT_debugger showIt:self theMessage:@"showIncorrect"];
    
	//change image
	[currentPointsImageView setImage:[UIImage imageNamed:@"bt_quiz_wrong.png"]];
	
	//play possible sound effect...
	if([quizSoundEffectFileNameIncorrect length] > 3){
		[appDelegate playSoundEffect:quizSoundEffectFileNameIncorrect];
	}
	
	//figure out points...
	[self transitionPoints];
	
	//move to next question or end?
	if((currentQuestionIndex >= quizNumberOfQuestions) || (currentQuestionIndex + 1 > [self.quizQuestions count]) ){
        
		//end after delay
		[self setQuizDidEnd:1];
		[self performSelector:@selector(quizEnded) withObject:nil afterDelay:quizQuestionDelay];
        
	}else{
        
		//go to next question after delay
		[self performSelector:@selector(transitionQuestion) withObject:nil afterDelay:quizQuestionDelay];
        
        
	}
	
    
}

//quizEnded
-(void)quizEnded{
	[BT_debugger showIt:self theMessage:@"quizEnded"];
    
	//ignore this if the quiz never ended (we never started a quiz)
	if(self.quizDidEnd > 0){
        
		//clear question and timer label
		[questionText setText:@""];
		[quizTimeLabel setText:@""];
		
		//show the start button again
		[startButtonBox setAlpha:1.0];
		[startButton setHidden:FALSE];
        
		//show question image
		[questionImageView setAlpha:1.0];
		
		//hide answers..
		[answerButtonBox setHidden:TRUE];
		
		//flag quiz is not running
		[self setQuizRunning:FALSE];
		
		//setup action sheet
		NSString *elapsedTime = [BT_strings formatTimeFromSeconds:[NSString stringWithFormat:@"%d", totalSeconds]];
		NSString *quizResultMessage = [NSString stringWithFormat:@"%@: %d\n%@: %d\n%@: %d\n%@: %@", NSLocalizedString(@"quizCorrectAnswers", @"Correct Answers"), numberCorrect, NSLocalizedString(@"quizIncorrectAnswers", @"Incorrect Answers"), numberIncorrect, NSLocalizedString(@"quizTotalScore", @"Total Score"), totalPoints, NSLocalizedString(@"quizElapsedTime", @"Elapsed Time"), elapsedTime];  
		
		//did user earn a reward?
		BOOL earnedReward = FALSE;
		if(totalPoints > quizRewardIfPointsOver){
			earnedReward = TRUE;
		}
		
		//do we play finished sound effect? Play reward sound if we have one, or the non-reward sound.
		if(earnedReward && self.didFinishOnce == 0){
			if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameReward" defaultValue:@""] length] > 3){
				[appDelegate playSoundEffect:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameReward" defaultValue:@""]];
			}else{
				if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameFinished" defaultValue:@""] length] > 3){
					[appDelegate playSoundEffect:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameFinished" defaultValue:@""]];
				}
			}
		}else{
			if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameFinished" defaultValue:@""] length] > 3){
				[appDelegate playSoundEffect:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizSoundEffectFileNameFinished" defaultValue:@""]];
			}
		}
		
		//flag the didFinishOnce so sound effect doens't play again if "back" pressed
		[self setDidFinishOnce:1];
		
		//do we have a reward screen setup?
		BT_item *rewardScreenObjectToLoad = nil;
		BOOL haveRewardScreen = FALSE;
		NSString *quizRewardScreenItemId = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizRewardScreenItemId" defaultValue:@""];
		NSString *quizRewardScreenNickname = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizRewardScreenNickname" defaultValue:@""];
		if([quizRewardScreenItemId length] > 1){
			rewardScreenObjectToLoad = [appDelegate.rootApp getScreenDataByItemId:quizRewardScreenItemId];
		}else{
			if([quizRewardScreenNickname length] > 1){
				rewardScreenObjectToLoad = [appDelegate.rootApp getScreenDataByNickname:quizRewardScreenNickname];
			}else{
				if([self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"]){
					rewardScreenObjectToLoad = [[BT_item alloc] init];
					[rewardScreenObjectToLoad setItemId:[[self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"] objectForKey:@"itemId"]];
					[rewardScreenObjectToLoad setItemNickname:[[self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"] objectForKey:@"itemNickname"]];
					[rewardScreenObjectToLoad setItemType:[[self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"] objectForKey:@"itemType"]];
					[rewardScreenObjectToLoad setJsonVars:[self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"]];
				}								
			}
		}	
		if(rewardScreenObjectToLoad != nil){
			haveRewardScreen = TRUE;
		}
		
		//do we have a finished screen setup?
		BT_item *finishScreenObjectToLoad = nil;
		BOOL haveFinishScreen = FALSE;
		NSString *quizFinishScreenItemId = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizFinishScreenItemId" defaultValue:@""];
		NSString *quizFinishScreenNickname = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizFinishScreenNickname" defaultValue:@""];
		if([quizFinishScreenItemId length] > 1){
			finishScreenObjectToLoad = [appDelegate.rootApp getScreenDataByItemId:quizFinishScreenItemId];
		}else{
			if([quizFinishScreenNickname length] > 1){
				finishScreenObjectToLoad = [appDelegate.rootApp getScreenDataByNickname:quizFinishScreenNickname];
			}else{
				if([self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"]){
					finishScreenObjectToLoad = [[BT_item alloc] init];
					[finishScreenObjectToLoad setItemId:[[self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"] objectForKey:@"itemId"]];
					[finishScreenObjectToLoad setItemNickname:[[self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"] objectForKey:@"itemNickname"]];
					[finishScreenObjectToLoad setItemType:[[self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"] objectForKey:@"itemType"]];
					[finishScreenObjectToLoad setJsonVars:[self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"]];
				}								
			}
		}	
		if(finishScreenObjectToLoad != nil){
			haveFinishScreen = TRUE;
		}	
		
		//buttons for action sheet
		finishedButtons = [[NSMutableArray alloc] init];
		
		//earned reward screen
		if(haveRewardScreen && earnedReward){
			[finishedButtons addObject:NSLocalizedString(@"quizShowReward", @"Show Reward")];
		}else{
			//regular finish screen
			if(haveFinishScreen){
				[finishedButtons addObject:NSLocalizedString(@"continue", @"Continue")];
			}
		}
        
 		
		//try again button...
		[finishedButtons addObject:NSLocalizedString(@"quizTryAgain", @"Try Again")];
		
		//action sheet
		UIActionSheet *actionSheet = [[UIActionSheet alloc] initWithTitle:quizResultMessage
                                                                 delegate:self
                                                        cancelButtonTitle:nil
                                                   destructiveButtonTitle:nil
                                                        otherButtonTitles:nil];
		//add the buttons
		for (int i = 0; i < [finishedButtons count]; i++) {
			[actionSheet addButtonWithTitle:[finishedButtons objectAtIndex:i]];
		}
		
		//add cancel button
		[actionSheet addButtonWithTitle:NSLocalizedString(@"quit", @"Quit")];
		actionSheet.destructiveButtonIndex = [finishedButtons count];
		[actionSheet setActionSheetStyle:UIActionSheetStyleBlackOpaque];
        
		//is this a tabbed app?
		if([appDelegate.rootApp rootTabBarController] != nil){
			[actionSheet showFromTabBar:[appDelegate.rootApp.rootTabBarController tabBar]];
		}else{
			[actionSheet showFromToolbar:self.quizToolbar];
		}
		
        
        
        /////////////////////////////////////////////////////////////////////////////////////////////////////////
        //SEND QUIZ RESULTS IN BACKGROUND
        [NSThread detachNewThreadSelector: @selector(sendQuizResultsToURL) toTarget:self withObject:nil];

 		
        
	}//self.quizDidEnd == 0
	
}

//disable, enable buttons
-(void)disableButtons{
	[answerButton1 setEnabled:FALSE];
	[answerButton2 setEnabled:FALSE];
	[answerButton3 setEnabled:FALSE];
	[answerButton4 setEnabled:FALSE];
}
-(void)enableButtons{
	[answerButton1 setEnabled:TRUE];
	[answerButton2 setEnabled:TRUE];
	[answerButton3 setEnabled:TRUE];
	[answerButton4 setEnabled:TRUE];
}


//send quiz results to URL...
-(void)sendQuizResultsToURL{
	
    //download image using Grand Central Dispath, iOS handles queue automagically...
    dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0ul);
    dispatch_async(queue, ^{

        //must have sendResultsToURL value...
        if([[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"sendResultsToURL" defaultValue:@""] length] > 3){
            
            //the sendResultsToURL data URL
            NSMutableString *sendResultsToURL = [[NSMutableString alloc] initWithString:@""];
            [sendResultsToURL appendString:[BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"sendResultsToURL" defaultValue:@""]];
            
            //append quiz results to the scoreboard URL (for remote backends, helps developers track score results)
            [sendResultsToURL appendString:[NSString stringWithFormat:@"&totalPoints=%i", [self totalPoints]]];
            [sendResultsToURL appendString:[NSString stringWithFormat:@"&totalSeconds=%d", [self totalSeconds]]];
            [sendResultsToURL appendString:[NSString stringWithFormat:@"&numberQuestions=%i", [self quizNumberOfQuestions]]];
            [sendResultsToURL appendString:[NSString stringWithFormat:@"&numberCorrect=%i", [self numberCorrect]]];
            [sendResultsToURL appendString:[NSString stringWithFormat:@"&numberIncorrect=%i", [self numberIncorrect]]];
            
            //merge fields in URL
            NSString *useURL = [BT_strings mergeBTVariablesInString:sendResultsToURL];
            [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"sendQuizResultsToURL:%@", sendResultsToURL]];
            
            
            //clean-up URL, encode as UTF8
            NSURL *escapedURL = [NSURL URLWithString:[useURL stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
            
            //make the http request
            NSMutableURLRequest  *theRequest = [NSMutableURLRequest requestWithURL:escapedURL cachePolicy:NSURLRequestReloadIgnoringLocalCacheData timeoutInterval:10.0];
            [theRequest setHTTPMethod:@"GET"];
            NSURLConnection *theConnection;
            if((theConnection = [[NSURLConnection alloc] initWithRequest:theRequest delegate:self])){
                //ingore returned data...
            }else{
                [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"sendQuizResultsToURL: ERROR. Could not send results to %@", sendResultsToURL]];
            }
            
        }else{
            [BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"sendQuizResultsToURL This quiz does not have a send results to URL setup. Not sending results anywhere.%@", @""]];
        }
        
        
        dispatch_async(dispatch_get_main_queue(), ^{
            
            //done...
       
        
        
        });
    });
        
    
    
}

//show quiz reward screen
-(void)showQuizRewardScreen{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"showQuizRewardScreen %@", @""]];
	
	//get possible itemId of the screen to load
	NSString *loadScreenItemId = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizRewardScreenItemId" defaultValue:@""];
	
	//get possible nickname of the screen to load
	NSString *loadScreenNickname = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizRewardScreenNickname" defaultValue:@""];
    
	//bail if load screen = "none"
	if([loadScreenItemId isEqualToString:@"none"]){
		return;
	}
	
	//check for loadScreenWithItemId THEN loadScreenWithNickname THEN loadScreenObject
	BT_item *screenObjectToLoad = nil;
	if([loadScreenItemId length] > 1){
		screenObjectToLoad = [appDelegate.rootApp getScreenDataByItemId:loadScreenItemId];
	}else{
		if([loadScreenNickname length] > 1){
			screenObjectToLoad = [appDelegate.rootApp getScreenDataByNickname:loadScreenNickname];
		}else{
			if([self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"]){
				screenObjectToLoad = [[BT_item alloc] init];
				[screenObjectToLoad setItemId:[[self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"] objectForKey:@"itemId"]];
				[screenObjectToLoad setItemNickname:[[self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"] objectForKey:@"itemNickname"]];
				[screenObjectToLoad setItemType:[[self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"] objectForKey:@"itemType"]];
				[screenObjectToLoad setJsonVars:[self.screenData.jsonVars objectForKey:@"quizRewardScreenObject"]];
			}								
		}
	}
	
	//load next screen if it's not nil
	if(screenObjectToLoad != nil){
        
		//build a temp menu-item to pass to screen load method. We need this because the transition type is in the menu-item
		BT_item *tmpMenuItem = [[BT_item alloc] init];
        
		//build an NSDictionary of values for the jsonVars property
		NSMutableDictionary *tmpDictionary = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                       @"unused", @"itemId", 
                                       [self.screenData.jsonVars objectForKey:@"quizRewardScreenTransitionType"], @"transitionType",
                                       nil];	
		[tmpMenuItem setJsonVars:tmpDictionary];
		[tmpMenuItem setItemId:@"0"];
        
		//load the next screen
        [self handleTapToLoadScreen:screenObjectToLoad theMenuItemData:tmpMenuItem];
		
	}else{
		//show debug
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"menuTapError",@"The application doesn't know how to handle this action?")]];
	}	
    
}



//show finish screen
-(void)showFinishScreen{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"showFinishScreen %@", @""]];
    
	//get possible itemId of the screen to load
	NSString *loadScreenItemId = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizFinishScreenItemId" defaultValue:@""];
	
	//get possible nickname of the screen to load
	NSString *loadScreenNickname = [BT_strings getJsonPropertyValue:self.screenData.jsonVars nameOfProperty:@"quizFinishScreenNickname" defaultValue:@""];
    
	//bail if load screen = "none"
	if([loadScreenItemId isEqualToString:@"none"]){
		return;
	}
	
	//check for loadScreenWithItemId THEN loadScreenWithNickname THEN loadScreenObject
	BT_item *screenObjectToLoad = nil;
	if([loadScreenItemId length] > 1){
		screenObjectToLoad = [appDelegate.rootApp getScreenDataByItemId:loadScreenItemId];
	}else{
		if([loadScreenNickname length] > 1){
			screenObjectToLoad = [appDelegate.rootApp getScreenDataByNickname:loadScreenNickname];
		}else{
			if([self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"]){
				screenObjectToLoad = [[BT_item alloc] init];
				[screenObjectToLoad setItemId:[[self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"] objectForKey:@"itemId"]];
				[screenObjectToLoad setItemNickname:[[self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"] objectForKey:@"itemNickname"]];
				[screenObjectToLoad setItemType:[[self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"] objectForKey:@"itemType"]];
				[screenObjectToLoad setJsonVars:[self.screenData.jsonVars objectForKey:@"quizFinishScreenObject"]];
			}								
		}
	}
	
	//load next screen if it's not nil
	if(screenObjectToLoad != nil){
        
		//build a temp menu-item to pass to screen load method. We need this because the transition type is in the menu-item
		BT_item *tmpMenuItem = [[BT_item alloc] init];
        
		//build an NSDictionary of values for the jsonVars property
		NSMutableDictionary *tmpDictionary = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                       @"unused", @"itemId", 
                                       [self.screenData.jsonVars objectForKey:@"quizFinishScreenTransitionType"], @"transitionType",
                                       nil];	
		[tmpMenuItem setJsonVars:tmpDictionary];
		[tmpMenuItem setItemId:@"0"];
        
		//load the next screen
        [self handleTapToLoadScreen:screenObjectToLoad theMenuItemData:tmpMenuItem];

		
	}else{
		//show debug
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"%@",NSLocalizedString(@"menuTapError",@"The application doesn't know how to handle this action?")]];
	}	
    
}


//////////////////////////////////////////////////////////////////////////////////////////////////
//delegate method for action sheet clicks
-(void)actionSheet:(UIActionSheet *)actionSheet  clickedButtonAtIndex:(NSInteger)buttonIndex {
    
	NSString *buttonTitle = [actionSheet buttonTitleAtIndex:buttonIndex];
    
	//quit
	if([buttonTitle isEqual:NSLocalizedString(@"quit", @"Quit")]){	
		[self navLeftTap];
	}		
    
	//show reward
	if([buttonTitle isEqual:NSLocalizedString(@"quizShowReward", @"Show Reward")]){	
		[self showQuizRewardScreen];
	}		
    
	//show continue
	if([buttonTitle isEqual:NSLocalizedString(@"continue", @"Continue")]){	
		[self showFinishScreen];
	}	
	
	//try again
	if([buttonTitle isEqual:NSLocalizedString(@"quizTryAgain", @"Try Again")]){	
		[self startQuiz];
	}
	
    
}


//sets the question images
-(void)setQuestionImage:(UIImage *)theImage{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"setQuestionImage: %@", @""]];
	[self.spinner stopAnimating];
	
	//do we round the corners?
	if(quizImageCornerRadius > 0){
        self.questionImageView.layer.cornerRadius = 5.0;
        self.questionImageView.layer.masksToBounds = YES;
	}
	[self.questionImageView setImage:theImage];
	[self.questionImageView setAlpha:1.0];
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//BT_Quiz_PhotoDelegate delegate methods

//photo finished loading..
-(void)photoDidFinishLoading:(BT_quiz_photo *)photo{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"photoDidFinishLoading: %@", @""]];
	[self setQuestionImage:[photo image]];
}

//photo failed to load
-(void)photoDidFailToLoad:(BT_quiz_photo *)photo {
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"photoDidFailToLoad: %@", @""]];
	
	if([appDelegate.rootDevice isIPad]){
		[self setQuestionImage:[UIImage imageNamed:@"bt_quiz_bg_large.png"]];
	}else{
		[self setQuestionImage:[UIImage imageNamed:@"bt_quiz_bg_small.png"]];
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////
//downloader delegate methods
-(void)downloadFileStarted:(NSString *)message{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloadFileStarted: %@", message]];
}
-(void)downloadFileInProgress:(NSString *)message{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloadFileInProgress: %@", message]];
	if(self.progressView != nil){
		UILabel *tmpLabel = (UILabel *)[self.progressView.subviews objectAtIndex:2];
		[tmpLabel setText:message];
	}
}
-(void)downloadFileCompleted:(NSString *)message{
	[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"downloadFileCompleted: %@", message]];
	[self hideProgress];
    
	//if message contains "error", look for previously cached data...
	if([message rangeOfString:@"ERROR-1968" options:NSCaseInsensitiveSearch].location != NSNotFound){
        
		//show alert
		[self showAlert:nil theMessage:NSLocalizedString(@"downloadError", @"There was a problem downloading some data. Check your internet connection then try again.") alertTag:0];
        
		[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"download error: There was a problem downloading data from the internet.%@", @""]];
		if([BT_fileManager doesLocalFileExist:[self saveAsFileName]]){
            
			//use stale data if we have it
			NSString *staleData = [BT_fileManager readTextFileFromCacheWithEncoding:self.saveAsFileName encodingFlag:-1];
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"building screen from stale configuration data saved at: %@", [self saveAsFileName]]];
			[self parseScreenData:staleData];
			
		}else{
			
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"There is no local data available for this screen?%@", @""]];
			[self layoutScreen];
		}
        
	}else{
        
		//parse previously saved data
		if([BT_fileManager doesLocalFileExist:[self saveAsFileName]]){
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"parsing downloaded screen data.%@", @""]];
			NSString *downloadedData = [BT_fileManager readTextFileFromCacheWithEncoding:[self saveAsFileName] encodingFlag:-1];
			[self parseScreenData:downloadedData];
            
		}else{
			[BT_debugger showIt:self theMessage:[NSString stringWithFormat:@"Error caching downloaded file: %@", [self saveAsFileName]]];
			[self layoutScreen];
            
			//show alert
			[self showAlert:nil theMessage:NSLocalizedString(@"appDownloadError", @"There was a problem saving some data downloaded from the internet.")alertTag:0];
            
		}	
		
	}
	
}



@end




