/*
 *	Copyright Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */

#import "BT_settingData.h"
#import "BT_screen_settingsDevice.h"


@implementation BT_screen_settingsDevice
@synthesize menuItems, myTableView, currentUserSetting;

//viewDidLoad
-(void)viewDidLoad{
	[BT_debugger showIt:self theMessage:@"viewDidLoad"];
	[super viewDidLoad];
		
	//build the table to display the list of menu items
	self.myTableView = [BT_viewUtilities getTableViewForScreen:[self screenData]];
	[self.myTableView setDataSource:self];
	[self.myTableView setDelegate:self];
	[self.view addSubview:myTableView];		
		
	//menu items are hard-coded this screen
	self.menuItems = [[NSMutableArray alloc] init];
						
}

//view will appear
-(void)viewWillAppear:(BOOL)animated{
	[super viewWillAppear:animated];
	[BT_debugger showIt:self theMessage:@"viewWillAppear"];
	
	//load table data
	[self performSelector:(@selector(loadTable)) withObject:nil afterDelay:0.1];

}


//clear cache
-(void)clearCache{
	[BT_debugger showIt:self theMessage:@"clearCache"];
	
	//remove all saved data
	[BT_fileManager deleteAllLocalData];
	
	//show message
	UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:nil message:NSLocalizedString(@"cacheCleared", "All cached-data has been removed from this device.") delegate:self
	cancelButtonTitle:NSLocalizedString(@"ok", "OK") otherButtonTitles:nil];
	[alertView show];
	
	//reload table after a moment
	[self performSelector:(@selector(loadTable)) withObject:nil afterDelay:0.5];
	
}

//reload table
-(void)loadTable{
	
	//menu items are hard-coded this screen
	self.menuItems = [[NSMutableArray alloc] init];
	
	//info from UIDevice 
	BT_settingData *tmpData;
	NSString *tmp;
	
	//model info localized
	UIDevice* device = [UIDevice currentDevice];
	tmp = [NSString stringWithFormat:@"%@: %@ %@", [device localizedModel], [device systemName], [device systemVersion]];
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmp];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];

	//camera support
	NSString *tmpCamera = NSLocalizedString(@"cameraSupportNo", "Still Camera: NO");
	if([appDelegate.rootDevice canTakePictures]){
		tmpCamera = NSLocalizedString(@"cameraSupportYes", "Still Camera: YES");
	}
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmpCamera];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];
	
	//video camera support
	NSString *tmpVideo = NSLocalizedString(@"videoCameraSupportNo", "Video Camera: NO");
	if([appDelegate.rootDevice canTakeVideos]){
   		tmpVideo = NSLocalizedString(@"videoCameraSupportYes", "Video Camera: YES");
	}	
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmpVideo];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];

	//resolution
	tmp = [NSString stringWithFormat:@"%@ %i x %i", NSLocalizedString(@"display", "Display:"), [appDelegate.rootDevice deviceWidth], [appDelegate.rootDevice deviceHeight]];
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmp];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];

	//can make phone calls?
	NSString *tmpCalls = NSLocalizedString(@"phoneCallSupportNo", "Can Make Calls: NO");
	if([appDelegate.rootDevice canMakePhoneCalls]){
		tmpCalls = NSLocalizedString(@"phoneCallSupportYes", "Can Make Calls: YES");
	}
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmpCalls];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];
	
	//can send emails?
	NSString *tmpEmails = NSLocalizedString(@"emailSupportNo", "Can Send Email: NO");
	if([appDelegate.rootDevice canSendEmails]){
		tmpEmails = NSLocalizedString(@"emailSupportYes", "Can Send Email: YES");
	}
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmpEmails];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];

	//can send sms?
	NSString *tmpSMS = NSLocalizedString(@"smsSupportNo", "Can Send SMS: NO");
	if([appDelegate.rootDevice canSendSMS]){
		tmpSMS = NSLocalizedString(@"smsSupportYes", "Can Send SMS: YES");
	}
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmpSMS];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];
	
	//local data cache
	int tmpSize = [BT_fileManager getLocalDataSizeInt];
	tmp = [NSString stringWithFormat:@"%@ %@ (%i files)", NSLocalizedString(@"cache",@"Cache:"), [BT_fileManager stringFromFileSize:tmpSize], [BT_fileManager countLocalFiles]];
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmp];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];

	//last row is "clear cache" row
	tmp = [NSString stringWithFormat:@"%@", NSLocalizedString(@"clearCache",@"Clear Cache")];
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"notUsedOnThisScreen"];
	[tmpData setSettingLabel:tmp];
	[tmpData setSettingValue:@""];
	[self.menuItems addObject:tmpData];

	//reload table
	[self.myTableView reloadData];
	
	
}



//////////////////////////////////////////////////////////////
//UITableView delegate methods
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

// number of rows
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.menuItems count];
}

//table view cells
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {

	static NSString *CellIdentifier = @"Cell";
	UITableViewCell *cell = (UITableViewCell *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil) {
	
		//init our custom cell
		cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
		cell.accessoryType = UITableViewCellAccessoryNone;
		
	}
	
	//this menu item
	BT_settingData *thisSettingItemData = [self.menuItems objectAtIndex:indexPath.row];
	[cell.textLabel setText:[thisSettingItemData settingLabel]];
    
	
	return cell;	
	
}

//on row select
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	[BT_debugger showIt:self theMessage:@"didSelectRowAtIndexPath"];
	
	//clear cache if this is the last row...
	if(indexPath.row == ([self.menuItems count] - 1)){
		[self clearCache];
	}
	
}



@end





