/*
 *	Copyright Buzztouch
 *
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification, are 
 *	permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice which includes the
 *	name(s) of the copyright holders. It must also retain this list of conditions and the 
 *	following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, this list 
 *	of conditions and the following disclaimer in the documentation and/or other materials 
 *	provided with the distribution. 
 *
 *	The name Buzztouch (or buzztouch) nor the names of its contributors
 *	may be used to endorse or promote products derived from this software without specific 
 *	prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 *	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
 *	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *	OF SUCH DAMAGE. 
 */

#import "BT_settingData.h"
#import "BT_screen_settingsLocation.h"


@implementation BT_screen_settingsLocation
@synthesize menuItems, myTableView, currentUserSetting, lastIndexPath;

//viewDidLoad
-(void)viewDidLoad{
	[BT_debugger showIt:self theMessage:@"viewDidLoad"];
	[super viewDidLoad];

	////////////////////////////////////////////////////////////////////////////////////////
	//UITableView for menu...
	self.myTableView = [BT_viewUtilities getTableViewForScreen:[self screenData]];
	[self.myTableView setDataSource:self];
	[self.myTableView setDelegate:self];
	[self.view addSubview:myTableView];		
	
	//menu items are hard-coded this screen
	self.menuItems = [[NSMutableArray alloc] init];
	
    //current setting...
    [self setCurrentUserSetting:[BT_strings getPrefString:@"userAllowLocation"]];
    if([[self currentUserSetting] length] < 1){
        [self setCurrentUserSetting:@"allowLocation"];
    }

    //items
	BT_settingData *tmpData;
	
	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"allowLocation"];
	[tmpData setSettingLabel:@"Prevent Location Reports"];
	[tmpData setSettingValue:@"prevent"];
	[self.menuItems addObject:tmpData];

	tmpData = [[BT_settingData alloc] init];
	[tmpData setSettingName:@"allowLocation"];
	[tmpData setSettingLabel:@"Allow Location Reports"];
	[tmpData setSettingValue:@"allow"];
	[self.menuItems addObject:tmpData];

}

//view will appear
-(void)viewWillAppear:(BOOL)animated{
	[super viewWillAppear:animated];
	[BT_debugger showIt:self theMessage:@"viewWillAppear"];
	
    
}

//////////////////////////////////////////////////////////////
//UITableView delegate methods
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

// number of rows
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.menuItems count];
}

//table view cells
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {

	static NSString *CellIdentifier = @"Cell";
	UITableViewCell *cell = (UITableViewCell *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil) {
	
		//init cell...
		cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
		cell.accessoryType = UITableViewCellAccessoryNone;
		
	}
	
	//this menu item
	BT_settingData *thisSettingItemData = [self.menuItems objectAtIndex:indexPath.row];
	
	//is this current setting?
	if([[thisSettingItemData settingValue] isEqualToString:currentUserSetting]){
		cell.accessoryType = UITableViewCellAccessoryCheckmark;
		[self setLastIndexPath:indexPath];
	}else{
		cell.accessoryType = UITableViewCellAccessoryNone;
	}
	
    //set the text...
    [cell.textLabel setText:[thisSettingItemData settingLabel]];
	
	return cell;	
	
}

//on row select
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	[BT_debugger showIt:self theMessage:@"didSelectRowAtIndexPath"];
	
	//selected setting object
	BT_settingData *thisSettingData = [self.menuItems objectAtIndex:indexPath.row];

	//remember...
	[BT_strings setPrefString:@"userAllowLocation" valueOfPref:[thisSettingData settingValue]];
	
	//unselect previous selection, select this selection	
	int newRow = (int)[indexPath row];
	int oldRow = (int)[lastIndexPath row];
	if (newRow != oldRow){
			UITableViewCell *newCell = [tableView cellForRowAtIndexPath:indexPath];
			newCell.accessoryType = UITableViewCellAccessoryCheckmark;

			UITableViewCell *oldCell = [tableView cellForRowAtIndexPath:lastIndexPath];
			oldCell.accessoryType = UITableViewCellAccessoryNone;

			lastIndexPath = indexPath;
	}
	[tableView deselectRowAtIndexPath:indexPath animated:YES];	

	
}




@end





